      subroutine latitude_grid(lat_method,Ns_theta,Ntheta,theta)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to compute the latitude grid
c
c     Inputs:
c       + lat_method:
c         - lat_method=1: use a constant angle step
c         - lat_method=2: use a constant cosine step to compute angles
c       + Ns_theta: number of angular sectors over each [0;pi/2] latitudinal interval
c
c     Outputs:
c       + Ntheta: number of angular sectors over the total [-pi/2;pi/2] latitudinal interval
c       + theta: limits of angular sectors in radians (Ntheta+1 values, from -pi/2 to pi/2)
c
      integer strlen
      integer lat_method,Ns_theta,Ntheta,i
      double precision theta(0:Ntheta_mx)
      double precision dtheta,dmu
      double precision mu(0:Ntheta_mx)
      character*100 label
      label='latitude_grid'

      theta(0)=-pi/2.0D+0

      if (lat_method.eq.1) then
         dtheta=pi/2.0D+0/Ns_theta
         do i=1,Ns_theta-1
            theta(i)=theta(i-1)+dtheta
         enddo
         theta(Ns_theta)=0.0D+0
         do i=Ns_theta+1,2*Ns_theta-1
            theta(i)=theta(i-1)+dtheta
         enddo
      else if (lat_method.eq.2) then
         mu(0)=0.0D+0
         dmu=1.0D+0/Ns_theta
         do i=1,Ns_theta-1
            mu(i)=mu(i-1)+dmu
         enddo
         mu(Ns_theta)=1.0D+0
         do i=1,Ns_theta-1
            theta(i)=-dacos(mu(i))
         enddo
         theta(Ns_theta)=0.0D+0
         do i=Ns_theta+1,2*Ns_theta-1
            theta(i)=-dacos(mu(2*Ns_theta-i))
         enddo
      else
         write(*,*) 'Error from routine ',label(1:strlen(label)),':'
         write(*,*) 'lat_method=',lat_method
      endif
      
      theta(2*Ns_theta)=pi/2.0D+0
      Ntheta=2*Ns_theta

c     Debug
c      do i=0,Ntheta
c         write(*,*) 'theta(',i,')=',theta(i)
c      enddo
c     Debug

      return
      end
