      program make_cloud_data
      implicit none
c
c     Purpose: to generate the clouds input data file for program "planet_EMC" (file "clouds.in")
c     See the end of this file (section "production of the grid file") for details on the
c     format used.
c
c     Values have to be given for the following variables:
c       + Nclouds: the number of clouds defined in the file
c       + Nb_clouds: the number of narrowband spectral intervals clouds spectral data is given for
c       + Na: the number of angles phase functions are given for
c       + lambda_clouds_lo(band): lower limit (in micrometers) for every clouds narrowband
c       + lambda_clouds_hi(band): higher limit (in micrometers) for every clouds narrowband
c       + clouds_coord(cloud,i): spatial coorinates for each cloud:
c         - clouds_coord(cloud,1): minimum radius (in kilometers) the cloud is defined for
c         - clouds_coord(cloud,2): maximum radius (in kilometers) the cloud is defined for
c         - clouds_coord(cloud,3): minimum latitude (in radians) the cloud is defined for
c         - clouds_coord(cloud,4): maximum latitude (in radians) the cloud is defined for
c         - clouds_coord(cloud,5): minimum longitude (in radians) the cloud is defined for
c         - clouds_coord(cloud,6): maximum longitude (in radians) the cloud is defined for
c       + clouds_ka(cloud,band): cloud absorption coefficient (in inverse meters), that must
c         be provided for every clouds narrowband spectral interval.
c       + clouds_ks(cloud,band): cloud scattering coefficient (in inverse meters), that must
c         be provided for every clouds narrowband spectral interval.
c       + clouds_phase(cloud,band,angle,i): cloud phase function, that is defined for 'Na' angles
c         The first angle must be pi (backward direction), and the last angle must be 0.0 (forward direction)
c         - clouds_phase(cloud,band,angle,1) is the cosine of the angle; therefore, clouds_phase(cloud,band,1,1)
c           must be equal to -1 and clouds_phase(cloud,band,Na,1) must be equal to 1.
c         - clouds_phase(cloud,band,angle,2) is the value of the phase function for that angle,
c           in narrowband index 'band'.
c     
c
      include 'max.inc'
      include 'formats.inc'
      include 'param.inc'
c     variables definition
      integer cloud,Nclouds,Na
      integer band,Nb_clouds,i,angle
      double precision lambda_clouds_lo(1:Nb_clouds_mx)
      double precision lambda_clouds_hi(1:Nb_clouds_mx)
      double precision clouds_coord(1:Nclouds_mx,1:6)
      double precision clouds_ka(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_ks(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_phase(1:Nclouds_mx,1:Nb_clouds_mx,
     &     1:Nangle_mx,1:2)
      double precision clouds_g(1:Nclouds_mx,1:Nb_clouds_mx)
      integer strlen
      character*100 label,clouds_file
      label='make_cloud_data'
      clouds_file='./clouds.in'

c     -------------------------------------------------------------------------------------------------------------
c     Data definition
c     -------------------------------------------------------------------------------------------------------------

      write(*,*) 'gathering data'

c     Nclouds is the total number of clouds in the planetary atmosphere
      Nclouds=4
c     clouds_coord(cloud,*) are the (r,theta,phi) spherical limits
c     of each cloud index 'cloud'
      clouds_coord(1,1)=6378.237D+0 ! minimum radius (km)
      clouds_coord(1,2)=6379.0D+0 ! maximum radius (km)
      clouds_coord(1,3)=0.21D+0 ! minimum latitude (rad)
      clouds_coord(1,4)=0.34D+0 ! maximum latitude (rad)
      clouds_coord(1,5)=0.31D+0 ! minimum longitude (rad)
      clouds_coord(1,6)=0.44D+0 ! maximum longitude (rad)

      clouds_coord(2,1)=6379.10D+0 ! minimum radius (km)
      clouds_coord(2,2)=6379.11D+0 ! maximum radius (km)
      clouds_coord(2,3)=0.20D+0 ! minimum latitude (rad)
      clouds_coord(2,4)=clouds_coord(2,3)+pi/2.0D+1 ! maximum latitude (rad)
      clouds_coord(2,5)=0.30 ! minimum longitude (rad)
      clouds_coord(2,6)=clouds_coord(2,5)+pi/2.0D+1 ! maximum longitude (rad)

      clouds_coord(3,1)=6378.20D+0 ! minimum radius (km)
      clouds_coord(3,2)=6378.40D+0 ! maximum radius (km)
      clouds_coord(3,3)=0.25D+0 ! minimum latitude (rad)
      clouds_coord(3,4)=0.30D+0 ! maximum latitude (rad)
      clouds_coord(3,5)=0.35D+0 ! minimum longitude (rad)
      clouds_coord(3,6)=0.40D+0 ! maximum longitude (rad)

      clouds_coord(4,1)=6377.1D+0 ! minimum radius (km)
      clouds_coord(4,2)=6389.1D+0 ! maximum radius (km)
      clouds_coord(4,3)=0.10D+0 ! minimum latitude (rad)
      clouds_coord(4,4)=0.40D+0 ! maximum latitude (rad)
      clouds_coord(4,5)=0.40D+0 ! minimum longitude (rad)
      clouds_coord(4,6)=0.60D+0 ! maximum longitude (rad)

c     Nb_clouds is the number of spectral narrowband intervals for clouds data definition
      Nb_clouds=10
      lambda_clouds_lo(1)=1.0D+0 ! m
      do band=2,Nb_clouds
         lambda_clouds_lo(band)=lambda_clouds_lo(band-1)+1.0D+0 ! m
         lambda_clouds_hi(band-1)=lambda_clouds_lo(band)! m
      enddo
      lambda_clouds_hi(Nb_clouds)=lambda_clouds_lo(Nb_clouds)+1.0D+0 ! m

c     clouds_ka(cloud,band) is the absorption coefficient (in inverse meters)
c     of cloud index 'cloud', in narrowband index 'band'
      do cloud=1,Nclouds
         do band=1,Nb_clouds
            clouds_ka(cloud,band)=1.0D-1 ! m
         enddo
      enddo
c     clouds_ks(cloud,band) is the scattering coefficient (in inverse meters)
c     of cloud index 'cloud', in narrowband index 'band'
      do cloud=1,Nclouds
         do band=1,Nb_clouds
            clouds_ks(cloud,band)=4.0D-3 ! m
         enddo
      enddo
c     g(cloud,band) is the asymetry factor of cloud index 'cloud', in narrowband index 'band'
      do cloud=1,Nclouds
         do band=1,Nb_clouds
            clouds_g(cloud,band)=0.5D+0
         enddo
      enddo
c     Na is the number of angles the phase function is discretized into
      Na=11
c     phase(cloud,band,angle,1) is the value of the cos(theta) (-1,1)
c     phase(cloud,band,angle,2) is the value of the phase function
      do cloud=1,Nclouds
         do band=1,Nb_clouds
            do angle=1,Na
               clouds_phase(cloud,band,angle,1)=-1.0D+0+2.0D+0
     &              /(Na-1)*(angle-1)
c               clouds_phase(cloud,band,angle,2)=0.0D+0
               clouds_phase(cloud,band,angle,2)=angle*band
            enddo ! angle
         enddo ! band
      enddo ! cloud

c     -------------------------------------------------------------------------------------------------------------
c     Production of the clouds data file
c     -------------------------------------------------------------------------------------------------------------

      write(*,*) 'now writing data file'

      open(10,file='./clouds.in')
c     Number of clouds, number of narrowband spectral intervals for clouds data,
c     number of angles phase functions are provided for
      write(10,*) Nclouds,Nb_clouds,Na
c     lower limit of each cloud narrowband (micrometers)
      write(10,50) (lambda_clouds_lo(band),band=1,Nb_clouds)
c     higher limit of each cloud narrowband (micrometers)
      write(10,50) (lambda_clouds_hi(band),band=1,Nb_clouds)
      do cloud=1,Nclouds
c     clouds coordinates (radius,latitude,longitude)
         write(10,50) (clouds_coord(cloud,i),i=1,6)
      enddo ! cloud
      do cloud=1,Nclouds
c     clouds absorption coefficients (inverse meters)
         write(10,50) (clouds_ka(cloud,band),band=1,Nb_clouds)
      enddo ! cloud
      do cloud=1,Nclouds
c     clouds scattering coefficients (inverse meters)
         write(10,50) (clouds_ks(cloud,band),band=1,Nb_clouds)
      enddo ! cloud
c     clouds asymetry parameters
      do cloud=1,Nclouds
         write(10,50) (clouds_g(cloud,band),band=1,Nb_clouds)
      enddo
      do cloud=1,Nclouds
         do angle=1,Na
c     cos(alpha) for each angle 'alpha'
            write(10,50) (clouds_phase(cloud,band,angle,1),
     &           band=1,Nb_clouds) ! cos(alpha)
c     clouds phase function for every angle
            write(10,50) (clouds_phase(cloud,band,angle,2),
     &           band=1,Nb_clouds) ! phase_function(cos(alpha))
         enddo
      enddo
      close(10)

      write(*,*) 'Output data file successfuly generated:'
      write(*,*) clouds_file(1:strlen(clouds_file))

      end
