c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine volume_exchange_position(db,dim,event,Nr,Ntheta,Nphi,
     &     r,theta,phi,ir,itheta,iphi,T,r_mc,P_in,u_in,surface_in,
     &     pf,Nc,Na,phase,g,ccoord,ka_gas,ks_gas,ka_cloud,ks_cloud,
     &     r_nodes,r_south_pole,r_north_pole,
     &     coeff_a,coeff_b,coeff_c,
     &     energy_in,weight_in,
     &     P_out,u_out,surface_out,Texch,path_length,
     &     energy_out,weight_out,valid)
      implicit none
      include 'mpif.h'
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to pick an exchange position in a cell
c
c     Inputs:
c       + dim: dimension of the vectors
c       + event: index of the event
c       + Nr,Ntheta,Nphi: number of cells
c       + r, theta, phi: limits of the spherical cells
c       + ir,itheta,iphi: indexes of the spherical cell
c       + T: temperatures at cells mass centers
c       + r_mc: radiuses of mass centers
c       + P_in: (r,theta,phi) spherical coordinates of the entry point
c       + u_in: entry direction
c       + surface_in: index of the surface where position "P_in" is located [1,6]
c       + pf: option for phase function choice
c       + Nc: number of clouds that intersect with the current cell
c       + Na: number of angles the phase function has been discretized into
c       + phase: discretized phase function array for the 'Nc' clouds
c       + g: asymetry parameter for the 'Nc' clouds
c       + ccoord: limits of the Nc clouds that intersect the current cell
c       + ka_gas,ks_gas: absorption and scattering coefficients (m) of the gas
c       + ka_cloud: absorption coefficient of the Nc clouds
c       + ks_cloud: scattering coefficient of the Nc clouds
c       + r_nodes: radius (altitude) at each one of the 4 nodes for
c                  each (theta,phi) angular sector
c       + r_south_pole: radius at south pole
c       + r_north_pole: radius at north pole
c       + coeff_a, coeff_b, coeff_c: a, b and c coefficients for each quarter
c       + energy_in: energy carried by the particle when hitting the surface
c       + weight_in: statistical weight of the particle when hitting the surface
c     
c     Outputs:
c       + P_out: (r,theta,phi) spherical coordinates of the path exit point
c       + u_out: propagation direction at exit point
c       + surface_out: index of the surface where position "P_out" is located [1,6]
c       + Texch: temperature of the chosen exchange position
c       + path_length: length of the path that has been generated (for tests only)
c       + energy_out: energy carried by the particle after reflexion
c       + weight_out: statistical weight of the particle after reflexion
c       + valid: 1 if generated path is valid; 0 otherwise
c
      
c     inputs
      integer dim,event
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      integer ir,itheta,iphi
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision P_in(1:Nmat_mx)
      double precision u_in(1:Nmat_mx)
      integer surface_in
      integer pf
      integer Nc,Na
      double precision phase(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g(1:Nclouds_mx)
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision ka_gas,ks_gas
      double precision ka_cloud(1:Nclouds_mx)
      double precision ks_cloud(1:Nclouds_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole,r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision energy_in,weight_in
c     outputs
      double precision P_out(1:Nmat_mx)
      double precision u_out(1:Nmat_mx)
      integer surface_out
      double precision Texch
      double precision path_length
      double precision energy_out,weight_out
      integer valid
c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
      integer strlen,val,val2,nnv,nnv_max,ok2go
      integer db,ons,surface,i,j,ong
      integer Npositions,app
      double precision n_in(1:Nmat_mx),prod
      double precision tau_path(1:2,1:Ndiff_mx)
      double precision x_path(1:Nmat_mx,1:Ndiff_mx)
      double precision raf,taua,pdf_taua
      double precision Pc(1:Nmat_mx)
      double precision Ps(1:Nmat_mx)
      double precision pos(1:2)
      double precision g_radius,r_ground
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision length
      double precision dz_max
      character*100 log_file
      character*100 string
      character*100 stringp_file
      integer nl
      character*100 label
c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH
      label='subroutine volume_exchange_position'
      nnv_max=10
      dz_max=0.0D+0

c     Debug
c      write(*,*) label(1:strlen(label))
c      write(*,*) 'Nr=',Nr,' Ntheta=',Ntheta,' Nphi=',Nphi
c      write(*,*) 'ir=',ir,' itheta=',itheta,' iphi=',iphi
c      write(*,*) 'Nc=',Nc,' Na=',Na
c      write(*,*) 'ka_gas=',ka_gas,' ks_gas=',ks_gas
c     Debug

      call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
     &     ca,cb,cc)
      call need2approximate(dim,Ntheta,r,theta,phi,
     &     r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)
c     modification 22/03/2009
c     I suppress surface check, because it has already been tested in the case type=1
c     by routine "next_intersection"
c     some quick checking
c      if ((ir.eq.1).and.(surface_in.eq.1).and.(app.eq.1)) then
c         call is_on_ground(P_in,theta(itheta-1),theta(itheta),
c     &        phi(iphi-1),phi(iphi),ca,cb,cc,ong)
c         if (ong.eq.0) then
c            write(*,*) 'Error from ',
c     &           label(1:strlen(label)),' :'
c            write(*,*) 'ong=',ong
c            write(*,*) 'P_in=',(P_in(i),i=1,dim)
c            pos(1)=P_in(2)
c            pos(2)=P_in(3)
c            call ground_radius(pos,theta(itheta-1),theta(itheta),
c     &           phi(iphi-1),phi(iphi),
c     &           ca,cb,cc,g_radius)
c            write(*,*) 'ground_radius=',g_radius
c            write(*,*) 'ir=',ir,' itheta=',itheta,' iphi=',iphi
c            write(*,*) 'r1=',r(ir-1,itheta,iphi),
c     &           ' r2=',r(ir,itheta,iphi)
c            write(*,*) 'theta1=',theta(itheta-1),
c     &           ' theta2=',theta(itheta)
c            write(*,*) 'phi1=',phi(iphi-1),
c     &           ' phi2=',phi(iphi)
c            stop
c         endif
c      else
c         call is_on_surface(dim,r(ir-1,itheta,iphi),r(ir,itheta,iphi),
c     &        theta(itheta-1),theta(itheta),
c     &        phi(iphi-1),phi(iphi),
c     &        P_in,ons,surface)
c         if (ons.eq.0) then
c            write(*,*) 'Error from ',
c     &           label(1:strlen(label)),' :'
c            write(*,*) 'Entry position does not belong to cell surface'
c            write(*,*) 'r1=',r(ir-1,itheta,iphi),
c     &           ' r2=',r(ir,itheta,iphi)
c            write(*,*) 'theta1=',theta(itheta-1),
c     &           ' theta2=',theta(itheta)
c            write(*,*) 'phi1=',phi(iphi-1),' phi2=',phi(iphi)
c            write(*,*) 'P_in=',(P_in(i),i=1,dim)
c            stop
c         endif
c      endif
c     modification 22/03/2009

c     coherence check
      if ((ir.eq.1).and.(surface_in.eq.1).and.(app.eq.1)) then
         call ground_normal(dim,P_in,theta(itheta-1),theta(itheta),
     &        phi(iphi-1),phi(iphi),ca,cb,cc,n_in)
      else
         call inside_normal(surface_in,P_in,n_in)
      endif
      call scalar_product(dim,u_in,n_in,prod)
      if (prod.lt.0.0D+0) then

         call logfile_name(pindex,event,log_file)
         string='./nlines'
         call stringp_file_name(pindex,string,stringp_file)
         call get_nlines(log_file,stringp_file,nl)
         open(10,file=log_file(1:strlen(log_file)))
         do i=1,nl
            read(10,*)
         enddo
         if (nl.gt.0) then
            write(10,*)
         endif

         write(10,*) 'Error from ',label(1:strlen(label)),' :'
         write(10,*) 'Emission position:'
         write(10,*) 'P_in=',(P_in(i),i=1,dim)
         write(10,*) 'Inside normal at emission position:'
         write(10,*) 'n_in=',(n_in(i),i=1,dim)
         write(10,*) 'ir=',ir,' itheta=',itheta,' iphi=',iphi
         write(10,*) 'r1=',r(ir-1,itheta,iphi),
     &        ' r2=',r(ir,itheta,iphi)
         write(10,*) 'theta1=',theta(itheta-1),
     &        ' theta2=',theta(itheta)
         write(10,*) 'phi1=',phi(iphi-1),
     &        ' phi2=',phi(iphi)
         if ((ir.eq.1).and.(surface_in.eq.1).and.(app.eq.1)) then
            write(10,*) 'computed by ground_normal'
            write(10,*) 'surface_in=',surface_in
            call vector_length(dim,n_in,length)
            write(10,*) 'length=',length
            call inside_normal(surface_in,P_in,n_in)
            write(10,*) 'normal computed by inside_normal:',
     &           (n_in(i),i=1,dim)
         else
            write(10,*) 'computed by inside_normal'
            write(10,*) 'surface_in=',surface_in
         endif
         write(10,*) 'Emission direction:'
         write(10,*) 'u_in=',(u_in(i),i=1,dim)
         write(10,*) 'scalar product:',prod
         write(10,*) 'emission direction points outwards !'
         close(10)

         valid=0
         goto 666
c         stop
      endif
c     determine whether or not the ground surface for the current column
c     is a sphere
      if (ir.eq.1) then
         call need2approximate(dim,Ntheta,r,theta,phi,
     &        r_nodes,r_south_pole,r_north_pole,itheta,iphi,
     &        app)              ! app=1: need to take into account the real surface at ground level
      else
         app=0
      endif

      if (app.eq.1) then
         call reset_onground(P_in,theta(itheta-1),theta(itheta),
     &        phi(iphi-1),phi(iphi),ca,cb,cc)
      endif

c     generation of the optical path within the cell
c     Debug
c      write(*,*) 'app=',app
c     Debug
c     Debug
      if ((surface_in.lt.1).or.(surface_in.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_in=',surface_in
         write(*,*) 'before path'
         stop
      endif
c     Debug
      call path(db,dim,event,app,
     &     r(ir-1,itheta,iphi),r(ir,itheta,iphi),
     &     theta(itheta-1),theta(itheta),
     &     phi(iphi-1),phi(iphi),
     &     P_in,u_in,surface_in,
     &     pf,Nc,Na,phase,g,
     &     ccoord,ka_gas,ks_gas,ka_cloud,ks_cloud,
     &     ca,cb,cc,
     &     Npositions,P_out,u_out,surface_out,path_length,tau_path,
     &     x_path,val)
c     Debug
      if ((val.ne.0).and.
     &     ((surface_out.lt.1).or.(surface_out.gt.6))) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_out=',surface_out
         write(*,*) 'after path'
         write(*,*) 'val=',val
         stop
      endif
c     Debug
      if (val.eq.1) then
         valid=1
      else
         valid=0
         goto 666
      endif

      if (app.eq.1) then
         call reset_onground(P_out,theta(itheta-1),theta(itheta),
     &        phi(iphi-1),phi(iphi),ca,cb,cc)
      endif
      
      nnv=0
      ok2go=0
      do while (ok2go.eq.0)
c     generate "taua", the absorption optical depth between the exchange position and
c     the origin of the path (position "P_in")
         call pdf_tau_abs(1,tau_path(1,Npositions),raf,taua,pdf_taua)
c     retrieving position "P" from the generated absorption optical depth
         call find_position(dim,Npositions,x_path,tau_path,taua,
     &        Pc,Ps,val2)
         if (val2.eq.0) then
            ok2go=0
            nnv=nnv+1
         else
            ok2go=1
         endif
         if (nnv.ge.nnv_max) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'could not find valid position for:'
            write(*,*) 'taua(',Npositions,')=',tau_path(1,Npositions)
            write(*,*) 'Number of failed attempts:',nnv
            stop
         endif
      enddo
c     interpolation of temperature at exchange position (not needed for solar)
c      call gas_temp(dim,Nr,Ntheta,Nphi,ir,itheta,iphi,r,theta,phi,
c     &     r_nodes,r_south_pole,r_north_pole,ca,cb,cc,
c     &     Ps(1),Ps(2),Ps(3),T,r_mc,Texch)

c     old call (before variable ground altitudes)
cc      call gas_temp(Nr,Ntheta,Nphi,ir,itheta,iphi,r,theta,phi,
cc     &     Ps(1),Ps(2),Ps(3),T,r_mc,Texch)

c     energy and statistical weight of the particle
c     Debug
c      write(*,*) 'from ',label(1:strlen(label))
c      write(*,*) 'path_length=',path_length
c      write(*,*) 'tau a(',Npositions,')=',tau_path(1,Npositions)
c     Debug

c     ----------------------------------------------------------
c     Absorption:
c     In order to take into account absorption by the medium,
c     please comment the following line:
      energy_out=energy_in
c     and uncomment this one:
c      energy_out=energy_in*dexp(-tau_path(1,Npositions))
c     ----------------------------------------------------------

      weight_out=weight_in

 666  continue
      return
      end


      subroutine find_position(dim,Npositions,x_path,tau_path,taua,
     &     Pc,Ps,valid)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify the position that corresponds to "taua"
c     in a scattering optical path
c
c     Inputs:
c       + dim: dimension of the vectors
c       + Npositions: number of positions that compose the optical path
c       + x_path: list of the spherical coordinates of each position
c         - x_path(1,i): r at position index i (i=1,Npositions)
c         - x_path(2,i): theta at position index i (i=1,Npositions)
c         - x_path(3,i): phi at position index i (i=1,Npositions)
c       + tau_path: absorption and scattering optical thickness along the path,
c                  at each scattering position
c         - tau_path(1,i): absorption optical depth at position i, i=1,Npositions
c         - tau_path(2,i): scattering optical depth at position i, i=1,Npositions
c       + taua: absorption optical depth at position P
c
c     Outputs
c       + Pc: (x,y,z) cartesian coordinates of the position corresponding to "taua"
c       + Ps: (r,theta,phi) spherical coordinates of the position corresponding to "taua"
c       + valid: 1 if found position is valid, 0 otherwise
c     

c     input
      integer dim,Npositions
      double precision x_path(1:Nmat_mx,1:Ndiff_mx)
      double precision tau_path(1:2,1:Ndiff_mx)
      double precision taua
c     output
      double precision Pc(1:Nmat_mx)
      double precision Ps(1:Nmat_mx)
      integer valid
c     temp
      integer strlen
      integer i,if,index
      double precision tau1,tau2
      double precision x1,y1,z1
      double precision x2,y2,z2
      integer col
      double precision u(1:Nmat_mx)
      double precision v(1:Nmat_mx)
      double precision epsilon_taua
      character*100 label
      parameter(epsilon_taua=1.0D-5)
      label='subroutine find_position'

      valid=1
c     some quick checking
      if (taua.gt.tau_path(1,Npositions)) then
         valid=0
         goto 666
      endif

c     now identifying 'index' so that tau_path(1,index-1) < taua < tau_path(1,index)
      if=0
      do i=2,Npositions
         if ((tau_path(1,i-1).le.taua).and.(tau_path(1,i).ge.taua)) then
            if=1
            index=i
            goto 123
         endif
      enddo
 123  continue
      if (if.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'tau',taua,' could not be retrieved in the path'
         write(*,*) 'tau_path(1,1)=',tau_path(1,1)
         write(*,*) 'tau_path(1,',Npositions,')=',tau_path(1,Npositions)
         stop
      endif
c     and finally, interpolation of the position
      tau1=tau_path(1,index-1)
      tau2=tau_path(1,index)
      call spher2cart(label,1,x_path(1,index-1),x_path(2,index-1),
     &     x_path(3,index-1),x1,y1,z1)
      call spher2cart(label,1,x_path(1,index),x_path(2,index),
     &     x_path(3,index),x2,y2,z2)
      call lin_interp1d(tau1,tau2,taua,x1,x2,Pc(1))
      call lin_interp1d(tau1,tau2,taua,y1,y2,Pc(2))
      call lin_interp1d(tau1,tau2,taua,z1,z2,Pc(3))
      call cart2spher(Pc(1),Pc(2),Pc(3),Ps(1),Ps(2),Ps(3))
c     consistency check
      u(1)=x2-x1
      u(2)=y2-y1
      u(3)=z2-z1
      v(1)=Pc(1)-x1
      v(2)=Pc(2)-y1
      v(3)=Pc(3)-z1
      if (taua.gt.epsilon_taua) then
         call are_colinear(dim,u,v,col)
         if (col.eq.0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Position P is not on the segment'
            write(*,*) 'P1:',(x_path(i,index-1),i=1,dim)
            write(*,*) 'P2:',(x_path(i,index),i=1,dim)
            write(*,*) 'P1:',x1,y1,z1
            write(*,*) 'P2:',x2,y2,z1
            write(*,*) 'taua=',taua
            write(*,*) 'Ps:',(Ps(i),i=1,dim)
            write(*,*) 'Pc:',(Pc(i),i=1,dim)
            stop
         endif
      endif

 666  continue
      return
      end
