c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine next_intersection(db,dim,event,app,
     &     r1,r2,theta1,theta2,phi1,phi2,
     &     Nc,ccoord,P1,u1,length,
     &     ca,cb,cc,Pint,type,surface_out)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c
c     Purpose: to identify the next intersection position 
c     along the line defined by position "P1" and propagation direction "u1"
c     
c     Inputs:
c       + dim: dimension of vectors
c       + event: index of the event
c       + app: if equal to 1, the real surface at ground level must be taken into account; 0 otherwise
c       + r1, theta1, phi1, r2, theta2, phi2: limits of the spherical box
c       + Nc: number of clouds that intersect with the current box
c       + ccoord: limits of these Nc clouds
c       + P1: (r,theta,phi) spherical coordinates of the first position
c       + u1: direction of propagation from P1 to P2
c       + length: scattering length
c       + ca,cb,cc: a, b and c coefficients for the 4 quarters of the angular sector
c
c     Outputs:
c       + Pint: (r,theta,phi) spherical coordinates of the intersection position
c       + type: intersection position type
c         - type=0: error during path generation; waste this path
c         - type=1: box output position
c         - type=2: scattering position
c         - type=3: cloud intersection position
c       + surface_out: surface index when type=1
c

c     input
      integer dim,event,app,Nc
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision P1(1:Nmat_mx)
      double precision u1(1:Nmat_mx)
      double precision length
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     output
      double precision Pint(1:Nmat_mx)
      integer type
      integer surface_out
c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
      integer strlen,j,i,srefused
      integer ons,surf1,ok,inc
      double precision P2(1:Nmat_mx)
      double precision alpha
      double precision Pt(1:Nmat_mx)
      double precision Pout(1:Nmat_mx)
      double precision Po_int(1:Nclouds_mx,1:Nmat_mx)
      double precision n1(1:Nmat_mx),prod
      integer surf_int(1:Nclouds_mx),surf
      integer solf,surface
      integer Nint,intf,cloud,in,err
      double precision distance_int(1:Nclouds_mx)
      double precision dist_min,d12,d,dmin,r_ground
      double precision Psca(1:Nmat_mx)
      double precision dz_max
      character*100 log_file
      character*100 string
      character*100 stringp_file
      integer nl
      character*100 label
c     Debug
      integer db,k,it
      double precision pos(1:2)
      double precision g_radius
c     Debug
c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH
      label='subroutine next_intersection'
      dz_max=0.0D+0 ! m
      Nint=0
c     Debug
      if (db.eq.1) then
         write(*,*)
         write(*,*) label(1:strlen(label)),' :in'
      endif

      if (app.eq.1) then
         call reset_onground(P1,theta1,theta2,phi1,phi2,ca,cb,cc)
      endif

      call is_in_cell(P1,app,r1,r2,theta1,theta2,
     &     phi1,phi2,ca,cb,cc,ok)

 111  continue
      if (ok.eq.0) then
         call logfile_name(pindex,event,log_file)
         string='./nlines'
         call stringp_file_name(pindex,string,stringp_file)
         call get_nlines(log_file,stringp_file,nl)
         open(10,file=log_file(1:strlen(log_file)))
         do i=1,nl
            read(10,*)
         enddo
         if (nl.gt.0) then
            write(10,*)
         endif
         write(10,*) 'Error from ',label(1:strlen(label)),':'
         write(10,*) 'P1=',(P1(j),j=1,dim)
         write(10,*) 'is not in cell'
         write(10,*) 'r1=',r1,' r2=',r2
         write(10,*) 'theta1=',theta1,' theta2=',theta2
         write(10,*) 'phi1=',phi1,' phi2=',phi2
         if (app.eq.1) then
            write(10,*) 'app=',app
            write(10,*) 'r_ground=',r_ground
         endif
         close(10)
         type=0
         goto 666
c         stop
      endif

         
c     compute position "P2", intersection over the box
c     Debug
      if (db.eq.1) then
         write(*,*) 'testing intersection with box'
      endif
c     Debug

      call intersection(db,dim,event,app,1,1,P1,u1,
     &     r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,
     &     1,1,solf,surface_out,P2,err)
c     Debug
      if (db.eq.1) then
         write(*,*) 'P2=',(P2(k),k=1,dim)
      endif
c     Debug
      if (err.eq.1) then
         type=0
         goto 666
      endif
      if (app.eq.1) then
         call reset_onground(P2,theta1,theta2,phi1,phi2,ca,cb,cc)
      endif
c     Debug
      if (db.eq.1) then
         write(*,*) 'P1=',(P1(k),k=1,dim)
         write(*,*) 'P2=',(P2(k),k=1,dim)
         write(*,*) 'app=',app
         write(*,*) 'r_ground=',r_ground
         write(*,*) 'theta=',theta1,theta2
         write(*,*) 'phi=',phi1,phi2
         write(*,*) 'ca=',(ca(i),i=1,4)
         write(*,*) 'cb=',(cb(i),i=1,4)
         write(*,*) 'cc=',(cc(i),i=1,4)
      endif
c     Debug
      call distance_spher(label,P1(1),P1(2),P1(3),
     &     P2(1),P2(2),P2(3),d12)
c     Debug
      if (db.eq.1) then
         write(*,*) 'd12=',d12, 'length=',length
      endif
c     Debug
      if (d12.lt.length) then
         dmin=d12
      else
         dmin=length
      endif

c     check intersections with clouds
      intf=0
      do cloud=1,Nc
c     Debug
         if (db.eq.1) then
            write(*,*) 'testing intersection with cloud=',cloud
            write(*,*) 'P1=',(P1(j),j=1,dim)
            write(*,*) 'u1=',(u1(j),j=1,dim)
         endif
c     stop
         call intersection(db,dim,event,0,0,1,P1,u1,
     &        ccoord(cloud,1),ccoord(cloud,2),
     &        ccoord(cloud,3),ccoord(cloud,4),
     &        ccoord(cloud,5),ccoord(cloud,6),
     &        ca,cb,cc,
     &        0,0,solf,surface,Pt,err)
         if (solf.eq.1) then
c     check that solution "Pt" is in the current cell
            call is_in_cell(Pt,app,r1,r2,theta1,theta2,
     &           phi1,phi2,ca,cb,cc,in)
            d=2.0D+0*dmin        ! default > dmin
            if (in.eq.1) then
c     Debug
               if (db.eq.1) then
                  write(*,*) 'P1=',(P1(k),k=1,dim)
                  write(*,*) 'Pt=',(Pt(k),k=1,dim)
               endif
c     Debug
               call distance_spher(label,P1(1),P1(2),P1(3),
     &              Pt(1),Pt(2),Pt(3),d)
               if ((d.gt.0.0D+0).and.(d.lt.dmin)) then
                  intf=1
                  type=3        ! intersection with a cloud
c     Debug
                  if (db.eq.1) then
                     write(*,*) 'd=',d,' type=',type
                  endif
c     Debug
                  goto 123
               endif
            endif               ! in=1
         endif                  ! solf=1
      enddo                     ! cloud


      if (intf.eq.0) then
c     if no intersection with a cloud has been found, then the next
c     intersection position is either the box output position, either
c     a scattering position

c     compute position after scattering from "P1" in direction "u1" by "length"
         if (length.lt.d12) then
            call jump_to(dim,P1,u1,length,Psca)
c     "Psca" is the position after scattering
            call is_in_cell(Psca,app,r1,r2,theta1,theta2,
     &           phi1,phi2,ca,cb,cc,in)
            if (in.eq.0) then
               srefused=1       ! scattering is refused
c     Debug
c               write(*,*) label(1:strlen(label)),
c     &              ' :scattering position refused'
c     Debug
            else
               srefused=0       ! ok for scattering
            endif
         endif
         
         if ((d12.le.length).or.(srefused.eq.1)) then
            type=1              ! box output position
c     if distance to box output position is lower than distance to the next scattering position
c            it=0
            it=1
            call copy_vector(dim,P2,Pint)
c     Debug
            if (db.eq.1) then
               write(*,*) 'type=',type
            endif
c     Debug
 444        continue
            if (app.eq.1) then
               pos(1)=Pint(2)
               pos(2)=Pint(3)
               call ground_radius(pos,theta1,theta2,phi1,phi2,
     &              ca,cb,cc,r_ground)
            endif

            if ((surface_out.eq.1).and.(app.eq.1)) then
               call is_on_ground(Pint,theta1,theta2,
     &              phi1,phi2,ca,cb,cc,ons)
            else if ((app.eq.1).and.
     &              ((surface_out.eq.3).or.(surface_out.eq.4))) then
               if ((Pint(2).ge.theta1).and.(Pint(2).le.theta2)
     &              .and.(Pint(1).ge.r_ground-1.0D-4)) then
                  ons=1
               else
                  ons=0
               endif
            else if ((app.eq.1).and.
     &              ((surface_out.eq.5).or.(surface_out.eq.6))) then
               if ((Pint(3).ge.phi1).and.(Pint(3).le.phi2)
     &              .and.(Pint(1).ge.r_ground-1.0D-4)) then
                  ons=1
               else
                  ons=0
               endif
            else
               call is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &              Pint,ons,surf_int)
            endif

            if ((ons.eq.0).and.(it.gt.0)) then
               call logfile_name(pindex,event,log_file)
               string='./nlines'
               call stringp_file_name(pindex,string,stringp_file)
               call get_nlines(log_file,stringp_file,nl)
               open(10,file=log_file(1:strlen(log_file)))
               do i=1,nl
                  read(10,*)
               enddo
               if (nl.gt.0) then
                  write(10,*)
               endif

               write(10,*) 'Error from ',
     &              label(1:strlen(label)),':'
               write(10,*) 'Path ending position:',(Pint(j),j=1,dim)
               write(10,*) 'does not belong to cell surface'
               write(10,*) 'r1=',r1
               write(10,*) 'r2=',r2
               write(10,*) 'theta1=',theta1
               write(10,*) 'theta2=',theta2
               write(10,*) 'phi1=',phi1
               write(10,*) 'phi2=',phi2
               write(10,*) 'intersection type:',type
               write(10,*) 'app=',app
               write(10,*) 'surface_out=',surface_out
               write(10,*) 'r_ground=',r_ground
               if ((surface_out.eq.1).and.(app.eq.1)) then
                  pos(1)=Pint(2)
                  pos(2)=Pint(3)
                  call ground_radius(pos,theta1,theta2,phi1,phi2,
     &                 ca,cb,cc,g_radius)
                  write(10,*) 'ground_radius=',g_radius
                  write(10,*) 'ca=',(ca(j),j=1,4)
                  write(10,*) 'cb=',(cb(j),j=1,4)
                  write(10,*) 'cc=',(cc(j),j=1,4)
               endif
               close(10)
               type=0
               goto 666
            endif
            goto 666 ! with type=1
         else ! if scattering is accepted
            type=2 ! scattering position
c     if distance to box output position is greater than distance to the next scattering position
            call copy_vector(dim,Psca,Pint)
c     Debug
            if (db.eq.1) then
               write(*,*) 'scattering at length=',length
               write(*,*) 'u1=',(u1(j),j=1,dim)
               write(*,*) 'Pint=',(Pint(j),j=1,dim)
               write(*,*) 'type=',type
               pos(1)=Pint(2)
               pos(2)=Pint(3)
               call ground_radius(pos,theta1,theta2,phi1,phi2,
     &              ca,cb,cc,r_ground)
               write(*,*) 'r_ground=',r_ground
            endif
c     Debug
            goto 666
         endif
      endif

 123  continue
c     Debug
      if (db.eq.1) then
         write(*,*) 'Intersection with cloud found'
      endif
c      stop
c     Debug
c     If an intersection with a cloud has been detected, check every cloud that intersects the box
      Nint=0
      do cloud=1,Nc
c     Debug
         if (db.eq.1) then
            write(*,*) 'testing intersection with cloud=',cloud
            write(*,*) 'P1=',(P1(j),j=1,dim)
            write(*,*) 'u1=',(u1(j),j=1,dim)
         endif
c     stop

         call intersection(db,dim,event,0,0,1,P1,u1,
     &        ccoord(cloud,1),ccoord(cloud,2),
     &        ccoord(cloud,3),ccoord(cloud,4),
     &        ccoord(cloud,5),ccoord(cloud,6),
     &        ca,cb,cc,
     &        0,0,solf,surface,Pout,err)

         if (solf.eq.1) then
            call is_in_cell(Pout,app,r1,r2,theta1,theta2,
     &           phi1,phi2,ca,cb,cc,in)

            if (in.eq.1) then
c     update the number of intersections found
               Nint=Nint+1
c     compute the distance from the starting position to the current intersection "P1"
c     Debug
               if (db.eq.1) then
                  write(*,*) 'P1=',(P1(k),k=1,dim)
                  write(*,*) 'Pout=',(Pout(k),k=1,dim)
               endif
c     Debug
               call distance_spher(label,P1(1),P1(2),P1(3),
     &              Pout(1),Pout(2),Pout(3),distance_int(Nint))
c     record coordinates of the current intersection position and surface index
               do j=1,dim
                  Po_int(Nint,j)=Pout(j)
               enddo            ! j
               surf_int(Nint)=surface
            endif               ! in=1

         endif                  ! solf=1

      enddo                     ! cloud
      
      if (Nint.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'No intersection found'
         stop
      endif

c     find the intersection that is the closest from the starting position "P1"
      if (Nint.ge.1) then
         dist_min=distance_int(1)
         do j=1,dim
            Pint(j)=Po_int(1,j)
         enddo                  ! j
         surf=surf_int(j)
      endif
      if (Nint.gt.1) then
         do i=2,Nint
            if (distance_int(i).lt.dist_min) then
               dist_min=distance_int(i)
c     get intersection position
               do j=1,dim
                  Pint(j)=Po_int(i,j)
               enddo            ! j
               surf=surf_int(j)
            endif
         enddo                  ! i
      endif                     ! Nint>1

c     Debug
      if (db.eq.1) then
c         write(*,*) 'd12=',d12
         write(*,*) 'u1=',(u1(j),j=1,dim)
         do i=1,Nint
            write(*,*) 'Po_int(',i,')=',(Po_int(i,j),j=1,dim)
            write(*,*) 'distance_int(',i,')=',distance_int(i)
         enddo
         write(*,*) 'dist_min=',dist_min
         write(*,*) 'Pint=',(Pint(j),j=1,dim)
      endif
c      stop

 666  continue


      if (type.ne.0) then
         pos(1)=Pint(2)
         pos(2)=Pint(3)
         call ground_radius(pos,theta1,theta2,phi1,phi2,
     &        ca,cb,cc,r_ground)
         if (Pint(1).lt.r_ground-1.0D+2) then ! this is REALLY a problem !!!
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'Intersection position is below ground:'
            write(*,*) 'Pint=',(Pint(i),i=1,dim)
            write(*,*) 'r_ground=',r_ground
            write(*,*) 'type=',type
            write(*,*) 'surface_out=',surface_out
            stop
         endif
      endif
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :out'
         write(*,*)
      endif
c     Debug

      return
      end



      subroutine intersection(db,dim,event,app,box,gmethod,Pstart,u,
     &     r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,inb,error0,
     &     solf,surface,Pint,aerr)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     
c     Purpose: to identify the intersection point between a line that initiates
c     at a position "Pstart" within a box, along direction "u", and the surface of the box
c
c     Inputs
c       + dim: dimensions of the vectors
c       + event: index of the event
c       + app: if equal to 1, an intersection with real ground must be looked for; 0 otherwise
c       + box: if 1, an intersection over a spherical box is looked for; 0 for clouds
c       + gmethod: option for choosing the method used in the detection of intersection with ground
c         - gmethod=1: standard Newton method
c         - gmethod=2: raw dichotomy procedure
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + r1,r2,theta1,theta2,phi1,phi2: variables that defines the box
c       + ca,cb,cc: a, b and c coefficients for the 4 quarters of the angular sector
c       + inb: option for using only "Pstart" in box
c         - inb=0: DO NOT check if "Pstart" is in box
c         - inb=1: check if "Pstart" is in box
c       + error0: option for error when n_sol=0
c         - error0=0: if n_sol=0, DO NOT generate an error
c         - error0=1: generate an errror if n_sol=0
c     
c     Outputs:
c       + solf: 0 if no intersection found; 1 if an intersection was found
c       + surface: index of the intersected surface
c       + Pint: (r,theta,phi) spherical coordinates of the intersection point, if any
c       + aerr: error code
c         - aerr=0: no error occured
c         - aerr=1: a known error occured
c

c     input
      integer dim,event,app,box,gmethod
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      integer inb,error0
c     output
      integer solf
      integer surface
      double precision Pint(1:Nmat_mx)
      integer aerr
c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
      integer solf_ai
      double precision pos2d(1:2)
      double precision r_ground
      integer strlen,ioe,edge,ps_ong,err_code
      double precision P1int(1:Nmat_mx)
      double precision P2int(1:Nmat_mx)
      double precision P3int(1:Nmat_mx)
      double precision P4int(1:Nmat_mx)
      double precision P5int(1:Nmat_mx)
      double precision P6int(1:Nmat_mx)
      double precision Pj(1:Nmat_mx)
      integer surf(1:6),valid
      integer intfs(1:6),intf,intfg,intfg1
      double precision Pground(1:Nmat_mx)
      integer in,i,j,n_sol,sol,solf_nv,nsol_nv
      integer sol_nv(1:6)
      double precision length_nv(1:6)
      double precision length(1:6),length_min
      integer err(1:Nclouds_mx)
      double precision Ptmp(1:Nmat_mx)
      integer surf_tmp,check_ground_last
      character*100 label
c     Debug
      integer db,col,k,ok,inc
      double precision Pstartc(1:Nmat_mx)
      double precision Pintc(1:Nmat_mx)
      double precision Pjc(1:Nmat_mx)
      double precision Pout(1:Nmat_mx)
      double precision Poutc(1:Nmat_mx)
      double precision Pn(1:Nmat_mx)
      double precision poss(1:Nmat_mx)
      double precision seg(1:Nmat_mx)
      double precision prod,d
      double precision n(1:Nmat_mx)
      double precision n_start(1:Nmat_mx)
      integer ons,surf_Pstart,surf_Pout,err_Pout
      integer surf1
      double precision epsilon_prod
      double precision epsilon_length
      double precision pos(1:2)
      double precision g_radius,dr_ground
      double precision dz_max
      character*100 log_file
      character*100 string
      character*100 stringp_file
      integer nl
      parameter(epsilon_prod=2.D-2)
      parameter(epsilon_length=1.D-5) ! m
c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH
      dz_max=0.0D+0 ! m
      dr_ground=1.0D-1 ! m

c     Debug
      label='subroutine intersection'
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :in'
         write(*,*) 'r1=',r1,' r2=',r2
         write(*,*) 'theta1=',theta1,' theta2=',theta2
         write(*,*) 'phi1=',phi1,' phi2=',phi2
      endif
c     Debug

      call test_box(r1,r2,theta1,theta2,phi1,phi2,label)

      if (app.eq.1) then
         call reset_onground(Pstart,theta1,theta2,phi1,phi2,ca,cb,cc)
      endif

      if (inb.eq.1) then
         call is_in_cell(Pstart,app,r1,r2,theta1,theta2,
     &        phi1,phi2,ca,cb,cc,ok)
         if (ok.eq.0) then
            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif
            write(10,*) 'Error from ',label(1:strlen(label)),':'
            write(10,*) 'Input argument Pstart is not in box'
            write(10,*) 'Pstart=',(Pstart(i),i=1,dim)
            write(10,*) 'r1=',r1,' r2=',r2
            write(10,*) 'theta1=',theta1,' theta2=',theta2
            write(10,*) 'phi1=',phi1,' phi2=',phi2
            if (app.eq.1) then
               write(10,*) 'app=',app
               write(10,*) 'r_ground=',r_ground
            endif
            close(10)
            aerr=1
            goto 666
c            stop
         endif
      endif

c     ground radius at starting position
      ps_ong=0
      if (app.eq.1) then
         if (dabs(Pstart(1)-r_ground).lt.dr_ground) then
            ps_ong=1 ! starting position is on ground
c     Debug
            if (db.eq.1) then
               write(*,*) 'Pstart=',(Pstart(i),i=1,dim)
               write(*,*) 'r_ground=',r_ground
               write(*,*) 'ps_ong=',ps_ong
c               stop
            endif
c     Debug
         endif
c     Debug
         if (db.eq.1) then
            write(*,*) 'ps_ong=',ps_ong
         endif
c     Debug
      endif

c     Debug
      if (db.eq.1) then
         call inside_normal(2,Pstart,n)
         call scalar_product(dim,u,n,prod)
         write(*,*) 'prod=',prod
      endif
c     Debug

      n_sol=0
      intfg=0
      do i=1,6
         intfs(i)=0
      enddo
c     testing intersection with r-type surfaces 1 and 2
      if (app.eq.1) then
c     testing intersection with real ground surface
c     Debug
         if (db.eq.1) then
            write(*,*) 'testing ground'
         endif
c     Debug
         call intersect_ground(dim,gmethod,Pstart,u,
     &        r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,
     &        intfg,Pground)
         if (intfg.eq.1) then
c     Debug
            if (db.eq.1) then
               write(*,*) 'intersection found with real ground'
               write(*,*) 'Pground=',(Pground(i),i=1,dim)
            endif
c     Debug
            intfs(1)=1
            call copy_vector(dim,Pground,P1int)
c     Debug
            call reset_onground(P1int,theta1,theta2,phi1,phi2,ca,cb,cc)
c     Debug
         endif
c     Debug
         if (db.eq.1) then
            write(*,*) 'intfg=',intfg
            write(*,*) 'intfs(1)=',intfs(1)
         endif
c     Debug
      else
c     Debug
         if (db.eq.1) then
            write(*,*) 'testing r1=',r1
         endif
c     Debug
         call line_surface_r_intersect(db,dim,Pstart,u,
     &        r1,theta1,theta2,phi1,phi2,intfs(1),P1int)
      endif
      if (intfs(1).eq.1) then
         n_sol=1
         surf(1)=1
      endif

c     Debug
      if (db.eq.1) then
c         write(*,*) 'intfs(1)=',intfs(1),' P1int=',(P1int(k),k=1,dim)
         write(*,*) 'testing r2=',r2
      endif
c     Debug
      call line_surface_r_intersect(db,dim,Pstart,u,
     &     r2,theta1,theta2,phi1,phi2,intfs(2),P2int)
      if (intfs(2).eq.1) then
         n_sol=n_sol+1
         surf(n_sol)=2
      endif
c     testing intersection with theta-type surfaces 3 and 4
c     Debug
      if (db.eq.1) then
c         write(*,*) 'intfs(2)=',intfs(2)
         write(*,*) 'testing phi1=',phi1
      endif
c     Debug
      call line_surface_theta_intersect(dim,app,Pstart,u,
     &     r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,phi1,
     &     intfs(3),P3int)
      if (intfs(3).eq.1) then
         n_sol=n_sol+1
         surf(n_sol)=3
      endif
c     Debug
      if (db.eq.1) then
c         write(*,*) 'intfs(3)=',intfs(3),' P3int=',(P3int(k),k=1,dim)
         write(*,*) 'testing phi2=',phi2
      endif
c     Debug
      call line_surface_theta_intersect(dim,app,Pstart,u,
     &     r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,phi2,
     &     intfs(4),P4int)
      if (intfs(4).eq.1) then
         n_sol=n_sol+1
         surf(n_sol)=4
      endif
c     testing intersection with phi-type surfaces 5 and 6
c     Debug
      if (db.eq.1) then
c         write(*,*) 'intfs(4)=',intfs(4)
         write(*,*) 'testing theta1=',theta1
      endif
c     Debug
      call line_surface_phi_intersect(0,dim,app,Pstart,u,
     &     r1,r2,theta1,theta1,theta2,phi1,phi2,ca,cb,cc,
     &     intfs(5),P5int)
      if (intfs(5).eq.1) then
         n_sol=n_sol+1
         surf(n_sol)=5
      endif
c     Debug
      if (db.eq.1) then
c         write(*,*) 'intfs(5)=',intfs(5)
         write(*,*) 'testing theta2=',theta2
      endif
c     Debug
      call line_surface_phi_intersect(0,dim,app,Pstart,u,
     &     r1,r2,theta2,theta1,theta2,phi1,phi2,ca,cb,cc,
     &     intfs(6),P6int)
      if (intfs(6).eq.1) then
         n_sol=n_sol+1
         surf(n_sol)=6
      endif


c     Debug
      if (db.eq.1) then
         write(*,*) 'n_sol=',n_sol
         do j=1,6
            if (intfs(j).eq.1) then
               write(*,*) 'intfs(',j,')=',intfs(j)
            endif
         enddo
      endif
c     Debug

      solf=0
      solf_nv=0
      nsol_nv=0
      sol=0
      aerr=0

c     there should be only one !
      if (n_sol.eq.0) then
         if (box.eq.1) then
            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif
            write(10,*) label(1:strlen(label)),' aborting path, error 1'
c     modif 2009-04-26
            close(10)

            call alternative_intersection(dim,event,app,Pstart,u,
     &           r1,r2,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,
     &           solf_ai,Pint,surface)

            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif

            if (solf_ai.eq.0) then
               solf=0
               aerr=1
               write(10,*) 'Alternative intersection method failed'
            else
               solf=1
               aerr=0
               write(10,*) 'Alternative intersection was successfull:'
               write(10,*) 'Pint=',(Pint(i),i=1,dim)
               write(10,*) 'surface=',surface
            endif
            close(10)
            goto 666
c     modif 2009-04-26
c            aerr=1
c            goto 666
         endif
         goto 666
      endif

      if (n_sol.eq.1) then
         call find_Pj(dim,surf(1),P1int,P2int,P3int,P4int,
     &        P5int,P6int,Pj)
         call check_validity(dim,app,intfg,r1,r2,theta1,theta2,
     &        phi1,phi2,
     &        ca,cb,cc,Pstart,u,Pj,surf(1),valid,length(1),err(1))
         if ((box.eq.1).and.(valid.eq.1).and.
     &        (length(1).gt.0.0D+0)) then
            sol=1
            surface=surf(1)
            solf=1
            if ((ps_ong.eq.1).and.(length(1).lt.dr_ground)) then
               call logfile_name(pindex,event,log_file)
               string='./nlines'
               call stringp_file_name(pindex,string,stringp_file)
               call get_nlines(log_file,stringp_file,nl)
               open(10,file=log_file(1:strlen(log_file)))
               do i=1,nl
                  read(10,*)
               enddo
               if (nl.gt.0) then
                  write(10,*)
               endif

               write(10,*) 'Warning from routine ',
     &           label(1:strlen(label)),':'
               write(10,*) 'only one intersection found:'
               write(10,*) 'Pj=',(Pj(i),i=1,dim)
               write(10,*) 'with starting position:'
               write(10,*) 'Pstart=',(Pstart(i),i=1,dim)
               write(10,*) 'u=',(u(i),i=1,dim)
               write(10,*) 'distance=',length(1)
               write(10,*) 'r1=',r1,' r2=',r2
               write(10,*) 'theta1=',theta1,' theta2=',theta2
               write(10,*) 'phi1=',phi1,' phi2=',phi2
               pos(1)=Pstart(2)
               pos(2)=Pstart(3)
               call ground_radius(pos,theta1,theta2,phi1,phi2,
     &              ca,cb,cc,r_ground)
               write(10,*) 'r_ground=',r_ground
               close(10)
c               stop
            endif
            goto 222
         else if ((box.eq.1).and.(valid.eq.0)) then
            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif
            write(10,*) label(1:strlen(label)),' aborting path, error 2'
            write(10,*) 'Error from ',
     &           label(1:strlen(label)),':'
            write(10,*) 'valid=',valid
            write(10,*) 'length=',length(1)
            write(10,*) 'Pstart=',(Pstart(k),k=1,dim)
            write(10,*) 'u=',(u(i),i=1,dim)
            write(10,*) 'Pj=',(Pj(k),k=1,dim)
            write(10,*) 'r1=',r1,' r2=',r2
            write(10,*) 'theta1=',theta1,' theta2=',theta2
            write(10,*) 'phi1=',phi1,' phi2=',phi2
            write(10,*) 'app=',app
            if (app.eq.1) then
               pos(1)=Pstart(2)
               pos(2)=Pstart(3)
               call ground_radius(pos,theta1,theta2,phi1,phi2,
     &              ca,cb,cc,g_radius)
               call line_surface_theta_intersect(dim,1,Pstart,u,
     &              r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,phi2,
     &              intfs(4),P4int)
               write(10,*) 'intfs(4)=',intfs(4)
               write(10,*) 'P4int=',(P4int(i),i=1,dim)
            endif
c     modif 2009-04-26
            close(10)

            call alternative_intersection(dim,event,app,Pstart,u,
     &           r1,r2,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,
     &           solf_ai,Pint,surface)

            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif

            if (solf_ai.eq.0) then
               solf=0
               aerr=1
               write(10,*) 'Alternative intersection method failed'
            else
               solf=1
               aerr=0
               write(10,*) 'Alternative intersection was successfull:'
               write(10,*) 'Pint=',(Pint(i),i=1,dim)
               write(10,*) 'surface=',surface
            endif
            close(10)
            goto 666
c     modif 2009-04-26
c            aerr=1
c            goto 666
c            stop
         else if (box.eq.0) then
            solf=0
         endif
      endif

      check_ground_last=0
      do j=1,n_sol
         call find_Pj(dim,surf(j),P1int,P2int,P3int,P4int,
     &        P5int,P6int,Pj)
         if ((intfg.eq.1).and.(surf(j).eq.1)) then
            check_ground_last=1
c     Debug
         if (db.eq.1) then
            write(*,*) 'intersection with ground'
            write(*,*) 'will be checked last'
         endif
c     Debug
            goto 333
         endif
         call check_validity(dim,app,0,r1,r2,theta1,theta2,
     &        phi1,phi2,
     &        ca,cb,cc,Pstart,u,Pj,surf(j),valid,length(j),err(j))
c         if ((intfg1.eq.1).and.(ps_ong.eq.1).and.
c     &        (length(j).lt.1.0D-1).and.(valid.eq.1)) then
c            valid=0
c     Debug
c            if (db.eq.1) then
c               write(*,*) 'solution declared as invalid:'
c               write(*,*) 'P(j=',j,')=',(Pj(i),i=1,dim)
c               write(*,*) 'because length=',length(j)
c            endif
c     Debug
c         endif

c     Debug
         if (db.eq.1) then
            write(*,*) 'n_sol=',n_sol,' j=',j
            write(*,*) 'surf(',j,')=',surf(j)
            write(*,*) 'Pstart=',(Pstart(k),k=1,dim)
            write(*,*) 'P(',j,')=',(Pj(k),k=1,dim)
            call find_Pj(dim,surf(j),P1int,P2int,P3int,P4int,
     &           P5int,P6int,Pj)
            pos2d(1)=Pj(2)
            pos2d(2)=Pj(3)
            call ground_radius(pos2d,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,r_ground)
            write(*,*) 'r_ground=',r_ground
            write(*,*) 'valid=',valid,' length=',length(j)
            if (Pj(1).gt.0.0D+0) then
               call spher2cart(label,1,Pstart(1),Pstart(2),Pstart(3),
     &              Pstartc(1),Pstartc(2),Pstartc(3))
               call spher2cart(label,1,Pj(1),Pj(2),Pj(3),
     &              Pjc(1),Pjc(2),Pjc(3))
               call substract_vectors(dim,Pjc,Pstartc,seg)
               call are_colinear(dim,u,seg,col)
               write(*,*) 'u(',j,')=',(u(k),k=1,dim)
               write(*,*) 'seg(',j,')=',(seg(k),k=1,dim)
               write(*,*) 'col=',col
            endif
         endif
c     Debug
         if (valid.eq.1) then
            if ((solf.eq.0).and.(length(j).gt.epsilon_length)) then
               length_min=length(j)
               sol=j
               surface=surf(sol)
               solf=1
            else
               if ((length(j).lt.length_min).and.
     &              (length(j).gt.epsilon_length)) then
                  length_min=length(j)
                  sol=j
                  surface=surf(sol)
               endif ! length
            endif ! j
         else if (valid.eq.-1) then
            solf_nv=1
            nsol_nv=nsol_nv+1
            sol_nv(nsol_nv)=j
            length_nv(nsol_nv)=length(j)
         endif ! valid
 333     continue
      enddo ! j


      if ((solf.eq.0).and.(check_ground_last.eq.1)) then
c     Debug
         if (db.eq.1) then
            write(*,*) 'now checking intersection with ground'
         endif
c     Debug
         j=1
         call find_Pj(dim,surf(j),P1int,P2int,P3int,P4int,
     &        P5int,P6int,Pj)
         if ((intfg.eq.1).and.(surf(j).eq.1)) then
            intfg1=1
c     Debug
            if (db.eq.1) then
               write(*,*) 'P(j=',j,')=',(Pj(i),i=1,dim)
            endif
c     Debug
         else
            intfg1=0
         endif
         call check_validity(dim,app,intfg1,r1,r2,theta1,theta2,
     &        phi1,phi2,
     &        ca,cb,cc,Pstart,u,Pj,surf(j),valid,length(j),err(j))
c     Debug
         if (db.eq.1) then
            write(*,*) 'valid=',valid,' length=',length
         endif
c     Debug
         if (valid.eq.1) then
            if (length(j).gt.epsilon_length) then
               length_min=length(j)
               sol=j
               surface=surf(sol)
               solf=1
c     Debug
               if (db.eq.1) then
                  write(*,*) 'solution accepted'
               endif
c     Debug
            endif
         else
            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif
            write(10,*) 'Error from ',
     &        label(1:strlen(label)),':'
            write(10,*) 'even intersection with ground is invalid'
            close(10)
            aerr=1
            goto 666
         endif
      endif


 222  continue
      if ((error0.eq.1).and.(solf.eq.0).and.(n_sol.eq.1).and.
     &     (valid.eq.1).and.(length(1).eq.0.0D+0)) then
c     Debug

         call logfile_name(pindex,event,log_file)
         string='./nlines'
         call stringp_file_name(pindex,string,stringp_file)
         call get_nlines(log_file,stringp_file,nl)
         open(10,file=log_file(1:strlen(log_file)))
         do i=1,nl
            read(10,*)
         enddo
         if (nl.gt.0) then
            write(10,*)
         endif

         write(10,*) 'Warning from routine ',
     &        label(1:strlen(label)),':'
         write(10,*) 'very rare event occured'
         write(10,*) 'solf=',solf
         write(10,*) 'n_sol=',n_sol
         write(10,*) 'Pstart=',(Pstart(i),i=1,dim)
         write(10,*) 'u=',(u(k),k=1,dim)
         write(10,*) 'r1=',r1,' r2=',r2
         write(10,*) 'theta1=',theta1,' theta2=',theta2
         write(10,*) 'phi1=',phi1,' phi2=',phi2
         pos(1)=Pstart(2)
         pos(2)=Pstart(3)
         call ground_radius(pos,theta1,theta2,phi1,phi2,
     &        ca,cb,cc,g_radius)
         write(10,*) 'r_ground at (theta,phi)start:',g_radius
         call ground_normal(dim,Pstart,theta1,theta2,phi1,phi2,
     &        ca,cb,cc,n)
         call scalar_product(dim,u,n,prod)
         write(10,*) 'prod=',prod
         do j=1,n_sol
            call find_Pj(dim,surf(j),P1int,P2int,P3int,P4int,
     &           P5int,P6int,Pj)
            write(10,*) 'Pj=',(Pj(i),i=1,dim)
            call check_validity(dim,app,0,r1,r2,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,Pstart,u,Pj,surf(j),valid,length(j),err(j))
            write(10,*) 'valid(',j,')=',valid,' err(',j,')=',err(j)
            write(10,*) 'length=',length(j)
         enddo
c     Debug
c         call special_surface2(dim,r1,r2,theta1,theta2,phi1,phi2,
c     &        n_sol,aerr,solf,Pstart,u,P2int,surf,epsilon_prod,
c     &        Ptmp,surf_tmp)
c         call copy_vector(dim,Ptmp,P2int)
c         if (solf.eq.0) then
c     Debug
         write(10,*) 'no solution found, aborting path'
c            stop
c     Debug
c         else
c            sol=2
c            surf(sol)=surf_tmp
c            surface=surf_tmp
c     Debug
c            write(10,*) 'solution found'
c     Debug
c         endif
         close(10)
         aerr=1
         goto 666
      endif

c     intersection whose validity could not be determined
      if ((solf.eq.0).and.(solf_nv.eq.1)) then
         do j=1,nsol_nv
            if ((solf.eq.0).and.(length_nv(j).gt.0.0D+0)) then
               solf=1
               length_min=length_nv(j)
               sol=sol_nv(j)
               surface=surf(sol)
            else
               if ((length_nv(j).lt.length_min).and.
     &              (length_nv(j).gt.0.0D+0)) then
                  length_min=length_nv(j)
                  sol=sol_nv(j)
                  surface=surf(sol)
               endif
            endif
         enddo ! j
      endif

      if (solf.eq.1) then
         call find_Pj(dim,surf(sol),P1int,P2int,P3int,P4int,
     &        P5int,P6int,Pint)
      else
         if (n_sol.ge.1) then
            aerr=1
            do j=1,n_sol
               if (err(j).eq.0) then
                  aerr=0
                  goto 211
               endif
            enddo
 211        continue
         endif
         if ((aerr.eq.1).and.(error0.eq.1)) then
c     If solutions have been found, and all are faulty:
c     one possibility is that both "Pstart" and the intersection position are on surface 2,
c     with a propagation direction normal to the local normal, which results in an
c     output position very close to "Pstart"

            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif

c     Debug
            write(10,*) 'Warning from routine ',
     &           label(1:strlen(label)),':'
            write(10,*) 'surface 2 bug detected'
            write(10,*) 'n_sol=',n_sol,' solf=',solf
            
            write(10,*) 'Pstart=',(Pstart(k),k=1,dim)
            write(10,*) 'P2int=',(P2int(k),k=1,dim)
            do j=1,n_sol
               write(10,*) 'surf(',j,')=',surf(j)
            enddo
c     Debug
            call special_surface2(dim,app,intfg,r1,r2,theta1,theta2,
     &           phi1,phi2,ca,cb,cc,n_sol,aerr,solf,Pstart,u,P2int,
     &           surf,epsilon_prod,Pint,surface)
            if (solf.eq.0) then
               aerr=1
c     Debug
               write(10,*) 'no solution found, aborting path'
            else
               write(10,*) 'solution found'
c     Debug
            endif
            close(10)
            goto 666
            
         endif
         if ((error0.eq.1).and.(aerr.eq.0)) then
            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif
            write(10,*) label(1:strlen(label)),' aborting path, error 3'
            write(10,*) 'Error from ',label(1:strlen(label)),':'
            write(10,*) 'NO SOLUTION FOUND'
            write(10,*) 'n_sol=',n_sol
            write(10,*) 'solf=',solf
            write(10,*) 'Pstart=',(Pstart(i),i=1,dim)
            write(10,*) 'u=',(u(i),i=1,dim)
            do j=1,n_sol
               write(10,*) 'surf(',j,')=',surf(j)
               call find_Pj(dim,surf(j),P1int,P2int,P3int,P4int,
     &              P5int,P6int,Pj)
               write(10,*) 'Pj=',(Pj(i),i=1,dim)
               write(10,*) 'ca=',(ca(i),i=1,4)
               write(10,*) 'cb=',(cb(i),i=1,4)
               write(10,*) 'cc=',(cc(i),i=1,4)
               call check_validity(dim,app,0,r1,r2,theta1,theta2,
     &              phi1,phi2,ca,cb,cc,Pstart,u,Pj,surf(1),
     &              valid,length(j),err(j))
               write(10,*) 'err=',err(j),' length=',length(j)
               write(10,*) 'valid=',valid
            enddo ! j
            close(10)

            call alternative_intersection(dim,event,app,Pstart,u,
     &           r1,r2,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,
     &           solf_ai,Pint,surface)

            call logfile_name(pindex,event,log_file)
            string='./nlines'
            call stringp_file_name(pindex,string,stringp_file)
            call get_nlines(log_file,stringp_file,nl)
            open(10,file=log_file(1:strlen(log_file)))
            do i=1,nl
               read(10,*)
            enddo
            if (nl.gt.0) then
               write(10,*)
            endif

            if (solf_ai.eq.0) then
               solf=0
               aerr=1
               write(10,*) 'Alternative intersection method failed'
            else
               solf=1
               aerr=0
               write(10,*) 'Alternative intersection was successfull:'
               write(10,*) 'Pint=',(Pint(i),i=1,dim)
               write(10,*) 'surface=',surface
            endif
            close(10)
            goto 666
            
         endif ! error 3
      endif

c     Debug
      if (db.eq.1) then
         write(*,*) 'sol=',sol,' length_min=',length_min
      endif
c     Debug

 666  continue
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :out'
      endif
c     Debug
      return
      end




      subroutine alternative_intersection(dim,event,app,Pstart,u,
     &     r1,r2,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,
     &     solf,Pint,surface)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c
c     Purpose: to try an intersection with the current cell, using
c     alternative intersection methods (dichotomy).
c
c     Inputs:
c       + dim: dimensions of the vectors
c       + event: index of the event
c       + app: if equal to 1, an intersection with real ground must be looked for; 0 otherwise
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + r1,r2,theta1,theta2,phi1,phi2: variables that defines the box
c       + ca,cb,cc: a, b and c coefficients for the 4 quarters of the angular sector
c
c     Outputs:
c       + solf: 0 if no intersection found; 1 if an intersection was found
c       + Pint: (r,theta,phi) spherical coordinates of the intersection point, if any
c       + surface: index of the intersected surface
c
c     input
      integer dim,event,app
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
c     output
      integer solf
      double precision Pint(1:Nmat_mx)
      integer surface
c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     temp
      integer i
      integer intfg,intf,err_code
      double precision Pground(1:Nmat_mx)
      character*100 log_file
      character*100 string
      character*100 stringp_file
      integer nl
      integer strlen
      character*100 label
c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH
      label='subroutine alternative_intersection'
      
      solf=0
      call logfile_name(pindex,event,log_file)
      string='./nlines'
      call stringp_file_name(pindex,string,stringp_file)
      call get_nlines(log_file,stringp_file,nl)
      open(10,file=log_file(1:strlen(log_file)))
      do i=1,nl
         read(10,*)
      enddo
      if (nl.gt.0) then
         write(10,*)
      endif
      write(10,*) 'This is routine ',label(1:strlen(label)),' :'
      write(10,*)
      write(10,*) 'Now trying modified intersection with ground'
      call intersect_ground(dim,2,Pstart,u,
     &     r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,
     &     intfg,Pground)
      write(10,*) 'intfg=',intfg
      if (intfg.eq.0) then
         write(10,*) 'Even this failed !'
         write(10,*)
         write(10,*) 'Now trying modified cell intersection'
         close(10)

         call intersect_cell(dim,event,app,Pstart,u,
     &        r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,
     &        intf,err_code,Pint,surface)
         
         call logfile_name(pindex,event,log_file)
         string='./nlines'
         call stringp_file_name(pindex,string,stringp_file)
         call get_nlines(log_file,stringp_file,nl)
         open(10,file=log_file(1:strlen(log_file)))
         do i=1,nl
            read(10,*)
         enddo
         if (nl.gt.0) then
            write(10,*)
         endif
         
         if (intf.eq.0) then
            write(10,*) 'It failed too... I give up.'
            write(10,*) 'err_code=',err_code
            close(10)
            goto 666
         else
            write(10,*) 'Intersection was found:'
            write(10,*) 'Pint=',(Pint(i),i=1,dim)
            write(10,*) 'surface=',surface
            close(10)
            solf=1
            goto 666
         endif
      else
         write(10,*) 'success:'
         write(10,*) 'Pground=',(Pground(i),i=1,dim)
         close(10)
         solf=1
         surface=1
         call copy_vector(dim,Pground,Pint)
         goto 666
      endif                     ! intfg

 666  continue
      return
      end




      subroutine intersect_ground(dim,gmethod,Pstart,u0,
     &     r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,
     &     intf,Pint)
      implicit none
      include 'max.inc'
c
c     Purpose: to find the intersection (if any) between a line defined
c     by a position and a direction, and the "real ground" defined by four
c     "spherical planes" that are defined by three coefficients a, b and c
c
c     Inputs:
c       + dim: dimension of vectors
c       + gmethod: option for choosing the method used in the detection of intersection with ground
c         - gmethod=1: standard Newton method
c         - gmethod=2: raw dichotomy procedure
c       + Pstart: (r,theta,phi) spherical coordinates of the line starting position
c       + u0: line propagation direction
c       + r1,r2,theta1,theta2,phi1,phi2: limits of the spherical box
c       + ca,cb,cc: a, b and c coefficients for the 4 quarters of the angular sector
c
c     Outputs:
c       + intf: 1 if an intersection was found; 0 otherwise
c       + Pint: (r,theta,phi) spherical coordinates of the intersection position, if any
c

c     input
      integer dim,gmethod
      double precision Pstart(1:Nmat_mx)
      double precision u0(1:Nmat_mx)
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     output
      integer intf
      double precision Pint(1:Nmat_mx)
c     temp
      double precision P(1:Nmat_mx)
      double precision pos(1:2)
      double precision Pint1(1:Nmat_mx)
      double precision Pint2(1:Nmat_mx)
      double precision Pint3(1:Nmat_mx)
      double precision Pint4(1:Nmat_mx)
      integer i,ns,ind(1:4),sol
      integer q,quarter,intfq(1:4),inc,inb,sol1f,sol1,lmf
      integer err_code(1:4)
      double precision length(1:4),length_min
      double precision g_radius
      integer strlen
      character*100 label
      label='subroutine intersect_ground'

      ns=0

      if (gmethod.eq.1) then
         call intersect_spherical_plane(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(1),cb(1),cc(1),
     &        intfq(1),Pint1)
c     Debug
c         write(*,*) 'intfq(1)=',intfq(1)
c         write(*,*) 'Pint1=',(Pint1(i),i=1,dim)
c     stop
c     Debug
         call intersect_spherical_plane(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(2),cb(2),cc(2),
     &        intfq(2),Pint2)
c     Debug
c         write(*,*) 'intfq(2)=',intfq(2)
c         write(*,*) 'Pint2=',(Pint2(i),i=1,dim)
c     Debug
         call intersect_spherical_plane(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(3),cb(3),cc(3),
     &        intfq(3),Pint3)
c     Debug
c         write(*,*) 'intfq(3)=',intfq(3)
c         write(*,*) 'Pint3=',(Pint3(i),i=1,dim)
c     Debug
         call intersect_spherical_plane(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(4),cb(4),cc(4),
     &        intfq(4),Pint4)
c     Debug
c         write(*,*) 'intfq(4)=',intfq(4)
c         write(*,*) 'Pint4=',(Pint4(i),i=1,dim)
c     Debug
      else if (gmethod.eq.2) then
         call intersect_spherical_plane2(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(1),cb(1),cc(1),
     &        intfq(1),err_code(1),Pint1)
         call intersect_spherical_plane2(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(2),cb(2),cc(2),
     &        intfq(2),err_code(2),Pint2)
         call intersect_spherical_plane2(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(3),cb(3),cc(3),
     &        intfq(3),err_code(3),Pint3)
         call intersect_spherical_plane2(dim,Pstart,u0,
     &        r1,r2,theta1,theta2,phi1,phi2,ca(4),cb(4),cc(4),
     &        intfq(4),err_code(4),Pint4)
      else
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),':'
         write(*,*) 'Bad input argument:'
         write(*,*) 'gmethod=',gmethod
         stop
      endif

      do quarter=1,4
         if (intfq(quarter).eq.1) then
c     Debug
c            write(*,*) 'intfq(',quarter,')=',intfq(quarter)
c     Debug
            if (quarter.eq.1) then
               call copy_vector(dim,Pint1,P)
            else if (quarter.eq.2) then
               call copy_vector(dim,Pint2,P)
            else if (quarter.eq.3) then
               call copy_vector(dim,Pint3,P)
            else if (quarter.eq.4) then
               call copy_vector(dim,Pint4,P)
            endif
            call is_in_column(P,theta1,theta2,phi1,phi2,inc)
c     Debug
c            write(*,*) 'P=',(P(i),i=1,dim)
c            write(*,*) 'inc=',inc
c     Debug
            if (inc.eq.1) then
               pos(1)=P(2)
               pos(2)=P(3)
               call identify_quarter(pos,theta1,theta2,phi1,phi2,q)
c     Debug
c               write(*,*) 'q=',q
c     Debug
               if (q.eq.quarter) then
                  ns=ns+1
                  ind(ns)=quarter
               endif            ! quarter
            endif               ! in column
         endif                  ! intfq=1
      enddo                     ! quarter
         
      if (ns.eq.0) then
         intf=0
      else if (ns.eq.1) then
         intf=1
         if (ind(ns).eq.1) then
            call copy_vector(dim,Pint1,Pint)
         else if (ind(ns).eq.2) then
            call copy_vector(dim,Pint2,Pint)
         else if (ind(ns).eq.3) then
            call copy_vector(dim,Pint3,Pint)
         else if (ind(ns).eq.4) then
            call copy_vector(dim,Pint4,Pint)
         endif
c     Debug
c         write(*,*) 'ns=',ns
c         write(*,*) 'ind(',ns,')=',ind(ns)
c         write(*,*) label(1:strlen(label)),' Pint=',(Pint(i),i=1,dim)
c         call reset_onground(Pint,theta1,theta2,phi1,phi2,ca,cb,cc)
c     Debug
      else
c     Debug
c         write(*,*) 'Error from ',
c     &        label(1:strlen(label)),':'
c         write(*,*) 'Number of solutions:',ns
c         write(*,*) 'Pstart=',(Pstart(i),i=1,dim)
c         write(*,*) 'u0=',(u0(i),i=1,dim)
c     Debug
         do sol=1,ns
c     Debug
c            write(*,*) 'ind(',sol,')=',ind(sol)
c     Debug
            if (ind(sol).eq.1) then
               call copy_vector(dim,Pint1,P)
c     Debug
c               write(*,*) 'Pint1=',(Pint1(i),i=1,dim)
c     Debug
            else if (ind(sol).eq.2) then
               call copy_vector(dim,Pint2,P)
c     Debug
c               write(*,*) 'Pint2=',(Pint2(i),i=1,dim)
c     Debug
            else if (ind(sol).eq.3) then
               call copy_vector(dim,Pint3,P)
c     Debug
c               write(*,*) 'Pint3=',(Pint3(i),i=1,dim)
c     Debug
            else if (ind(sol).eq.4) then
               call copy_vector(dim,Pint4,P)
c     Debug
c               write(*,*) 'Pint4=',(Pint4(i),i=1,dim)
c     Debug
            endif
            pos(1)=P(2)
            pos(2)=P(3)
c            call ground_radius(pos,theta1,theta2,phi1,phi2,
c     &           ca,cb,cc,g_radius)
c            write(*,*) 'g_radius=',g_radius
            call distance_spher(label,Pstart(1),Pstart(2),Pstart(3),
     &           P(1),P(2),P(3),length(sol))
c            write(*,*) 'length(',sol,')=',length(sol)
         enddo
         lmf=0
         do sol=1,ns
            if (length(sol).gt.0.0D+0) then
               lmf=1
               length_min=length(sol)
               goto 222
            endif
         enddo ! sol
 222     continue
         if (lmf.eq.0) then
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),':'
            write(*,*) 'Number of solutions:',ns
            do sol=1,ns
               write(*,*) 'length(',sol,')=',length(sol)
            enddo ! sol
            stop
         endif
         sol1f=0
         do sol=1,ns
            if ((length(sol).gt.0.0D+0).and.
     &           (length(sol).le.length_min)) then
               sol1f=1
               sol1=sol
               length_min=length(sol)
            endif
         enddo ! sol
         if (sol1f.eq.0) then ! no solution found
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),':'
            write(*,*) 'Number of solutions:',ns
            write(*,*) 'none adequate:'
            write(*,*) 'length_min=',length_min
            do sol=1,ns
               write(*,*) 'length(',sol,')=',length(sol)
            enddo ! sol
            stop
         else ! unique solution was found
            intf=1
            if (ind(sol1).eq.1) then
               call copy_vector(dim,Pint1,Pint)
            else if (ind(sol1).eq.2) then
               call copy_vector(dim,Pint2,Pint)
            else if (ind(sol1).eq.3) then
               call copy_vector(dim,Pint3,Pint)
            else if (ind(sol1).eq.4) then
               call copy_vector(dim,Pint4,Pint)
            endif
            call reset_onground(Pint,theta1,theta2,phi1,phi2,ca,cb,cc)
         endif
      endif

      return
      end

      




      subroutine special_surface2(dim,app,intfg,r1,r2,theta1,theta2,
     &     phi1,phi2,ca,cb,cc,n_sol,aerr,solf,Pstart,u,P2int,surf,
     &     epsilon_prod,Pint,surface)
      implicit none
      include 'max.inc'
c
c     Purpose: try to find a solution in the special case of a surface 2 error
c
c     Inputs:
c       + dim: dimension of vectors
c       + app: if 1, rela ground surface must be computed
c       + intfg: if equal to 1, an intersection has been found with the real ground surface
c       + r1, theta1, phi1, r2, theta2, phi2: limits of the spherical box
c       + ca,cb,cc: a, b and c coefficients for the 4 quarters of the angular sector
c       + n_sol: number of solutions found
c       + aerr= error code
c       + solf: 0: no solution found; 1: a solution was found
c       + Pstart: starting position
c       + u: propagation direction
c       + P2int: intersection position with surface 2
c       + surf: array of surface indexes
c       + epsilon_prod: value of the scalar product below which
c                       a surface 2 error is encountered
c
c     Outputs:
c       + aerr= error code
c       + solf: 0: no solution found; 1: a solution was found
c       + Pint: corrected intersection position
c       + surface: intersection surface index
c

c     input
      integer dim,app,intfg,Nc
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      integer n_sol,aerr,solf
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision P2int(1:Nmat_mx)
      integer surf(1:6)
      double precision epsilon_prod
c     output
      double precision Pint(1:Nmat_mx)
c     temp
      integer strlen
      integer ons,valid,surface,k
      integer surf_Pout,surf_Pstart,err_Pout
      double precision n(1:Nmat_mx)
      double precision prod,d,length
      double precision Pout(1:Nmat_mx)
      character*100 label
c     Debug
      double precision un(1:Nmat_mx)
      double precision poss(1:Nmat_mx)
      integer nk,n_int
      double precision k0,k1,k2
      double precision a,b,c
      double precision P0(1:Nmat_mx)
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision poss2(1:Nmat_mx)
      double precision seg(1:Nmat_mx)
      double precision segn(1:Nmat_mx)
      double precision lseg
c     Debug
      label='subroutine special_surface2'

c     Debug
      call normalize_vector(dim,u,un)
      call spher2cart(label,1,Pstart(1),Pstart(2),Pstart(3),
     &     poss(1),poss(2),poss(3))
      write(*,*) 'Position P1:',(Pstart(k),k=1,dim)
      write(*,*) 'Position poss1:',(poss(k),k=1,dim)
      write(*,*) 'direction u:',(un(k),k=1,dim)
      a=un(1)**2.0D+0+un(2)**2.0D+0+un(3)**2.0D+0
      b=2.0D+0*(un(1)*poss(1)+un(2)*poss(2)+un(3)*poss(3))
      c=poss(1)**2.0D+0+poss(2)**2.0D+0+poss(3)**2.0D+0-r2**2.0D+0
      write(*,*) 'a=',a
      write(*,*) 'b=',b
      write(*,*) 'c=',c
      call eq2deg(a,b,c,nk,k0,k1,k2)
      write(*,*) 'nk=',nk
      if (nk.eq.1) then
         write(*,*) 'k0=',k0
      else if (nk.eq.2) then
         write(*,*) 'k1=',k1
         write(*,*) 'k2=',k2
      endif
      call scalar_product(dim,u,n,prod)
      write(*,*) 'u=',(u(k),k=1,dim)
      write(*,*) 'n=',(n(k),k=1,dim)
      write(*,*) 'prod=',prod
      write(*,*) '2*r2*cos(alpha)=',2.0D+0*r2*prod
      call line_sphere_intersect(dim,Pstart,u,r2,n_int,
     &     P0,k0,P1,k1,P2,k2)
      write(*,*) 'n_int=',n_int
      if (n_int.eq.1) then
         write(*,*) 'P0=',(P0(k),k=1,dim)
      else if (n_int.eq.2) then
         write(*,*) 'P1=',(P1(k),k=1,dim)
         write(*,*) 'P2=',(P2(k),k=1,dim)
      endif
      call spher2cart(label,1,P2(1),P2(2),P2(3),
     &     poss2(1),poss2(2),poss2(3))
      call substract_vectors(dim,poss2,poss,seg)
      call normalize_vector(dim,seg,segn)
      write(*,*) 'segn=',(segn(k),k=1,dim)
      call vector_length(dim,seg,lseg)
      write(*,*) 'length(seg)=',lseg
c     Debug

      if (n_sol.ne.1) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),':'
         write(*,*) 'aerr=',aerr,' with:'
         write(*,*) 'n_sol=',n_sol
         goto 666
      endif
      call is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &     Pstart,ons,surf_Pstart)
      if ((ons.eq.0).or.(surf_Pstart.ne.2)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),':'
         write(*,*) 'aerr=',aerr,' with:'
         write(*,*) 'ons=',ons
         write(*,*) 'surf_Pstart=',surf_Pstart
         goto 666
      endif
      call inside_normal(surf(1),P2int,n)
      call scalar_product(dim,u,n,prod)
      if (prod.gt.epsilon_prod) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),':'
         write(*,*) 'aerr=',aerr,' with:'
         write(*,*) 'prod=',prod
         goto 666
      endif
      d=2.0D+0*r2*prod
      call jump_to(dim,Pstart,u,d,Pout)
      call is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &     Pout,ons,surf_Pout)
      if ((ons.eq.0).or.(surf_Pout.ne.2)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),':'
         write(*,*) 'aerr=',aerr,' with:'
         write(*,*) 'ons=',ons
         write(*,*) 'surf_Pout=',surf_Pout
         write(*,*) 'Pstart=',(Pstart(k),k=1,dim)
         write(*,*) 'Pout=',(Pout(k),k=1,dim)
         goto 666
      endif
      call check_validity(dim,app,0,r1,r2,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,Pstart,u,Pout,surf_Pout,valid,length,err_Pout)
      if (valid.eq.0) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),':'
         write(*,*) 'aerr=',aerr,' with:'
         write(*,*) 'valid=',valid
         goto 666
      endif
      solf=1
      surface=surf_Pout
      call copy_vector(dim,Pout,Pint)
      aerr=0

 666  continue
      return
      end




      subroutine check_validity(dim,app,intfg,
     &     r1,r2,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,
     &     Pstart,u,Pint,surface,valid,length,err)
      implicit none
      include 'max.inc'
c
c     Purpose: to check the validity of the input intersection;
c     An intersection is considered as valid if the intersection position is
c     on the intersected surface, and if the propagation direction "u" and
c      the vector between the starting position and the intersection position
c     are colinear.
c
c     Inputs:
c       + dim: dimension of vectors
c       + app: if 1, the real surface at ground level must be computed
c       + intfg: if equal to 1, an intersection has been found with the real ground surface
c       + r1,r2,theta1,theta2,phi1,phi2: variables that define the current box
c       + ca,cb,cc: a, b and c coefficients for the 4 quarters of the angular sector
c       + Pstart: (r,theta,phi) spherical coordinates of the starting position
c       + u: direction of propagation
c       + Pint: (r,theta,phi) spherical coordinates of the intersection position
c       + surface: index of the intersected surface
c
c     Outputs:
c       + valid:
c         - -1: validity could not be determined
c         - 0: intersection is not valid
c         - 1: intersection is valid
c       + length: distance between the starting position and the intersection
c       + err: an error occured
c         - err=0: no known error
c         - err=1: an error occured
c

c     input
      integer dim,app,intfg
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision Pint(1:Nmat_mx)
      integer surface
c     output
      integer valid ! 0: solution is not valid; 1: solution is valid
      double precision length
      integer err
c     temp
      integer i,in,col
      double precision Pstartc(1:Nmat_mx)
      double precision Pintc(1:Nmat_mx)
      double precision seg(1:Nmat_mx)
      double precision d,epsilon_d,dist
      double precision pos(1:2)
      double precision r_ground
      double precision eps_r,eps_theta,eps_phi
c      double precision length_min
      integer strlen
      character*100 label
c      parameter(length_min=1.0D-1) ! m
      label='subroutine check_validity'
      epsilon_d=1.0D-11*r1 ! m
      eps_r=1.0D-11*r1 ! m
      eps_theta=1.0D-7
      eps_phi=1.0D-7

c     strange bugs...
      if (Pint(1).lt.0.0D+0) then
         valid=0
         goto 123
      endif

c     checking validity of solution
      if (surface.eq.1) then
         if (intfg.eq.0) then
            call is_on_surface_r(r1,theta1,theta2,phi1,phi2,Pint,
     &           in,dist)
         else if (intfg.eq.1) then
            call is_on_ground(Pint,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,in)
c     Debug
            if (in.eq.0) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'intfg=',intfg
               write(*,*) 'in=',in
               write(*,*) 'Pint=',(Pint(i),i=1,dim)
               pos(1)=Pint(2)
               pos(2)=Pint(3)
               call ground_radius(pos,theta1,theta2,phi1,phi2,
     &              ca,cb,cc,r_ground)
               write(*,*) 'r_ground=',r_ground
c               write(*,*) 'Pint=',(Pint(i),i=1,dim)
c               write(*,*) Pint(1),r_ground,Pint(1)-r_ground
               write(*,*)' difference:',dabs(Pint(1)-r_ground)
               write(*,*) 'r1=',r1,' r2=',r2
               stop
            endif
c     Debug
         else
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'Bad input argument:'
            write(*,*) 'intfg=',intfg
            stop
         endif
      else if (surface.eq.2) then
         call is_on_surface_r(r2,theta1,theta2,phi1,phi2,Pint,
     &        in,dist)
      else if (surface.eq.3) then
         if (app.eq.0) then
            call is_on_surface_theta(r1,r2,theta1,theta2,phi1,Pint,
     &           in,dist)
         else if (app.eq.1) then
            pos(1)=Pint(2)
            pos(2)=Pint(3)
            call ground_radius(pos,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,r_ground)
            if ((dabs(Pint(3)-phi1).lt.eps_phi).and.
     &           (Pint(2).ge.theta1).and.(Pint(2).le.theta2)
     &           .and.(Pint(1).ge.r_ground-1.0D-4).and.
     &           (Pint(1).le.r2)) then
               in=1
            else
               in=0
            endif
         endif
      else if (surface.eq.4) then
         if (app.eq.0) then
         call is_on_surface_theta(r1,r2,theta1,theta2,phi2,Pint,
     &        in,dist)
         else if (app.eq.1) then
            pos(1)=Pint(2)
            pos(2)=Pint(3)
            call ground_radius(pos,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,r_ground)
            if ((dabs(Pint(3)-phi2).lt.eps_phi).and.
     &           (Pint(2).ge.theta1).and.(Pint(2).le.theta2)
     &           .and.(Pint(1).ge.r_ground-1.0D-4).and.
     &           (Pint(1).le.r2)) then
               in=1
            else
               in=0
            endif
         endif
      else if (surface.eq.5) then
         if (app.eq.0) then
         call is_on_surface_phi(r1,r2,theta1,phi1,phi2,Pint,
     &        in,dist)
         else if (app.eq.1) then
            pos(1)=Pint(2)
            pos(2)=Pint(3)
            call ground_radius(pos,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,r_ground)
            if ((dabs(Pint(2)-theta1).lt.eps_theta).and.
     &           (Pint(3).ge.phi1).and.(Pint(3).le.phi2)
     &           .and.(Pint(1).ge.r_ground-1.0D-4).and.
     &           (Pint(1).le.r2)) then
               in=1
            else
               in=0
            endif
         endif
      else if (surface.eq.6) then
         if (app.eq.0) then
         call is_on_surface_phi(r1,r2,theta2,phi1,phi2,Pint,
     &        in,dist)
         else if (app.eq.1) then
            pos(1)=Pint(2)
            pos(2)=Pint(3)
            call ground_radius(pos,theta1,theta2,phi1,phi2,
     &           ca,cb,cc,r_ground)
            if ((dabs(Pint(2)-theta2).lt.eps_theta).and.
     &           (Pint(3).ge.phi1).and.(Pint(3).le.phi2)
     &           .and.(Pint(1).ge.r_ground-1.0D-4).and.
     &           (Pint(1).le.r2)) then
               in=1
            else
               in=0
            endif
         endif
      endif

c     2009-01-16
c     I remove the colinearity check because:
c     1) it never helped solving a problem
c     2) it creates bugs
c     Debug
c      write(*,*) 'Pstart(1)=',Pstart(1)
c     Debug
      call spher2cart(label,1,Pstart(1),Pstart(2),Pstart(3),
     &     Pstartc(1),Pstartc(2),Pstartc(3))
c     Debug
c      write(*,*) 'Pint(1)=',Pint(1)
c     Debug
      call spher2cart(label,1,Pint(1),Pint(2),Pint(3),
     &     Pintc(1),Pintc(2),Pintc(3))
      call substract_vectors(dim,Pintc,Pstartc,seg)
      call vector_length(dim,seg,length)
cc     if vector "seg" has a very small length, it means that positions "Pstart"
cc     and "Pint" are very close: probably a very small path. In this case, it
cc     is not possible to determine if vectors "seg" and "u" are colinear
c      if (length.gt.length_min) then
c         call are_colinear(dim,u,seg,col)
c      else
c         valid=-1
c         err=0
c         goto 123
c      endif

c      if ((in.eq.1).and.(col.eq.1)) then
      if (in.eq.1) then
         valid=1
         err=0
      else
         valid=0
c         if ((in.eq.1).and.(col.eq.0).and.(length.lt.epsilon_d)) then
         if (length.lt.epsilon_d) then
c     This is a known error: when "Pint" is too close from "Pstart" to compute colinearity
            err=1
         else
            err=0
         endif
      endif
 123  continue

      return
      end



      subroutine is_on_ground(P,theta1,theta2,phi1,phi2,
     &     ca,cb,cc,ong)
      implicit none
      include 'max.inc'
c
c     Purpose: to determine whether or not the provided position
c     is on the ground
c
c     Inputs:
c       + P: (r,theta,phi) spherical coordinates of the position
c       + theta1,theta2,phi1,phi2: variables that define the current box
c       + ca,cb,cc: a, b and c coefficients for each quarter
c
c     Outputs:
c       + ong:
c         - ong=0: "P" is NOT on the surface of the ground
c         - ong=1: "P" is on the surface of the ground
c     
c     input
      double precision P(1:Nmat_mx)
      double precision theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     output
      integer ong
c     temp
      double precision radius
      double precision pos(1:2)
      double precision err_r_max
      integer strlen
      character*100 label
      label='subroutine is_on_ground'

      err_r_max=1.0D-2 ! m
      ong=1

      pos(1)=P(2)
      pos(2)=P(3)
      call ground_radius(pos,theta1,theta2,phi1,phi2,ca,cb,cc,radius)
      if (dabs(P(1)-radius).gt.err_r_max) then
         ong=0
      endif

      return
      end



      subroutine is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &     P,ons,surface)
      implicit none
      include 'max.inc'
c
c     Purpose: to determine if the provided position "P" is on the
c     surface of the box
c
c     Inputs:
c       + dim: dimension of vectors
c       + r1,r2,theta1,theta2,phi1,phi2: variables that define the current box
c       + P: (r,theta,phi) spherical coordinates of the position
c
c     Outputs:
c       + ons:
c         - ons=0: "P" is NOT on the surface of the box
c         - ons=1: "P" is on the surface of the box
c       + surface: index of the surface position "P" is located on
c

c     input
      integer dim
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision P(1:Nmat_mx)
c     output
      integer ons
      integer surface
c     temp
      integer i,strlen,minf
      integer inv,nsol
      integer ins(1:6)
      double precision dist(1:6),min
      character*100 label
      label='subroutine is_on_surface'
      
      ons=0
      call is_in_box(P,r1,r2,theta1,theta2,phi1,phi2,inv)
      if (inv.eq.0) then
         ons=0
         goto 666
      else
         call is_on_surface_r(r1,theta1,theta2,phi1,phi2,P,
     &        ins(1),dist(1))
         call is_on_surface_r(r2,theta1,theta2,phi1,phi2,P,
     &        ins(2),dist(2))
         call is_on_surface_theta(r1,r2,theta1,theta2,phi1,P,
     &        ins(3),dist(3))
         call is_on_surface_theta(r1,r2,theta1,theta2,phi2,P,
     &        ins(4),dist(4))
         call is_on_surface_phi(r1,r2,theta1,phi1,phi2,P,
     &        ins(5),dist(5))
         call is_on_surface_phi(r1,r2,theta2,phi1,phi2,P,
     &        ins(6),dist(6))
         nsol=0
         do i=1,6
            nsol=nsol+ins(i)
         enddo
         if (nsol.eq.0) then
c     position "P" is not on a surface of the cell
            ons=0
         else if (nsol.eq.1) then
c     identify the only surface position "P" belongs to
            do i=1,6
               if (ins(i).eq.1) then
                  ons=1
                  surface=i
                  goto 666
               endif            ! ins(i)=1
            enddo               ! i
         else if (nsol.gt.1) then
c     identify the closest surface from position "P'
            ons=1
c     Debug
c            write(*,*) 'nsol=',nsol
c            do i=1,6
c               write(*,*) 'dist(',i,')=',dist(i)
c            enddo
c     Debug
            minf=0
            do i=1,6
               if ((ins(i).eq.1).and.(minf.eq.0)) then
                  minf=1
                  min=dist(i)
                  surface=i
               else if ((ins(i).eq.1).and.(minf.eq.1)) then
                  if (dist(i).lt.min) then
                     min=dist(i)
                     surface=i
                  endif
               endif
            enddo ! i
            if (minf.eq.0) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'no candidate surface was found'
               stop
            endif
         else
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'nsol=',nsol
            stop
         endif
      endif ! inv=0
      
 666  continue
      return
      end



      subroutine is_in_column(P,theta1,theta2,phi1,phi2,inc)
      implicit none
      include 'max.inc'
c
c     Purpose: to check whether or not the provided position resides within
c     the specified column
c
c     Inputs:
c       + p: (r,theta,phi) spherical coordinates of the position
c       + theta1,theta2,phi1,phi2: limits of the column
c     
c     Outputs:
c       + inc: 1 if position is in column; 0 otherwise
c

c     input
      double precision P(1:Nmat_mx)
      double precision theta1,theta2,phi1,phi2
c     output
      integer inc

      if ((P(2).ge.theta1).and.(P(2).le.theta2)
     &     .and.(P(3).ge.phi1).and.(P(3).le.phi2)) then
         inc=1
      else
         inc=0
      endif

      return
      end




      subroutine set_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &     P_in,surface,P_out)
      implicit none
      include 'max.inc'
c
c     Purpose: to set input position on specified surface. This routine is used in the
c     case a scattering position has been detected very close to a surface, and is
c     therefore considered as a path ending position
c
c     Inputs:
c       + dim: dimension of vectors
c       + r1,r2,theta1,theta2,phi1,phi2: variables that define the current cell
c       + P_in: (r,theta,phi) spherical coordinates of the input position
c       + surface: index of the surface position "P_in" has been detected on
c
c     Outputs:
c       + P_out: (r,theta,phi) spherical coordinated of the adjusted position
c

c     input
      integer dim
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision P_in(1:Nmat_mx)
      integer surface
c     output
      double precision P_out(1:Nmat_mx)
c     temp
      integer i
      
      do i=1,dim
         P_out(i)=P_in(i)
      enddo
      if (surface.eq.1) then
         P_out(1)=r1
      else if (surface.eq.2) then
         P_out(1)=r2
      else if (surface.eq.3) then
         P_out(3)=phi1
      else if (surface.eq.4) then
         P_out(3)=phi2
      else if (surface.eq.5) then
         P_out(2)=theta1
      else if (surface.eq.6) then
         P_out(2)=theta2
      endif
         
      
 666  continue
      return
      end


      

      subroutine find_Pj(dim,surface,P1int,P2int,P3int,
     &     P4int,P5int,P6int,Pj)
      implicit none
      include 'max.inc'
      integer dim,surface
      double precision P1int(1:Nmat_mx)
      double precision P2int(1:Nmat_mx)
      double precision P3int(1:Nmat_mx)
      double precision P4int(1:Nmat_mx)
      double precision P5int(1:Nmat_mx)
      double precision P6int(1:Nmat_mx)
      double precision Pj(1:Nmat_mx)

      if (surface.eq.1) then
         call copy_vector(dim,P1int,Pj)
      else if (surface.eq.2) then
         call copy_vector(dim,P2int,Pj)
      else if (surface.eq.3) then
         call copy_vector(dim,P3int,Pj)
      else if (surface.eq.4) then
         call copy_vector(dim,P4int,Pj)
      else if (surface.eq.5) then
         call copy_vector(dim,P5int,Pj)
      else if (surface.eq.6) then
         call copy_vector(dim,P6int,Pj)
      else
         write(*,*) 'Error from Pj:'
         write(*,*) 'Bad input parameter surface=',surface
         stop
      endif

      return
      end



      subroutine line_surface_phi_intersect(db,dim,app,Pstart,u,
     &     r1,r2,theta,theta1,theta2,phi1,phi2,ca,cb,cc,intf,Pint)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the intersection point between a line
c      and a phi-type surface (theta=cst)
c
c     Inputs
c       + dim: dimensions of the vectors
c       + app: if 1, real surface at ground level must be taken into account
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + r1,r2,theta,phi1,phi2: variables that defines the theta-type surface
c       + theta1,theta2,ca,cb,cc: to compute ground radius
c     
c     Outputs:
c       + intf: 0 if no intersection found; 1 if an intersection was found
c       + Pint: (r,theta,phi) spherical coordinates of the intersection point, if any
c

c     input
      integer dim,app
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision r1,r2,theta,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     output
      integer intf
      double precision Pint(1:Nmat_mx)
c     temp
      integer strlen
      double precision epsilon,epsilon_d,d,dist
      integer n_int,in
      double precision P0(1:Nmat_mx),k0
      double precision P1(1:Nmat_mx),k1
      double precision P2(1:Nmat_mx),k2
      character*100 label
      double precision pos(1:2),r_ground
c     Debug
      integer j,db
c     Debug
      parameter(epsilon=1.0D-6)
      parameter(epsilon_d=1.0D-5)
      label='subroutine line_surface_phi_intersect'

      call line_cone_intersect(db,dim,Pstart,u,theta,n_int,
     &     P0,k0,P1,k1,P2,k2)
c     Debug
      if (db.eq.1) then
         write(*,*) 'This is line_surface_phi_intersect'
         write(*,*) 'n_int=',n_int
         write(*,*) 'k0=',k0
      endif
c     Debug

      if (n_int.eq.0) then
         intf=0
      else if (n_int.eq.1) then
         if (k0.gt.epsilon) then
            intf=1
            call copy_vector(dim,P0,Pint)
         else if (k0.lt.epsilon) then
            intf=0
         else
            write(*,*) 'Error from ',label(1:strlen(label)),':'
            write(*,*) 'k0=',k0
            stop
         endif
      else if (n_int.eq.2) then
         if ((k1.gt.epsilon).and.(k2.lt.epsilon)) then
            intf=1
            call copy_vector(dim,P1,Pint)
         else if ((k1.lt.epsilon).and.(k2.gt.epsilon)) then
            intf=1
            call copy_vector(dim,P2,Pint)
         else if ((k1.lt.epsilon).and.(k2.lt.epsilon)) then
            intf=0
         else if ((k1.gt.epsilon).and.(k2.gt.epsilon)) then
            if (k2.lt.k1) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),':'
               write(*,*) '2 solutions found with:'
               write(*,*) 'k1=',k1,' k2=',k2
               stop
            else
               intf=1
               call copy_vector(dim,P1,Pint)
            endif
         else 
c     2009/04/02
c     Do not generate an error in this case: this is probably because propagation
c     direction is [0,0,-1], and no intersection is found between the line and
c     any vertical plane
c            write(*,*) 'Error from ',label(1:strlen(label)),':'
c            write(*,*) 'k1=',k1,' k2=',k2
c            stop
            intf=0
c     2009/04/02
         endif
      endif

 123  continue
      if (intf.eq.1) then
         if (app.eq.0) then
c     once an intersection is found, it is checked to be on the required surface
            call is_on_surface_phi(r1,r2,theta,phi1,phi2,Pint,in,dist)
c     if it is not, then the intersection is not valid
            if (in.eq.0) then
c     Debug
c            write(*,*) 'theta=',theta,' solution not on surface'
c            write(*,*) 'Pint=',(Pint(j),j=1,dim)
c     Debug
               intf=0
c     Debug
               if (db.eq.1) then
                  write(*,*) 'not on surface'
               endif
c     Debug
c         else
c     if the intersection is on the required surface, its distance to the starting position
c     must be checked
c            call distance_spher(label,Pstart(1),Pstart(2),Pstart(3),
c     &           Pint(1),Pint(2),Pint(3),d)
c            if (d.lt.epsilon_d) then
c               intf=0
c            endif
            else if (app.eq.1) then
               pos(1)=Pstart(2)
               pos(2)=Pstart(3)
               call ground_radius(pos,theta1,theta2,phi1,phi2,
     &              ca,cb,cc,r_ground)
               if ((Pstart(3).lt.phi1).or.(Pstart(3).gt.phi2).or.
     &              (Pstart(1).lt.r_ground)) then
                  intf=0
               endif
            endif !app
         endif
      endif

      if (intf.eq.1) then
c     in any case, if an intersection was found, P(2) has to be adjusted to the required theta
         Pint(2)=theta
      endif

      return
      end



      subroutine is_on_surface_phi(r1,r2,theta,phi1,phi2,P,in,dist)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to check if the provided position "P" is on
c     the provided phi-type surface
c
c     Inputs:
c       + r1,r2,theta,phi1,phi2: variables that defines the phi-type surface
c       + P: (r,theta,phi) spherical coordinates of the point on the surface
c
c     Outputs:
c       + in:
c         - in=0: "P" is not on surface
c         - in=1: "P" is on surface
c       + dist: distance to surface
c

c     input
      double precision r1,r2,theta,phi1,phi2
      double precision P(1:Nmat_mx)
c     output
      integer in
      double precision dist
c     temp
      double precision epsilon
      parameter(epsilon=1.0D-5)

      if ((P(1).ge.r1).and.(P(1).le.r2)
     &     .and.(P(3).ge.phi1).and.(P(3).le.phi2)
     &     .and.(dabs(dabs(P(2))-dabs(theta)).lt.epsilon)) then
         in=1
         dist=P(1)*dabs(dabs(P(2))-dabs(theta))
      else
         in=0
      endif

      return
      end



      subroutine line_surface_theta_intersect(dim,app,Pstart,u,
     &     r1,r2,theta1,theta2,phi1,phi2,ca,cb,cc,phi,intf,Pint)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the intersection point between a line
c      and a theta-type surface (phi=cst)
c
c     Inputs
c       + dim: dimensions of the vectors
c       + app: if 1, real ground surface must be taken into account
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + r1,r2,theta1,theta2,phi: variables that defines the theta-type surface
c       + phi1,phi2,ca,cb,cc: to compute ground radius
c     
c     Outputs:
c       + intf: 0 if no intersection found; 1 if an intersection was found
c       + Pint: (r,theta,phi) spherical coordinates of the intersection point, if any
c

c     input
      integer dim,app
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision r1,r2,theta1,theta2,phi1,phi2,phi
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     output
      integer intf
      double precision Pint(1:Nmat_mx)
c     temp
      double precision epsilon,epsilon_d,d,dist
      integer n_int,in
      double precision P0(1:Nmat_mx),k0
      double precision pos(1:2),r_ground
      character*100 label
      parameter(epsilon=1.0D-6)
      parameter(epsilon_d=1.0D-5)
c     Debug
      integer j
c     Debug
      label='subroutine line_surface_theta_intersect'

c      call test_surface_theta(r1,r2,theta1,theta2,label)
c      call is_on_surface_theta(r1,r2,theta1,theta2,phi,Pstart,in)
c      if (in.eq.1) then
c         intf=1
c         call copy_vector(dim,Pstart,Pint)
c         goto 123
c      endif

      call line_plane_intersect(dim,Pstart,u,phi,n_int,P0,k0)

      if ((n_int.eq.1).and.(k0.gt.epsilon)) then
         intf=1
         call copy_vector(dim,P0,Pint)
      else
         intf=0
      endif

 123  continue
      if (intf.eq.1) then
         if (app.eq.0) then
            call is_on_surface_theta(r1,r2,theta1,theta2,phi,Pint,
     &           in,dist)
            if (in.eq.0) then
c     Debug
c     write(*,*) 'phi=',phi,' solution not on surface'
c     write(*,*) 'Pint=',(Pint(j),j=1,dim)
c     Debug
               intf=0
c         else
c     if the intersection is on the required surface, its distance to the starting position
c     must be checked
c            call distance_spher(label,Pstart(1),Pstart(2),Pstart(3),
c     &           Pint(1),Pint(2),Pint(3),d)
c            if (d.lt.epsilon_d) then
c               intf=0
c            endif
            else if (app.eq.1) then
               pos(1)=Pstart(2)
               pos(2)=Pstart(3)
               call ground_radius(pos,theta1,theta2,phi1,phi2,
     &              ca,cb,cc,r_ground)
               if ((Pstart(2).lt.theta1).or.(Pstart(2).gt.theta2).or.
     &              (Pstart(1).lt.r_ground)) then
                  intf=0
               endif
            endif
         endif
      endif

      return
      end



      subroutine is_on_surface_theta(r1,r2,theta1,theta2,phi,P,in,dist)
      implicit none
      include 'max.inc'
c
c     Purpose: to check if the provided position "P" is on
c     the provided theta-type surface
c
c     Inputs:
c       + r1,r2,theta1,theta2,phi: variables that defines the theta-type surface
c       + P: (r,theta,phi) spherical coordinates of the point on the surface
c
c     Outputs:
c       + in:
c         - in=0: "P" is not on surface
c         - in=1: "P" is on surface
c       + dist: distance to surface
c

c     input
      double precision r1,r2,theta1,theta2,phi
      double precision P(1:Nmat_mx)
c     output
      integer in
      double precision dist
c     temp
      double precision epsilon
      parameter(epsilon=1.0D-7)
      
      if ((P(1).ge.r1).and.(P(1).le.r2)
     &     .and.(P(2).ge.theta1).and.(P(2).le.theta2)
     &     .and.(dabs(dabs(P(3))-dabs(phi)).lt.epsilon)) then
         in=1
         dist=P(1)*dabs(dabs(P(3))-dabs(phi))
      else
         in=0
      endif

      return
      end



      subroutine line_surface_r_intersect(db,dim,Pstart,u,
     &     r,theta1,theta2,phi1,phi2,intf,Pint)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the intersection point between a line
c      and a r-type surface (r=cst)
c
c     Inputs
c       + dim: dimensions of the vectors
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + r,theta1,theta2,phi1,phi2: variables that defines the r-type surface
c     
c     Outputs:
c       + intf: 0 if no intersection found; 1 if an intersection was found
c       + Pint: (r,theta,phi) spherical coordinates of the intersection point, if any
c

c     input
      integer dim
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision r,theta1,theta2,phi1,phi2
c     output
      integer intf
      double precision Pint(1:Nmat_mx)
c     temp
      integer strlen,db
      double precision epsilon,epsilon_d,d,dist
      integer n_int,in,i
      double precision P0(1:Nmat_mx),k0
      double precision P1(1:Nmat_mx),k1
      double precision P2(1:Nmat_mx),k2
      character*100 label
c     if epsilon is too small, wrong intersections (starting position) may be detected as correct
c      parameter(epsilon=1.0D-6)
      parameter(epsilon=5.0D-7)
      parameter(epsilon_d=1.0D-5)
      label='subroutine line_surface_r_intersect'

c      call test_surface_r(r,theta1,theta2,phi1,phi2,label)
c      call is_on_surface_r(r,theta1,theta2,phi1,phi2,Pstart,in)
c      if (in.eq.1) then
c         intf=1
c         call copy_vector(dim,Pstart,Pint)
c         goto 123
c      endif

      call line_sphere_intersect(db,dim,Pstart,u,r,n_int,
     &     P0,k0,P1,k1,P2,k2)
c     Debug
      if (db.eq.1) then
         write(*,*) 'This is line_surface_r_intersect'
         write(*,*) 'n_int=',n_int
         write(*,*) 'k1=',k1,' k2=',k2
         write(*,*) 'P1=',(P1(i),i=1,dim)
         write(*,*) 'P2=',(P2(i),i=1,dim)
      endif
c     Debug
      if (n_int.eq.0) then
         intf=0
      else if (n_int.eq.1) then
         if (k0.gt.epsilon) then
            intf=1
            call copy_vector(dim,P0,Pint)
         else if (k0.lt.0.0D+0) then
            intf=0
         else
            write(*,*) 'Error from ',label(1:strlen(label)),':'
            write(*,*) 'k0=',k0
            stop
         endif
      else if (n_int.eq.2) then
         if ((k1.gt.epsilon).and.(k2.lt.epsilon)) then
            intf=1
            call copy_vector(dim,P1,Pint)
         else if ((k1.lt.epsilon).and.(k2.gt.epsilon)) then
            intf=1
            call copy_vector(dim,P2,Pint)
         else if ((k1.lt.epsilon).and.(k2.lt.epsilon)) then
            intf=0
         else if ((k1.gt.epsilon).and.(k2.gt.epsilon)) then
c     Debug
            if (db.eq.1) then
               write(*,*) 'ok'
            endif
c     Debug
            if (k2.lt.k1) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),':'
               write(*,*) '2 solutions found with:'
               write(*,*) 'k1=',k1,' k2=',k2
               stop
            else
               intf=1
               call copy_vector(dim,P1,Pint)
            endif
         else
            write(*,*) 'Error from ',label(1:strlen(label)),':'
            write(*,*) 'k1=',k1,' k2=',k2
            stop
         endif
      endif
c     Debug
      if (db.eq.1) then
         write(*,*) 'intf=',intf
c         write(*,*) 'P1=',(P1(i),i=1,dim)
      endif
c     Debug

 123  continue
      if (intf.eq.1) then
         call is_on_surface_r(r,theta1,theta2,phi1,phi2,Pint,in,dist)
c     Debug
         if (db.eq.1) then
            write(*,*) 'Pint=',(Pint(i),i=1,dim)
            write(*,*) 'r=',r
            write(*,*) dabs(dabs(Pint(1))-dabs(r))
            write(*,*) 'theta=',theta1,theta2
            write(*,*) Pint(2)-theta1,Pint(2)-theta2
            write(*,*) 'phi=',phi1,phi2
            write(*,*) Pint(3)-phi1,Pint(3)-phi2
            write(*,*) 'in=',in
         endif
c     Debug
         if (in.eq.0) then
c     Debug
c            write(*,*) 'r=',r,' solution not on surface'
c     Debug
            intf=0
c         else
c     if the intersection is on the required surface, its distance to the starting position
c     must be checked
c            call distance_spher(label,Pstart(1),Pstart(2),Pstart(3),
c     &           Pint(1),Pint(2),Pint(3),d)
c            if (d.lt.epsilon_d) then
c               intf=0
c            endif
         endif
      endif
c     Debug
c      write(*,*) 'r=',r,' intf=',intf
c     Debug

      return
      end



      subroutine is_on_surface_r(r,theta1,theta2,phi1,phi2,P,in,dist)
      implicit none
      include 'max.inc'
c
c     Purpose: to check if the provided position "P" is on
c     the provided r-type surface
c
c     Inputs:
c       + r,theta1,theta2,phi1,phi2: variables that defines the r-type surface
c       + P: (r,theta,phi) spherical coordinates of the point on the surface
c
c     Outputs:
c       + in:
c         - in=0: "P" is not on surface
c         - in=1: "P" is on surface
c       + dist: distance to surface
c

c     input
      double precision r,theta1,theta2,phi1,phi2
      double precision P(1:Nmat_mx)
c     output
      integer in
      double precision dist
c     temp
      double precision epsilon_r
      double precision epsilon_theta
      double precision epsilon_phi
      parameter(epsilon_theta=1.0D-5)
      parameter(epsilon_phi=1.0D-5)
      epsilon_r=1.0D-11*r
      
      if ((P(2).ge.theta1-epsilon_theta).and.
     &     (P(2).le.theta2+epsilon_theta)
     &     .and.(P(3).ge.phi1-epsilon_phi).and.
     &     (P(3).le.phi2+epsilon_phi)
     &     .and.(dabs(dabs(P(1))-dabs(r)).lt.epsilon_r)) then
         in=1
         dist=dabs(dabs(P(1))-dabs(r))
      else
         in=0
      endif

      return
      end




      subroutine line_plane_intersect(dim,Pstart,u,phi,n_int,P0,k0)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the intersection point(s) between a line
c      and a cone
c
c     Inputs:
c       + dim: dimension of the vectors
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + phi: longitude that defines the vertical plane
c
c     Outputs:
c       + n_int: number of intersections
c       + P0: (r,theta,phi) spherical coordinates of the intersection point when n_int=1
c       + k0: index of k for the solution when n_int=1
c

c     input
      integer dim
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision phi
c     output
      integer n_int
      double precision P0(1:Nmat_mx)
      double precision k0
c     temp
      integer strlen
      double precision poss(1:Nmat_mx)
      double precision pos0(1:Nmat_mx)
      double precision un(1:Nmat_mx)
      double precision flight0(1:Nmat_mx)
      character*100 label
      double precision epsilon
      parameter(epsilon=1.0D-5)
      label='subroutine line_plane_intersect'
      
      call test_phi(phi,label)

      call normalize_vector(dim,u,un)
      call spher2cart(label,1,Pstart(1),Pstart(2),Pstart(3),
     &     poss(1),poss(2),poss(3))

      if ((un(1).eq.0.0D+0).and.(un(2).eq.0.0D+0)) then
         if ((poss(1).eq.0.0D+0).and.(poss(2).eq.0.0D+0)) then
c     the starting position is on the z-axis, and the propagation direction is also along
c      the z-axis: all points of the z-axis are solution !
            write(*,*) 'warning from routine ',
     &           label(1:strlen(label)),':'
            write(*,*) 'VERY RARE random event occured'
            n_int=0
c     in this case, it is considered that there is no intersection;
c     an intersection should be found with either type 5 or 6 surfaces
         else
c     the propagation direction is along the z-axis, but the starting position is not
c     on the z-axis: there is no intersection
            n_int=0
         endif
      else
c     un(2)/un(1) can be computed
         if (un(2)*dcos(phi).eq.un(1)*dsin(phi)) then
c     the propagation direction is parallel to the plane -> no intersection
            n_int=0
         else
c     the propagation direction is not parallel to the plane -> 1 intersection
            n_int=1
            k0=(poss(1)*dsin(phi)-poss(2)*dcos(phi))
     &           /(un(2)*dcos(phi)-un(1)*dsin(phi))
            call scalar_vector(dim,k0,un,flight0)
            call add_vectors(dim,poss,flight0,pos0)
            call cart2spher(pos0(1),pos0(2),pos0(3),P0(1),P0(2),P0(3))
            P0(3)=phi
         endif
      endif

 123  continue

      return
      end


      subroutine line_cone_intersect(db,dim,Pstart,u,theta,n_int,
     &     P0,k0,P1,k1,P2,k2)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the intersection point(s) between a line
c      and a cone
c
c     Inputs:
c       + dim: dimension of the vectors
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + theta: latitude that defines the cone
c
c     Outputs:
c       + n_int: number of intersections
c       + P0: (r,theta,phi) spherical coordinates of the intersection point when n_int=1
c       + k0: index of k for the solution when n_int=1
c       + P1,P2: (r,theta,phi) spherical coordinates of the intersections point when n_int=2
c       + k1,k2: indexes of k for the solutions when n_int=2
c

c     input
      integer dim
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision theta
c     output
      integer n_int
      double precision P0(1:Nmat_mx)
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision k0,k1,k2
c     temp
      integer strlen
      double precision a,b,c
      integer nsol,db
      double precision poss(1:Nmat_mx)
      double precision pos0(1:Nmat_mx)
      double precision pos1(1:Nmat_mx)
      double precision pos2(1:Nmat_mx)
      double precision un(1:Nmat_mx)
      double precision flight0(1:Nmat_mx)
      double precision flight1(1:Nmat_mx)
      double precision flight2(1:Nmat_mx)
      double precision sign
      character*100 label
      double precision epsilon,epsilon_r
      parameter(epsilon=1.0D-5)
      label='subroutine line_cone_intersect'
      epsilon_r=1.0D-11*Pstart(1)
      
      call test_theta(theta,label)
      call normalize_vector(dim,u,un)
      call spher2cart(label,1,Pstart(1),Pstart(2),Pstart(3),
     &     poss(1),poss(2),poss(3))
      if (theta.eq.0.0D+0) then
c     Debug
         if (db.eq.1) then
            write(*,*) 'ok1, un(3)=',un(3)
         endif
c     Debug
c     In theta=0, the surface the line intersects with is not a cone, but the plane z=0
         if (un(3).eq.0.0D+0) then
c     if the propagation direction is parallel to plane z=0, then no intersection is possible
            n_int=0
         else
            k0=-poss(3)/un(3)
c     Debug
            if (db.eq.1) then
               write(*,*) 'k0=',k0
            endif
c     Debug
            if (k0.gt.0.0D+0) then
c     intersection found in the right direction !
               n_int=1
               pos0(1)=poss(1)+k0*un(1)
               pos0(2)=poss(2)+k0*un(2)
               pos0(3)=0.0D+0
               call cart2spher(pos0(1),pos0(2),pos0(3),
     &              P0(1),P0(2),P0(3))
               if (k0.eq.0.0D+0) then
                  k0=1.0D-3
               endif
            else
c     no intersection
               n_int=0
            endif
c     Debug
            if (db.eq.1) then
               write(*,*) 'n_int=',n_int
            endif
c     Debug
         endif
      else
c     If theta is different from 0, then the surface the line intersects with is a cone
         a=(un(1)**2.0D+0+un(2)**2.0D+0)*dsin(theta)**2.0D+0
     &        -(dcos(theta)**2.0D+0)*(un(3)**2.0D+0)
         b=2.0D+0*((poss(1)*un(1)+poss(2)*un(2))*dsin(theta)**2.0D+0
     &        -poss(3)*un(3)*dcos(theta)**2.0D+0)
         c=(poss(1)**2.0D+0+poss(2)**2.0D+0)*dsin(theta)**2.0D+0
     &        -(poss(3)**2.0D+0)*(dcos(theta)**2.0D+0)

         call eq2deg(a,b,c,nsol,k0,k1,k2)
         n_int=nsol
         if (n_int.eq.1) then
            call scalar_vector(dim,k0,un,flight0)
            call add_vectors(dim,poss,flight0,pos0)
            call cart2spher(pos0(1),pos0(2),pos0(3),P0(1),P0(2),P0(3))
            if ((dabs(dabs(P0(2))-dabs(theta)).gt.epsilon).
     &           and.(P0(1).gt.epsilon_r)) then
c               write(*,*) 'Error from ',
c     &              label(1:strlen(label)),':'
c               write(*,*) 'theta=',theta
c               write(*,*) '1 solution with theta found=',P0(2)
c               stop
               n_int=0
               goto 666
            endif
            sign=P0(2)*theta
            if (sign.gt.0.0D+0) then
               P0(2)=theta
            else
               P0(2)=-theta
            endif
         else if (n_int.eq.2) then
            call scalar_vector(dim,k1,un,flight1)
            call scalar_vector(dim,k2,un,flight2)
            call add_vectors(dim,poss,flight1,pos1)
            call add_vectors(dim,poss,flight2,pos2)
            call cart2spher(pos1(1),pos1(2),pos1(3),P1(1),P1(2),P1(3))
            call cart2spher(pos2(1),pos2(2),pos2(3),P2(1),P2(2),P2(3))
            if ((dabs(dabs(P1(2))-dabs(theta)).gt.epsilon).
     &           and.(P1(1).gt.epsilon_r)) then
c               write(*,*) 'Error from ',
c     &              label(1:strlen(label)),':'
c               write(*,*) 'theta=',theta
c               write(*,*) 'solution 1 with theta found=',P1(2)
c               write(*,*) 'P1=',P1(1),P1(2),P1(3)
c               stop
               n_int=0
               goto 666
            endif
            if ((dabs(dabs(P2(2))-dabs(theta)).gt.epsilon).
     &           and.(P2(1).gt.epsilon_r)) then
c               write(*,*) 'Error from ',
c     &              label(1:strlen(label)),':'
c               write(*,*) 'theta=',theta
c               write(*,*) 'solution 2 with theta found=',P2(2)
c               stop
               n_int=0
               goto 666
            endif
            sign=P1(2)*theta
            if (sign.gt.0.0D+0) then
               P1(2)=theta
            else
               P1(2)=-theta
            endif
            sign=P2(2)*theta
            if (sign.gt.0.0D+0) then
               P2(2)=theta
            else
               P2(2)=-theta
            endif
         endif
      endif ! theta=0

 666  continue
      return
      end



      subroutine line_sphere_intersect(db,dim,Pstart,u,r,n_int,
     &     P0,k0,P1,k1,P2,k2)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the intersection point(s) between a line
c      and a sphere
c
c     Inputs:
c       + dim: dimension of the vectors
c       + Pstart: (r,theta,phi) spherical coordinates of the starting point of the line
c       + u: coordinates of the direction vector
c       + r: radius of the sphere
c
c     Outputs:
c       + n_int: number of intersections
c       + P0: (r,theta,phi) spherical coordinates of the intersection point when n_int=1
c       + k0: index of k for the solution when n_int=1
c       + P1,P2: (r,theta,phi) spherical coordinates of the intersections point when n_int=2
c       + k1,k2: indexes of k for the solutions when n_int=2
c

c     input
      integer dim
      double precision Pstart(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision r
c     output
      integer n_int
      double precision P0(1:Nmat_mx)
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision k0,k1,k2
c     temp
      integer strlen,db
      double precision a,b,c
      integer nsol
      double precision poss(1:Nmat_mx)
      double precision pos0(1:Nmat_mx)
      double precision pos1(1:Nmat_mx)
      double precision pos2(1:Nmat_mx)
      double precision un(1:Nmat_mx)
      double precision flight0(1:Nmat_mx)
      double precision flight1(1:Nmat_mx)
      double precision flight2(1:Nmat_mx)
      character*100 label
      double precision epsilon_r
      label='subroutine line_sphere_intersect'
      epsilon_r=1.0D-11*r
      
      call test_r(r,label)
      call normalize_vector(dim,u,un)
      call spher2cart(label,1,Pstart(1),Pstart(2),Pstart(3),
     &     poss(1),poss(2),poss(3))
      a=un(1)**2.0D+0+un(2)**2.0D+0+un(3)**2.0D+0
      b=2.0D+0*(un(1)*poss(1)+un(2)*poss(2)+un(3)*poss(3))
      c=poss(1)**2.0D+0+poss(2)**2.0D+0+poss(3)**2.0D+0-r**2.0D+0

      call eq2deg(a,b,c,nsol,k0,k1,k2)
      n_int=nsol
      if (n_int.eq.1) then
         call scalar_vector(dim,k0,un,flight0)
         call add_vectors(dim,poss,flight0,pos0)
         call cart2spher(pos0(1),pos0(2),pos0(3),P0(1),P0(2),P0(3))
         if (dabs(P0(1)-r).gt.epsilon_r) then
c            write(*,*) 'Error from ',
c     &           label(1:strlen(label)),':'
c            write(*,*) 'r=',r
c            write(*,*) '1 solution with r found=',P0(1)
c            stop
            n_int=0
            goto 666
         endif
         P0(1)=r
      else if (n_int.eq.2) then
         call scalar_vector(dim,k1,un,flight1)
         call scalar_vector(dim,k2,un,flight2)
         call add_vectors(dim,poss,flight1,pos1)
         call add_vectors(dim,poss,flight2,pos2)
         call cart2spher(pos1(1),pos1(2),pos1(3),P1(1),P1(2),P1(3))
         call cart2spher(pos2(1),pos2(2),pos2(3),P2(1),P2(2),P2(3))
         if (dabs(P1(1)-r).gt.epsilon_r) then
c            write(*,*) 'Error from ',
c     &           label(1:strlen(label)),':'
c            write(*,*) 'r=',r
c            write(*,*) 'solution 1 with r found=',P1(1)
c            stop
            n_int=0
            goto 666
         endif
         if (dabs(P2(1)-r).gt.epsilon_r) then
c            write(*,*) 'Error from ',
c     &           label(1:strlen(label)),':'
c            write(*,*) 'r=',r
c            write(*,*) 'solution 2 with r found=',P2(1)
c            stop
            n_int=0
            goto 666
         endif
         P1(1)=r
         P2(1)=r
      endif

 666  continue
      return
      end



      subroutine eq2deg(a,b,c,nsol,x0,x1,x2)
      implicit none
c
c     Purpose: to solve a 2nd degree equation ax+bx+c=0
c
c     Inputs:
c       + a,b,c: coefficients of the 2nd degree equation ax+bx+c=0
c
c     Outputs:
c       + nsol: number of solution (0,1 or 2)
c       + x0: solution when nsol=1
c       + x1,x2: solutions when nsol=2 (x1<x2
c
      double precision a,b,c
      integer nsol
      double precision x0,x1,x2
      double precision t1,t2
      double precision delta

      delta=b**2.0D+0-4.0D+0*a*c
      if (delta.lt.0.0D+0) then
         nsol=0
      else if (delta.eq.0.0D+0) then
         nsol=1
         x0=-b/(2.0D+0*a)
      else if (delta.gt.0.0D+0) then
         nsol=2
         t1=(-b-dsqrt(delta))/(2.0D+0*a)
         t2=(-b+dsqrt(delta))/(2.0D+0*a)
         if (t1.lt.t2) then
            x1=t1
            x2=t2
         else
            x1=t2
            x2=t1
         endif
      else
         write(*,*) 'Error from eq2deg:'
         write(*,*) 'delta=',delta
         stop
      endif

      return
      end
