c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine stopped_processes(free,nproc,nsp)
      implicit none
c
c     Purpose: to identify the number of child processes that have been stopped
c
c     Input:
c       + free: array of free processes (a value of 2 means that the corresponding child process has been stopped)
c       + nproc: number of child processes
c     
c     Output:
c       + nsp: number of child processes that have been stopped
c     
      include 'max.inc'
      integer free(1:Nproc_mx)
      integer proc,nproc,nsp
      nsp=0
      do proc=1,nproc
         if (free(proc).eq.2) then
            nsp=nsp+1
         endif
      enddo

      return
      end



      subroutine set_chunks(nchunks,np,Nevent,Nev)
      implicit none
c
c     Purpose: to set the number of statistical events each chunks will have to simulate
c
c     Inputs:
c       + nchunks: number of chunks (if -1 value is passed, nchunks is set to the number of slave processes, np-1)
c       + np: total number of processes
c       + Nevent: total number of statistical events that has to be simulated
c
c     Output:
c       + Nev: array of statistical events (for each chunk)
c
      include 'max.inc'
      integer nchunks,np,Nevent
      integer Nkpp,Nklp,chunk
      integer Nev(1:Nchunk_mx)

      if (nchunks.eq.-1) then
         nchunks=np-1
      endif
      if (Nevent.ge.nchunks) then
         Nkpp=Nevent/nchunks
         Nklp=Nevent-Nkpp*(nchunks-1)
         
         do chunk=1,nchunks
            if (chunk.lt.nchunks) then
               Nev(chunk)=Nkpp
            else
               Nev(chunk)=Nklp
            endif
         enddo                  ! chunk

      else
         do chunk=1,Nevent
            Nev(chunk)=1
         enddo
         do chunk=Nevent+1,nchunks
            Nev(chunk)=0
         enddo
      endif

      return
      end



      subroutine send_psums(proc,toa_ad,psum_flux_up,psum_flux_dn,
     &     psum_flux_net,psum_flux_toa,psum_budget,
     &     psum2_flux_up,psum2_flux_dn,
     &     psum2_flux_net,psum2_flux_toa,psum2_budget,
     &     ninvalid)
      implicit none
      include 'mpif.h'
      include 'max.inc'

      integer proc,toa_ad,ninvalid
      double precision psum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH

      call MPI_SEND(psum_flux_up,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(psum_flux_dn,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(psum_flux_net,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(psum_budget,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(psum2_flux_up,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(psum2_flux_dn,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(psum2_flux_net,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(psum2_budget,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(ninvalid,1,MPI_INTEGER,proc,1,MPI_COMM_WORLD,ierr)
      if (toa_ad.eq.1) then
         call MPI_SEND(psum_flux_toa,
     &        Ntheta_mx*Nphi_mx*Ntheta_mx*Nphi_mx*Nb_gas_mx,
     &        MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
         call MPI_SEND(psum2_flux_toa,
     &        Ntheta_mx*Nphi_mx*Ntheta_mx*Nphi_mx*Nb_gas_mx,
     &        MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      endif

      return
      end




      subroutine receive_psums(proc,toa_ad,psum_flux_up,psum_flux_dn,
     &     psum_flux_net,psum_flux_toa,psum_budget,
     &     psum2_flux_up,psum2_flux_dn,
     &     psum2_flux_net,psum2_flux_toa,psum2_budget,
     &     ninvalid)
      implicit none
      include 'mpif.h'
      include 'max.inc'

      integer proc,toa_ad,ninvalid
      double precision psum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH

      call MPI_RECV(psum_flux_up,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(psum_flux_dn,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(psum_flux_net,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(psum_budget,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(psum2_flux_up,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(psum2_flux_dn,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(psum2_flux_net,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(psum2_budget,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &     stat,ierr)
      call MPI_RECV(ninvalid,1,MPI_INTEGER,proc,MPI_ANY_TAG,
     &     MPI_COMM_WORLD,stat,ierr)
      if (toa_ad.eq.1) then
         call MPI_RECV(psum_flux_toa,
     &        Ntheta_mx*Nphi_mx*Ntheta_mx*Nphi_mx*Nb_gas_mx,
     &        MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &        stat,ierr)
         call MPI_RECV(psum2_flux_toa,
     &        Ntheta_mx*Nphi_mx*Ntheta_mx*Nphi_mx*Nb_gas_mx,
     &        MPI_DOUBLE_PRECISION,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &        stat,ierr)
      endif

      return
      end




      subroutine receive_spectral_data(ka_gas_1q,ks_gas_1q,n_ref_1b,
     &     ground_emissivity_1b,space_emissivity_1b,emissivity_1b)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     MPICH
      integer ierr,errorcode,pindex,proc
      integer stat(MPI_STATUS_SIZE)
c     MPICH
      double precision ka_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ks_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision n_ref_1b(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ground_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision space_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_1b(1:Nzones_mx)

      call MPI_RECV(ka_gas_1q,Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_ANY_TAG,
     &     MPI_COMM_WORLD,stat,ierr)
      call MPI_RECV(ks_gas_1q,Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_ANY_TAG,
     &     MPI_COMM_WORLD,stat,ierr)
      call MPI_RECV(n_ref_1b,Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_ANY_TAG,
     &     MPI_COMM_WORLD,stat,ierr)
      call MPI_RECV(ground_emissivity_1b,Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_ANY_TAG,
     &     MPI_COMM_WORLD,stat,ierr)
      call MPI_RECV(space_emissivity_1b,Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_ANY_TAG,
     &     MPI_COMM_WORLD,stat,ierr)
      call MPI_RECV(emissivity_1b,Nzones_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_ANY_TAG,
     &     MPI_COMM_WORLD,stat,ierr)

      return
      end




      subroutine send_spectral_data(ka_gas_1q,ks_gas_1q,n_ref_1b,
     &     ground_emissivity_1b,space_emissivity_1b,
     &     emissivity_1b,proc)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     MPICH
      integer ierr,errorcode,pindex,proc
      integer stat(MPI_STATUS_SIZE)
c     MPICH
      double precision ka_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ks_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision n_ref_1b(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ground_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision space_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_1b(1:Nzones_mx)

      call MPI_SEND(ka_gas_1q,
     &     Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(ks_gas_1q,
     &     Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(n_ref_1b,
     &     Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(ground_emissivity_1b,
     &     Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(space_emissivity_1b,
     &     Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)
      call MPI_SEND(emissivity_1b,
     &     Nzones_mx,
     &     MPI_DOUBLE_PRECISION,proc,1,MPI_COMM_WORLD,ierr)

      return
      end




      subroutine send_global_data_mpibcast(np,dim,seeds,Nevent,
     &     Nr,Ntheta,Nphi,Nb_gas,Nq,Nb_clouds,Nclouds,Na,Nzones,
     &     ref,vri,toa_ad,pf,Radius,Tsun,Rsun,dist,
     &     theta0,phi0,clouds_coord,
     &     lambda_gas_lo,lambda_gas_hi,w_quad,
     &     lambda_clouds_lo,lambda_clouds_hi,
     &     clouds_ka,clouds_ks,clouds_g,clouds_phase,
     &     r,theta,phi,rground,rspace,T,r_mc,
     &     emissivity_zone,band_min,band_max,spectral_pdf,
     &     spectral_cumul,r_nodes,r_south_pole,r_north_pole,
     &     coeff_a,coeff_b,coeff_c)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
      integer np,dim
      integer seeds(1:Nproc_mx)
      integer Nevent,Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      integer Nb_gas,Nq,Nb_clouds,Nclouds,Na,Nzones,ref,vri,toa_ad,pf
      double precision Radius,Tsun,Rsun,dist,theta0,phi0
      double precision clouds_coord(1:Nclouds_mx,1:6)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision w_quad(1:Nq_mx)
      double precision lambda_clouds_lo(1:Nb_clouds_mx)
      double precision lambda_clouds_hi(1:Nb_clouds_mx)
      double precision clouds_ka(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_ks(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_g(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_phase(1:Nclouds_mx,1:Nb_clouds_mx,
     &     1:Nangle_mx,1:2)
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision rground,rspace
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_zone(1:Nzones_mx,1:4)
      integer band_min,band_max
      double precision spectral_pdf(1:Nb_gas_mx,1:Nq_mx)
      double precision spectral_cumul(0:Ncumul_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole,r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)

      call MPI_BCAST(dim,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(seeds,np-1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nevent,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nr,Ntheta_mx*Nphi_mx,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Ntheta,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nphi,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nb_gas,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nq,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(band_min,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(band_max,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nb_clouds,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nclouds,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Na,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nzones,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ref,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(vri,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(toa_ad,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(pf,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Radius,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Tsun,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Rsun,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(dist,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(theta0,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(phi0,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_coord,Nclouds_mx*6,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_gas_lo,Nb_gas_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_gas_hi,Nb_gas_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(w_quad,Nq_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_clouds_lo,Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_clouds_hi,Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_ka,Nclouds_mx*Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_ks,Nclouds_mx*Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_g,Nclouds_mx*Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_phase,Nclouds_mx*Nb_clouds_mx
     &     *Nangle_mx*2,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r,(Nz_mx+1)*Ntheta_mx*Nphi_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(theta,Ntheta_mx+1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(phi,Nphi_mx+1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(rground,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(rspace,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(T,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_mc,Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(emissivity_zone,Nzones_mx*4,
     &     MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(spectral_pdf,Nb_gas_mx*Nq_mx,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(spectral_cumul,Ncumul_mx+1,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_nodes,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_south_pole,1,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_north_pole,1,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(coeff_a,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(coeff_b,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(coeff_c,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)

      return
      end



      subroutine receive_global_data_mpibcast(np,dim,seeds,
     &     Nevent,Nr,Ntheta,Nphi,Nb_gas,Nq,Nb_clouds,Nclouds,Na,Nzones,
     &     ref,vri,toa_ad,pf,Radius,Tsun,Rsun,dist,
     &     theta0,phi0,clouds_coord,
     &     lambda_gas_lo,lambda_gas_hi,w_quad,
     &     lambda_clouds_lo,lambda_clouds_hi,
     &     clouds_ka,clouds_ks,clouds_g,clouds_phase,
     &     r,theta,phi,rground,rspace,T,r_mc,
     &     emissivity_zone,band_min,band_max,spectral_pdf,
     &     spectral_cumul,r_nodes,r_south_pole,r_north_pole,
     &     coeff_a,coeff_b,coeff_c)
      implicit none
      include 'mpif.h'
      include 'max.inc'
c     MPICH
      integer ierr
      integer stat(MPI_STATUS_SIZE)
c     MPICH
      integer np,dim
      integer seeds(1:Nproc_mx)
      integer Nevent,Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi
      integer Nb_gas,Nq,Nb_clouds,Nclouds,Na,Nzones,ref,vri,toa_ad,pf
      double precision Radius,Tsun,Rsun,dist,theta0,phi0
      double precision clouds_coord(1:Nclouds_mx,1:6)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision w_quad(1:Nq_mx)
      double precision lambda_clouds_lo(1:Nb_clouds_mx)
      double precision lambda_clouds_hi(1:Nb_clouds_mx)
      double precision clouds_ka(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_ks(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_g(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_phase(1:Nclouds_mx,1:Nb_clouds_mx,
     &     1:Nangle_mx,1:2)
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision rground,rspace
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_zone(1:Nzones_mx,1:4)
      integer band_min,band_max
      double precision spectral_pdf(1:Nb_gas_mx,1:Nq_mx)
      double precision spectral_cumul(0:Ncumul_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole,r_north_pole
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)

      call MPI_BCAST(dim,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(seeds,np-1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nevent,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nr,Ntheta_mx*Nphi_mx,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Ntheta,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nphi,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nb_gas,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nq,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(band_min,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(band_max,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nb_clouds,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nclouds,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Na,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Nzones,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(ref,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(vri,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(toa_ad,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(pf,1,MPI_INTEGER
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Radius,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Tsun,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(Rsun,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(dist,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(theta0,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(phi0,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_coord,Nclouds_mx*6,
     &     MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_gas_lo,Nb_gas_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_gas_hi,Nb_gas_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(w_quad,Nq_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_clouds_lo,Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(lambda_clouds_hi,Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_ka,Nclouds_mx*Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_ks,Nclouds_mx*Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_g,Nclouds_mx*Nb_clouds_mx,
     &     MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(clouds_phase,Nclouds_mx*Nb_clouds_mx*
     &     Nangle_mx*2,MPI_DOUBLE_PRECISION,0,
     &     MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r,(Nz_mx+1)*Ntheta_mx*Nphi_mx,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(theta,Ntheta_mx+1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(phi,Nphi_mx+1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(rground,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(rspace,1,MPI_DOUBLE_PRECISION
     &     ,0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(T,(Nz_mx+1)*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_mc,Nz_mx*Ntheta_mx*Nphi_mx,
     &     MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(emissivity_zone,Nzones_mx*4,
     &     MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(spectral_pdf,Nb_gas_mx*Nq_mx,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(spectral_cumul,Ncumul_mx+1,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_nodes,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_south_pole,1,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(r_north_pole,1,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(coeff_a,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(coeff_b,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)
      call MPI_BCAST(coeff_c,Ntheta_mx*Nphi_mx*4,MPI_DOUBLE_PRECISION,
     &     0,MPI_COMM_WORLD,ierr)

      return
      end




      subroutine finalize(np,nc_phost)
      implicit none
      include 'max.inc'

      integer np
      integer nc_phost(1:Nproc_mx)
      integer proc

      open(10,file='./logs/Nchunks_per_host.log')
      write(10,*) 'Process ID / Nchunks'
      do proc=1,np-1
         write(10,*) proc,nc_phost(proc)
      enddo ! proc
      close(10)

      return
      end
