c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      program planet_EMC
      implicit none
c     ------------------------------------------------------------------------
c     First, install mpich2:
c     retrieve the archive, unzip and untar it, install it on the system:
c     > ./configure --prefix=/home/[user]/parallele/mpich2
c     > make
c     > make install
c     
c     Before running MPD:
c     
c     1) create a ~/.mpd.conf file:
c     > echo secretword=[secretword] >> ~/.mpd.conf
c     > chmod 600 .mpd.conf
c     
c     2) allow your machine to connect by ssh to every other machine, without password:
c     create a dsa key:
c     > ssh-keygen -t dsa
c     then leave everything blank (only use 'enter' to proceed)
c     > cd .ssh
c     > cat id_dsa.pub >> authorized_keys
c     
c     3) create the mpd.hosts file: each line must contain the name of a machine
c     that is part of your cluster, by order of availability:
c     [host1].[domain]
c     [host2].[domain]
c     [host3].[domain]
c     etc
c     ------------------------------------------------------------------------
c     Running MPD:
c     > mpdboot -n [#]
c     with # the number of machines MPD has to run on
c     
c     use 'mpif77' in order to compile fortran 77 code in your makefile (or whatever)
c     
c     Code execution:
c     > mpirun -np [#] [executable]
c     with # the number of processes; in a master-slave algorithm, there is a master process and
c     'np-1' slave processes. If the master process is idle most of the time, it might be a good idea
c     to have 'np-1' equal to the number of physical cores, so that each (time consuming) process
c     uses a core.  In this case, 'np' is equal to the number of cores plus one. The total computation
c     time should be roughly divided by the number of cores (if communication times are negligeable,
c     and of course if the master process does not use too much time).
c     
c     In the particular case of the present code, there is no problem using np=number of cores+1
c     
c     Exit MPD:
c     > mpdallexit
c     ------------------------------------------------------------------------

      include 'mpif.h'
      include 'max.inc'
      include 'param.inc'

c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
      integer code,codep,proc,ret_code
      integer free(1:Nproc_mx),fpf,nsp
      integer nchunks,chunk
      integer Nev(1:Nchunk_mx)
      integer ch_idx(1:Nproc_mx)
      integer nc_phost(1:Nproc_mx)
c     MPICH
      integer strlen
      integer Nb_gas,Nq,Nb_clouds,Na
      integer end
      double precision eps_st4
      integer Nevent,Nevp,ref,intp,pf,vri,toa_ad
      integer seeds(1:Nproc_mx)
c     basic data
      integer Nclouds
      integer Nr(1:Ntheta_mx,1:Nphi_mx),Ntheta,Nphi,Nzones
      double precision z(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision r(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision theta(0:Ntheta_mx)
      double precision phi(0:Nphi_mx)
      double precision rground,rspace
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      double precision w_quad(1:Nq_mx)
      double precision gas_ka(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision gas_ks(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision Radius,Tsun,Rsun,dist,theta0,phi0
      double precision clouds_coord(1:Nclouds_mx,1:6)
      double precision lambda_clouds_lo(1:Nb_clouds_mx)
      double precision lambda_clouds_hi(1:Nb_clouds_mx)
      double precision clouds_ka(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_ks(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_phase(1:Nclouds_mx,1:Nb_clouds_mx,
     &     1:Nangle_mx,1:2)
      double precision clouds_g(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision r_mc(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision Tground(1:Ntheta_mx,1:Nphi_mx)
      double precision T(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision Tspace(1:Ntheta_mx,1:Nphi_mx)
      double precision ground_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      double precision space_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      double precision emissivity_zone(1:Nzones_mx,1:4)
      double precision emissivity(1:Nzones_mx,1:Nb_gas_mx)
      double precision n_ref(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
c     sums for process 0
      double precision sum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision sum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision sum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision sum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
c     (temporary) sums for each child process
      double precision psum_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision psum2_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision psum2_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
c     results
      double precision solar_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision solar_inc_flux_up(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision solar_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision solar_inc_flux_dn(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision solar_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision solar_inc_flux_net(0:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision solar_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision solar_inc_flux_toa(1:Ntheta_mx,1:Nphi_mx,
     &     1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision solar_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
      double precision solar_inc_budget(0:Nz_mx+1,1:Ntheta_mx,1:Nphi_mx)
c     temp
      integer dim,ir,itheta,iphi,j
      integer u_itheta,u_iphi
      double precision xr,xtheta,xphi,Tinterp
      integer band_min,band_max
      double precision Tsolar(1:Nz_mx)
      double precision spectral_pdf(1:Nb_gas_mx,1:Nq_mx)
      double precision spectral_cumul(0:Ncumul_mx)
      double precision r_nodes(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision r_south_pole,r_north_pole
      double precision ground_surfaces(1:Ntheta_mx,1:Nphi_mx)
      double precision ka_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ks_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision n_ref_1b(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ground_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision space_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_1b(1:Nzones_mx)
      double precision ka_clouds(1:Nclouds_mx)
      double precision ks_clouds(1:Nclouds_mx)
      double precision phase_clouds(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g_clouds(1:Nclouds_mx)
      double precision coeff_a(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_b(1:Ntheta_mx,1:Nphi_mx,1:4)
      double precision coeff_c(1:Ntheta_mx,1:Nphi_mx,1:4)
      integer ninvalid,ninv(1:Nproc_mx)
      character*100 label
c     tests
      integer band,quad
      double precision solar_power,sigmaT4,planck_int_wavlng
      double precision fnet1,inc_fnet1
      double precision fnet2,inc_fnet2
      double precision surface_r,surf,surf1,surf2
      double precision dlambda,spectral_weight
      double precision P(1:2),radius_ground
c     tests

c     Debug
      integer err_code
      double precision Pstart(1:Nmat_mx)
      double precision u0(1:Nmat_mx)
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
      double precision nx0(1:Nmat_mx)
      integer i,intf,solf,surface,aerr,app
      integer valid,s_out
      double precision Pint(1:Nmat_mx)
      double precision P_out(1:Nmat_mx)
      double precision u_out(1:Nmat_mx)
      integer Nc
      double precision phase(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g(1:Nclouds_mx)
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision ka_gas,ks_gas
      double precision ka_cloud(1:Nclouds_mx)
      double precision ks_cloud(1:Nclouds_mx)
      integer Npositions
      integer surface_out
      double precision path_length
      double precision tau_path(1:2,1:Ndiff_mx)
      double precision x_path(1:Nmat_mx,1:Ndiff_mx)
      double precision P0(1:Nmat_mx)
      double precision P1(1:Nmat_mx)
      double precision x0(1:Nmat_mx)
      double precision x1(1:Nmat_mx)
      double precision seg(1:Nmat_mx)
c     Debug

      label='program planet_EMC'

c     MPICH
      call MPI_INIT(ierr)
      if (ierr.NE.MPI_SUCCESS) THEN
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'MPI could not be initialized'
         call MPI_ABORT(MPI_COMM_WORLD,errorcode,ierr)
      endif
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
      if (np.lt.2) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Number of processes is:',np
         write(*,*) 'and it should be greater than 1'
         stop
      endif
      if (np.gt.Nproc_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Number of processes is:',np
         write(*,*) 'is greater than Nproc_mx=',Nproc_mx
         stop
      endif
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH

c     MPICH
      if (pindex.eq.0) then
c     MPICH
         write(*,*) 'Planet_EMC Copyright (C) 2008-2009 Vincent Eymet'
         write(*,*) 'This program comes with ABSOLUTELY NO WARRANTY;'
         write(*,*) 'This is free software, and you are welcome to'//
     &        ' redistribute it'
         write(*,*) 'under certain conditions; see file "COPYING"'//
     &        ' for details.'
         write(*,*)

c     some initialization
         write(*,*) 'Initializing...'
         call init(np,seeds,ninv,dim)

c     reading "data.in"
         call read_data(eps_st4,Nevent,nchunks)
c     reading "options.in"
         call read_options(ref,intp,pf,vri,toa_ad)
c     reading "data/planet.in"
         call read_planet(Radius,Tsun,Rsun,dist,theta0,phi0)
c     reading "data/clouds.in"
         call read_clouds(Nclouds,Nb_clouds,Na,
     &        lambda_clouds_lo,lambda_clouds_hi,clouds_coord,
     &        clouds_ka,clouds_ks,clouds_g,clouds_phase)
c     reading "data/grid.in"
         call read_grid(Radius,Nr,Ntheta,Nphi,Nb_gas,Nq,Nzones,
     &        z,r,theta,phi,rground,rspace,lambda_gas_lo,lambda_gas_hi,
     &        w_quad,gas_ka,gas_ks,r_mc,Tground,T,Tspace,
     &        ground_emissivity,space_emissivity,
     &        emissivity_zone,emissivity,n_ref)
c     Debug
         write(*,*) 'Looking for known errors...'
c     Debug
         call look_for_errors(eps_st4,Nevent,
     &        nchunks,ref,intp,pf,vri,toa_ad,
     &        Radius,Tsun,Rsun,dist,theta0,phi0)
c     Debug
         write(*,*) '...none found, resuming initialization'
c     Debug

c     computation of solar spectral range
         Tsolar(1)=Tsun
         call spectral_range(1,Tsolar,eps_st4,Nb_gas,Nq,w_quad,
     &        lambda_gas_lo,lambda_gas_hi,band_min,band_max,
     &        spectral_pdf,spectral_cumul)

c     interpolation of ground altitude at grid nodes
         call nodes_ground_altitude(dim,Nr,Ntheta,Nphi,
     &        r,theta,phi,r_nodes,r_south_pole,r_north_pole)
c     computation of coefficients a, b and c
         call surface_coeffs(dim,Nr,Ntheta,Nphi,r,theta,phi,
     &        r_nodes,r_south_pole,r_north_pole,
     &        coeff_a,coeff_b,coeff_c)
c     computation of real surfaces at ground level
         call surfaces_on_ground(dim,Ntheta,Nphi,r,theta,phi,
     &        r_nodes,r_south_pole,r_north_pole,
     &        coeff_a,coeff_b,coeff_c,ground_surfaces)
c     more initializations
         call init_sums(Nr,Ntheta,Nphi,Nb_gas,
     &        toa_ad,
     &        sum_flux_up,
     &        sum_flux_dn,
     &        sum_flux_net,
     &        sum_flux_toa,
     &        sum_budget,
     &        sum2_flux_up,
     &        sum2_flux_dn,
     &        sum2_flux_net,
     &        sum2_flux_toa,
     &        sum2_budget)
c     Debug
         write(*,*) '...done'
c     Debug

c     Debug
c         ir=1
c         itheta=15
c         iphi=1
c         Pstart(1)=6378137.00000000
c         Pstart(2)=0.781508022220847
c         Pstart(3)=0.000000000000000E+000
c         u0(1)=0.667561960513273
c         u0(2)=0.735050106323478
c         u0(3)=0.118584864420041
c         call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
c     &        ca,cb,cc)
c         call need2approximate(dim,Ntheta,r,theta,phi,
c     &        r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)
c         call intersect_cell(dim,1,app,
c     &        Pstart,u0,
c     &        r(ir-1,itheta,iphi),r(ir,itheta,iphi),
c     &        theta(itheta-1),theta(itheta),
c     &        phi(iphi-1),phi(iphi),
c     &        ca,cb,cc,
c     &        intf,err_code,Pint,surface)
c         write(*,*) 'intf=',intf
c         if (intf.eq.1) then
c            write(*,*) 'Pint=',(Pint(i),i=1,dim)
c            write(*,*) 'surface=',surface
c         endif
c         stop
c         write(*,*) 'now last_chance'
c         call last_chance(dim,1,Nr,Ntheta,Nphi,r,theta,phi,
c     &        rspace,r_nodes,r_south_pole,r_north_pole,
c     &        coeff_a,coeff_b,coeff_c,Pstart,u0,
c     &        ir,itheta,iphi,P_out,u_out,s_out,valid)
c         write(*,*) 'valid=',valid
c         if (valid.eq.1) then
c            write(*,*) ir,itheta,iphi
c            write(*,*) 'P_out=',(P_out(i),i=1,dim)
c            write(*,*) 'u_out=',(u_out(i),i=1,dim)
c            write(*,*) 's_out=',s_out
c         endif
c         stop

c         call extract_coeffs(itheta,iphi,coeff_a,coeff_b,coeff_c,
c     &        ca,cb,cc)
c         call need2approximate(dim,Ntheta,r,theta,phi,
c     &        r_nodes,r_south_pole,r_north_pole,itheta,iphi,app)
c         write(*,*) 'app=',app
c         Nc=0
c         call path(1,dim,1,app,
c     &        r(ir-1,itheta,iphi),r(ir,itheta,iphi),
c     &        theta(itheta-1),theta(itheta),
c     &        phi(iphi-1),phi(iphi),
c     &        Pstart,u0,1,
c     &        pf,Nc,Na,phase,g,
c     &        ccoord,ka_gas,ks_gas,ka_cloud,ks_cloud,
c     &        ca,cb,cc,
c     &        Npositions,P_out,u_out,surface_out,path_length,tau_path,
c     &        x_path,valid)
c         write(*,*) 'valid=',valid
c         write(*,*) 'surface_out=',surface_out
c         write(*,*) 'P_out=',(P_out(i),i=1,dim)
c         write(*,*) 'u_out=',(u_out(i),i=1,dim)
c         stop

c         call intersection(1,dim,1,app,1,1,Pstart,u0,
c     &        r(ir-1,itheta,iphi),r(ir,itheta,iphi),
c     &        theta(itheta-1),theta(itheta),
c     &        phi(iphi-1),phi(iphi),ca,cb,cc,1,1,
c     &        solf,surface,Pint,aerr)
c         write(*,*) 'solf=',solf
c         write(*,*) 'aerr=',aerr
c         if (solf.eq.1) then
c            write(*,*) 'surface=',surface
c            write(*,*) 'Pint=',(Pint(i),i=1,dim)
c         endif
c        stop

c         write(*,*) 'now last_chance'
c         call last_chance(dim,Nr,Ntheta,Nphi,r,theta,phi,
c     &        rspace,r_nodes,r_south_pole,r_north_pole,
c     &        coeff_a,coeff_b,coeff_c,Pstart,u0,
c     &        ir,itheta,iphi,P_out,u_out,s_out,valid)
c         write(*,*) 'valid=',valid
c         if (valid.eq.1) then
c            write(*,*) 'P_out=',(P_out(i),i=1,dim)
c            write(*,*) 'u_out=',(u_out(i),i=1,dim)
c            write(*,*) 's_out=',s_out
c         endif
c         stop
c     Debug

c     Set computation load for each chunk
         call set_chunks(nchunks,np,Nevent,Nev)

c     sending data to all processes
c     Debug
         write(*,*) 'Sending data to all processes...'
c     Debug
         code=1
         do proc=1,np-1
            call MPI_SEND(code,1,MPI_INTEGER
     &           ,proc,1,MPI_COMM_WORLD,ierr)
         enddo

         call send_global_data_mpibcast(np,dim,seeds,Nevent,
     &        Nr,Ntheta,Nphi,Nb_gas,Nq,Nb_clouds,Nclouds,Na,Nzones,
     &        ref,vri,toa_ad,pf,Radius,Tsun,Rsun,dist,
     &        theta0,phi0,clouds_coord,
     &        lambda_gas_lo,lambda_gas_hi,w_quad,
     &        lambda_clouds_lo,lambda_clouds_hi,
     &        clouds_ka,clouds_ks,clouds_g,clouds_phase,
     &        r,theta,phi,rground,rspace,T,r_mc,
     &        emissivity_zone,band_min,band_max,spectral_pdf,
     &        spectral_cumul,r_nodes,r_south_pole,r_north_pole,
     &        coeff_a,coeff_b,coeff_c)
c     Debug
         write(*,*) '...done'
c     Debug
         
         do proc=1,np-1
            free(proc)=1
            nc_phost(proc)=0
         enddo
         end=0
         chunk=0
c     Debug
         write(*,*) 'Now computing...'
c     Debug

         do while (end.eq.0)
 444        continue
            call find_free_process(free,np-1,fpf,proc)
            if (fpf.eq.1) then  ! a free child process found
               chunk=chunk+1
               if ((chunk.le.nchunks).and.(Nev(chunk).gt.0)) then
                  ch_idx(proc)=chunk
                  code=2 ! means a set of 'Nev(chunk)' solar photons has to be simulated
                  call MPI_SEND(code,1,MPI_INTEGER
     &                 ,proc,1,MPI_COMM_WORLD,ierr)
                  call MPI_SEND(Nev(chunk),1,MPI_INTEGER
     &                 ,proc,1,MPI_COMM_WORLD,ierr)
                  free(proc)=0  ! child process index 'proc' is busy
                  nc_phost(proc)=nc_phost(proc)+1
c     Debug
c               write(*,*) 'now computing chunk=',chunk,' /',Nchunks
c     Debug
               else             ! chunk>nchunks or Nkrun(chunk)=0
                  free(proc)=2  ! means child process index 'proc' has been stopped
               endif
               goto 444
            else                ! no free child process found
               call stopped_processes(free,np-1,nsp)
               if (nsp.ge.np-1) then
                  end=1
                  goto 111
               else
c     wait for results
                  call MPI_RECV(proc,1,MPI_INTEGER
     &                 ,MPI_ANY_SOURCE,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                 stat,ierr)
                  call MPI_RECV(ret_code,1,MPI_INTEGER
     &                 ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,
     &                 stat,ierr)
                  if (ret_code.eq.0) then
c     process index 'proc' requires spectral data
c     Debug
c                     write(*,*) 'process ',proc,' is requiring data'
c     Debug
                     call MPI_RECV(band,1,MPI_INTEGER
     &                    ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
                     call MPI_RECV(quad,1,MPI_INTEGER
     &                    ,proc,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
                     call spectral_data(Nr,Ntheta,Nphi,
     &                    gas_ka,gas_ks,n_ref,
     &                    ground_emissivity,space_emissivity,
     &                    Nzones,emissivity,
     &                    band,quad,proc)
c     Debug
c                     write(*,*) 'data was sent to process ',proc
c     Debug
                  else if (ret_code.eq.1) then
c     process index 'proc' will now send computation results
                     call receive_psums(proc,toa_ad,
     &                    psum_flux_up,psum_flux_dn,
     &                    psum_flux_net,psum_flux_toa,psum_budget,
     &                    psum2_flux_up,psum2_flux_dn,
     &                    psum2_flux_net,psum2_flux_toa,psum2_budget,
     &                    ninvalid)
c     Debug
c            do itheta=1,Ntheta
c               do iphi=1,Nphi
c                  do u_itheta=1,Ntheta
c                     do u_iphi=1,Nphi
c                        if (psum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi).ne.0.0D+0) then
c                           write(*,*)
c                           write(*,*) itheta,iphi,u_itheta,u_iphi
c                           write(*,*) psum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi)
c                        endif
c                     enddo
c                  enddo
c               enddo
c            enddo
c     Debug
                     free(proc)=1 ! child process index 'proc' is free again
                     ninv(proc)=ninv(proc)+ninvalid
                     call update_sums(Nr,Ntheta,Nphi,Nb_gas,toa_ad,
     &                    psum_flux_up,
     &                    psum_flux_dn,
     &                    psum_flux_net,
     &                    psum_flux_toa,
     &                    psum_budget,
     &                    psum2_flux_up,
     &                    psum2_flux_dn,
     &                    psum2_flux_net,
     &                    psum2_flux_toa,
     &                    psum2_budget,
     &                    sum_flux_up,
     &                    sum_flux_dn,
     &                    sum_flux_net,
     &                    sum_flux_toa,
     &                    sum_budget,
     &                    sum2_flux_up,
     &                    sum2_flux_dn,
     &                    sum2_flux_net,
     &                    sum2_flux_toa,
     &                    sum2_budget)
c     Debug
c            do itheta=1,Ntheta
c               do iphi=1,Nphi
c                  do u_itheta=1,Ntheta
c                     do u_iphi=1,Nphi
c                        if (sum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi).ne.0.0D+0) then
c                           write(*,*)
c                           write(*,*) itheta,iphi,u_itheta,u_iphi
c                           write(*,*) sum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi)
c                        endif
c                     enddo
c                  enddo
c               enddo
c            enddo
c     Debug
                  else
                     write(*,*) 'Error from ',
     &                    label(1:strlen(label)),' :'
                     write(*,*) 'ret_code=',ret_code
                     stop
                  endif

               endif            ! nsp=np-1
            endif               ! fpf=0 or 1
 111        continue
         enddo                  ! while end=0

c     stopping all child processes
         do proc=1,np-1
            if (ninv(proc).ne.0) then
               write(*,*) 'process :',proc,
     &              ' ; aborted path:',ninv(proc)
            endif
            code=0
            call MPI_SEND(code,1,MPI_INTEGER,proc,1,MPI_COMM_WORLD,ierr)
         enddo
c     Debug
         write(*,*) 'Post processing...'
c     Debug
         call solar_postcalc(dim,label,Nevent,toa_ad,
     &        Nr,Ntheta,Nphi,Nb_gas,
     &        Radius,rspace,
     &        z,r,theta,phi,
     &        lambda_gas_lo,lambda_gas_hi,
     &        sum_flux_up,
     &        sum_flux_dn,
     &        sum_flux_net,
     &        sum_flux_toa,
     &        sum_budget,
     &        sum2_flux_up,
     &        sum2_flux_dn,
     &        sum2_flux_net,
     &        sum2_flux_toa,
     &        sum2_budget,
     &        solar_flux_up,
     &        solar_inc_flux_up, 
     &        solar_flux_dn,
     &        solar_inc_flux_dn,
     &        solar_flux_net,
     &        solar_inc_flux_net,
     &        solar_flux_toa,
     &        solar_inc_flux_toa,
     &        solar_budget,
     &        solar_inc_budget,
     &        rground,phi0)
c     Debug
         write(*,*) '...done'
c     Debug
         call save_logs(Nevent)
         call write_seed()
c     Debug
         
         write(*,*) 'solar_power=',pi*
     &        planck_int_wavlng(lambda_gas_lo(band_min),
     &        lambda_gas_hi(band_max),Tsun)*(Rsun/dist)**2.0D+0
     &        /4.0D+0

         fnet1=0.0D+0
         inc_fnet1=0.0D+0
         fnet2=0.0D+0
         inc_fnet2=0.0D+0
         do itheta=1,Ntheta
            do iphi=1,Nphi
               ir=Nr(itheta,iphi)
               fnet1=fnet1+solar_flux_dn(ir,itheta,iphi)*
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi))
               inc_fnet1=inc_fnet1+solar_inc_flux_dn(ir,itheta,iphi)*
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi))
               fnet2=fnet2+solar_flux_up(ir,itheta,iphi)*
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi))
               inc_fnet2=inc_fnet2+solar_inc_flux_up(ir,itheta,iphi)*
     &              surface_r(r(ir,itheta,iphi),
     &              theta(itheta-1),theta(itheta),
     &              phi(iphi-1),phi(iphi))
            enddo
         enddo
         surf1=4.0D+0*pi*r(Nr(1,1),1,1)**2.0D+0
         surf2=4.0D+0*pi*r(Nr(1,1),1,1)**2.0D+0
         fnet1=fnet1/surf1
         inc_fnet1=inc_fnet1/surf1
         fnet2=fnet2/surf2
         inc_fnet2=inc_fnet2/surf2
         write(*,*) 'f_dn(',ir,')=',fnet1,' +/-',inc_fnet1
         write(*,*) 'f_up(',ir,')=',fnet2,' +/-',inc_fnet2
c     Debug

         call finalize(np,nc_phost)
c         call clean()

c     Debug
         write(*,*) '...done'
c     Debug


c     MPICH
      else                      ! every other process
c     MPICH

 333     continue
c     wait for "codep" from root process
         call MPI_RECV(codep,1,MPI_INTEGER
     &        ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)

c     codep=0 means exit
         if (codep.eq.0) then
            goto 666
c     codep=1 means data reception for solar computation
         else if (codep.eq.1) then

            call receive_global_data_mpibcast(np,dim,seeds,Nevent,
     &           Nr,Ntheta,Nphi,Nb_gas,Nq,Nb_clouds,Nclouds,Na,Nzones,
     &           ref,vri,toa_ad,pf,Radius,Tsun,Rsun,dist,theta0,phi0,
     &           clouds_coord,lambda_gas_lo,lambda_gas_hi,w_quad,
     &           lambda_clouds_lo,lambda_clouds_hi,
     &           clouds_ka,clouds_ks,clouds_g,clouds_phase,
     &           r,theta,phi,rground,rspace,T,r_mc,
     &           emissivity_zone,band_min,band_max,spectral_pdf,
     &           spectral_cumul,r_nodes,r_south_pole,r_north_pole,
     &           coeff_a,coeff_b,coeff_c)
            call zufalli(seeds(pindex))
c     codep=2 means the process has to run a solar simulation
         else if (codep.eq.2) then
            call MPI_RECV(Nevp,1,MPI_INTEGER
     &           ,0,MPI_ANY_TAG,MPI_COMM_WORLD,stat,ierr)
c     computation
            call mc_solar(dim,ref,vri,toa_ad,
     &           pf,Nr,Ntheta,Nphi,
     &           Nb_gas,Nq,Nb_clouds,Na,
     &           r,theta,phi,rspace,T,r_mc,Nevp,Radius,Tsun,Rsun,dist,
     &           theta0,phi0,Nclouds,clouds_coord,
     &           lambda_gas_lo,lambda_gas_hi,w_quad,
     &           band_min,band_max,spectral_pdf,spectral_cumul,
     &           lambda_clouds_lo,lambda_clouds_hi,
     &           clouds_ka,clouds_ks,
     &           clouds_phase,clouds_g,
     &           Nzones,emissivity_zone,
     &           r_nodes,r_south_pole,r_north_pole,
     &           coeff_a,coeff_b,coeff_c,
     &           psum_flux_up,
     &           psum_flux_dn,
     &           psum_flux_net,
     &           psum_flux_toa,
     &           psum_budget,
     &           psum2_flux_up,
     &           psum2_flux_dn,
     &           psum2_flux_net,
     &           psum2_flux_toa,
     &           psum2_budget,
     &           ninvalid)
c     Debug
c            do itheta=1,Ntheta
c               do iphi=1,Nphi
c                  do u_itheta=1,Ntheta
c                     do u_iphi=1,Nphi
c                        if (psum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi).ne.0.0D+0) then
c                           write(*,*)
c                           write(*,*) itheta,iphi,u_itheta,u_iphi
c                           write(*,*) psum_flux_toa(itheta,iphi,
c     &                       u_itheta,u_iphi)
c                        endif
c                     enddo
c                  enddo
c               enddo
c            enddo
c     Debug
c     sendind results to the process 0
            call MPI_SEND(pindex,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
            call MPI_SEND(1,1,MPI_INTEGER,0,1,MPI_COMM_WORLD,ierr)
            call send_psums(0,toa_ad,psum_flux_up,psum_flux_dn,
     &           psum_flux_net,psum_flux_toa,psum_budget,
     &           psum2_flux_up,psum2_flux_dn,
     &           psum2_flux_net,psum2_flux_toa,psum2_budget,
     &           ninvalid)

         else ! unknown code
            write(*,*) 'Error from ',
     &           label(1:strlen(label)),' :'
            write(*,*) 'Unknown codep=',codep
            write(*,*) 'received by process:',pindex
            stop
         endif                  ! codep
c     wait for a new execution code
         goto 333
 666     continue
         
c     MPICH
      endif                     ! pindex
c     MPICH

c     MPICH
      call MPI_FINALIZE(ierr)
c     MPICH

      end
