c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine read_clouds(Nclouds,Nb_clouds,Na,
     &     lambda_clouds_lo,lambda_clouds_hi,clouds_coord,
     &     clouds_ka,clouds_ks,clouds_g,clouds_phase)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to read clouds coordinates and wavelength limits
c
      integer cloud,Nclouds,Nb_clouds,band,Na,angle
      double precision clouds_coord(1:Nclouds_mx,1:6)
      double precision lambda_clouds_lo(1:Nb_clouds_mx)
      double precision lambda_clouds_hi(1:Nb_clouds_mx)
      double precision clouds_ka(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_ks(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_phase(1:Nclouds_mx,1:Nb_clouds_mx,
     &     1:Nangle_mx,1:2)
      double precision clouds_g(1:Nclouds_mx,1:Nb_clouds_mx)
      integer strlen,ios,i
      character*100 clouds_file,label

      clouds_file='./data/clouds.in'
      label='subroutine read_clouds'

      open(10,file=clouds_file(1:strlen(clouds_file)),
     &     status='old',iostat=ios)
      if (ios.ne.0) then        ! file not found
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'File could not be found:'
         write(*,*) clouds_file(1:strlen(clouds_file))
         stop
      endif

      read(10,*) Nclouds,Nb_clouds,Na
      if (Nclouds.gt.Nclouds_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Nclouds=',Nclouds
         write(*,*) 'is greater than Nclouds_mx=',Nclouds_mx
         stop
      endif
      if (Nb_clouds.gt.Nb_clouds_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Nb_clouds=',Nb_clouds
         write(*,*) 'is greater than Nb_clouds_mx=',Nb_clouds_mx
         stop
      endif
      if (Nb_clouds.gt.Nangle_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'Na=',Na
         write(*,*) 'is greater than Nangle_mx=',Nangle_mx
         stop
      endif
      read(10,50) (lambda_clouds_lo(band),band=1,Nb_clouds)
      read(10,50) (lambda_clouds_hi(band),band=1,Nb_clouds)
      do cloud=1,Nclouds
         read(10,50) (clouds_coord(cloud,i),i=1,6)
      enddo ! cloud
      do cloud=1,Nclouds
         read(10,50) (clouds_ka(cloud,band),band=1,Nb_clouds)
      enddo ! cloud
      do cloud=1,Nclouds
         read(10,50) (clouds_ks(cloud,band),band=1,Nb_clouds)
      enddo ! cloud
      do cloud=1,Nclouds
         read(10,50) (clouds_g(cloud,band),band=1,Nb_clouds)
      enddo
      do cloud=1,Nclouds
         do angle=1,Na
            read(10,50) (clouds_phase(cloud,band,angle,1),
     &           band=1,Nb_clouds)
            read(10,50) (clouds_phase(cloud,band,angle,2),
     &           band=1,Nb_clouds)
         enddo ! angle
      enddo ! cloud
      close(10)

c     conversions
      do cloud=1,Nclouds
         clouds_coord(cloud,1)=clouds_coord(cloud,1)*1.0D+3 ! km -> m
         clouds_coord(cloud,2)=clouds_coord(cloud,2)*1.0D+3 ! km -> m
      enddo

      return
      end



      subroutine extract_spectral_cloud_data(Nclouds,Na,
     &     clouds_ka,clouds_ks,clouds_phase,clouds_g,band_in,
     &     Nb_gas,lambda_gas_lo,lambda_gas_hi,
     &     Nb_clouds,lambda_clouds_lo,lambda_clouds_hi,
     &     ka_clouds,ks_clouds,phase_clouds,g_clouds)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to extract ka_clouds and ks_clouds for every cloud,
c     for the specified spectral interval
c
c     Inputs:
c       + Nclouds: total number of clouds
c       + Na: number of points for phase function
c       + clouds_ka: absorption coefficient for every cloud
c       + clouds_ks: scattering coefficient for every cloud
c       + clouds_phase: phase function for every cloud
c       + clouds_g: asymetry parameter for every cloud
c       + band_in: index of the narrowband interval ka and ks have to be retrieved for
c         THIS IS a narrowband index for GAS DATA, not for clouds data !
c       + Nb_gas: number of narrowbands for gas data
c       + lambda_gas_lo: lower limits of narrowband spectral intervals for gas data
c       + lambda_gas_hi: higher limits of narrowband spectral intervals for gas data
c       + Nb_clouds: number of narrowbands for clouds data
c       + lambda_clouds_lo: lower limits of narrowband spectral intervals for clouds data
c       + lambda_clouds_hi: higher limits of narrowband spectral intervals for clouds data
c
c     Outputs:
c       + ka_clouds: absorption coefficient for every cloud, and for
c         the narrowband spectral interval specified
c       + ks_clouds: scattering coefficient for every cloud, and for
c         the narrowband spectral interval specified
c       + phase_clouds: phase functions for every cloud, and for
c         the narrowband spectral interval specified
c       + g_clouds: asymetry parameters for every cloud, and for
c         the narrowband spectral interval specified
c

c     inputs
      integer Nclouds,Na
      double precision clouds_ka(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_ks(1:Nclouds_mx,1:Nb_clouds_mx)
      double precision clouds_phase(1:Nclouds_mx,1:Nb_clouds_mx,
     &     1:Nangle_mx,1:2)
      double precision clouds_g(1:Nclouds_mx,1:Nb_clouds_mx)
      integer band_in
      integer Nb_gas
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
      integer Nb_clouds
      double precision lambda_clouds_lo(1:Nb_clouds_mx)
      double precision lambda_clouds_hi(1:Nb_clouds_mx)
c     outputs
      double precision ka_clouds(1:Nclouds_mx)
      double precision ks_clouds(1:Nclouds_mx)
      double precision phase_clouds(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g_clouds(1:Nclouds_mx)
c     temp
      integer band,cloud,angle
      integer cb1,cb2,cb1f,cb2f
      integer cc
      integer strlen,ios,i,j
      double precision dlambda
      character*100 label

      label='subroutine extract_spectral_cloud_data'

      if (band_in.gt.Nb_gas) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'input argument band_in=',band_in
         write(*,*) 'is greater than Nb_gas=',Nb_gas
         stop
      endif

c     Indentify CLOUDS narrowbands that are affected by GAS narrowband index 'band_in'
      cb1f=0
      do band=1,Nb_clouds
         if ((lambda_gas_lo(band_in).ge.lambda_clouds_lo(band)).and.
     &        (lambda_gas_lo(band_in).le.lambda_clouds_hi(band))) then
            cb1f=1
            cb1=band
            goto 111
         endif
      enddo
 111  continue
      if (cb1f.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'lower limit lambda_gas_lo(',band_in,')=',
     &        lambda_gas_lo(band_in)
         write(*,*) 'could not be found among clouds spectral limits'
         stop
      endif
      cb2f=0
      do band=cb1,Nb_clouds
         if ((lambda_gas_hi(band_in).ge.lambda_clouds_lo(band)).and.
     &        (lambda_gas_hi(band_in).le.lambda_clouds_hi(band))) then
            cb2f=1
            cb2=band
            goto 112
         endif
      enddo
 112  continue
      if (cb2f.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'higher limit lambda_gas_hi(',band_in,')=',
     &        lambda_gas_hi(band_in)
         write(*,*) 'could not be found among clouds spectral limits'
         stop
      endif

c     compute average cloud data
      do cloud=1,Nclouds
         if (cb1.eq.cb2) then
            band=cb1
            ka_clouds(cloud)=clouds_ka(cloud,band)
            ks_clouds(cloud)=clouds_ks(cloud,band)
            g_clouds(cloud)=clouds_g(cloud,band)
            do angle=1,Na
               do j=1,2
                  phase_clouds(cloud,angle,j)=
     &                 clouds_phase(cloud,band,angle,j)
               enddo ! j
            enddo ! angle
         else if (cb2.gt.cb1) then
c     initialization
            ka_clouds(cloud)=0.0D+0
            ks_clouds(cloud)=0.0D+0
            g_clouds(cloud)=0.0D+0
            do angle=1,Na
               phase_clouds(cloud,angle,1)=
     &              clouds_phase(cloud,1,angle,1) ! angles are all the same
               phase_clouds(cloud,angle,2)=0.0D+0
            enddo ! angle
c     sum of elements
            do band=cb1,cb2
               if (band.eq.cb1) then
                  dlambda=lambda_clouds_hi(band)-lambda_gas_lo(band_in)
               else if (band.eq.cb2) then
                  dlambda=lambda_gas_hi(band_in)-lambda_clouds_lo(band)
               else
                  dlambda=lambda_clouds_hi(band)-lambda_clouds_lo(band)
               endif
               ka_clouds(cloud)=ka_clouds(cloud)+
     &              clouds_ka(cloud,band)*dlambda
               ks_clouds(cloud)=ks_clouds(cloud)+
     &              clouds_ks(cloud,band)*dlambda
               g_clouds(cloud)=g_clouds(cloud)+
     &              clouds_g(cloud,band)*dlambda
               do angle=1,Na
                  phase_clouds(cloud,angle,2)=
     &                 phase_clouds(cloud,angle,2)+
     &                 clouds_phase(cloud,band,angle,2)*
     &                 dlambda
               enddo ! angle
            enddo ! band
            dlambda=lambda_gas_hi(band_in)-lambda_gas_lo(band_in)
            ka_clouds(cloud)=ka_clouds(cloud)/dlambda
            ks_clouds(cloud)=ks_clouds(cloud)/dlambda
            g_clouds(cloud)=g_clouds(cloud)/dlambda
            do angle=1,Na
               phase_clouds(cloud,angle,2)=
     &              phase_clouds(cloud,angle,2)/dlambda
            enddo
         else
            write(*,*) 'Error from ',label(1:strlen(label)),':'
            write(*,*) 'cb1=',cb1
            write(*,*) 'cb2=',cb2
            stop
         endif
      enddo ! cloud

      return
      end




      subroutine get_cloud_data(Nclouds,Na,Nc,cindex,
     &     ka_clouds,ks_clouds,phase_clouds,g_clouds,
     &     ka,ks,phase,g)
      implicit none
      include 'max.inc'
      include 'formats.inc'
c
c     Purpose: to extract ka and ks for identified clouds
c
c     Inputs:
c       + Nclouds: total number of clouds
c       + Na: number of points for phase function
c       + Nc: number of clouds ka and ks have to be retrieved for
c       + cindex: indexes of the clouds ka and ks have to be retrieved for
c       + ka_clouds: absorption coefficient for every cloud
c       + ks_clouds: scattering coefficient for every cloud
c       + phase_clouds: phase function for every cloud
c       + g_clouds: asymetry parameter for every cloud
c
c     Outputs:
c       + ka: absorption coefficient for every cloud required, and for
c         the narrowband spectral interval specified
c       + ks: scattering coefficient for every cloud required, and for
c         the narrowband spectral interval specified
c       + phase: phase functions for every cloud required, and for
c         the narrowband spectral interval specified
c       + g: asymetry parameters for every cloud required, and for
c         the narrowband spectral interval specified
c

c     inputs
      integer Nclouds,Na,Nc
      integer cindex(1:Nclouds_mx)
      double precision ka_clouds(1:Nclouds_mx)
      double precision ks_clouds(1:Nclouds_mx)
      double precision phase_clouds(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g_clouds(1:Nclouds_mx)
c     outputs
      double precision ka(1:Nclouds_mx)
      double precision ks(1:Nclouds_mx)
      double precision phase(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g(1:Nclouds_mx)
c     temp
      integer cloud,angle
      integer strlen,ios,i,j
      character*100 label

      label='subroutine get_cloud_data'

      if (Nc.gt.Nclouds) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'input argument Nc=',Nc
         write(*,*) 'is greater than Nclouds=',Nclouds
         stop
      endif

      do cloud=1,Nc
         if (cindex(cloud).gt.Nclouds) then
            write(*,*) 'Error from ',label(1:strlen(label)),':'
            write(*,*) 'input argument cindex(',cloud,')=',
     &           cindex(cloud)
            write(*,*) 'is greater than Nclouds=',Nclouds
            stop
         endif
      enddo

c     extract data for the "Nc" required clouds
      do cloud=1,Nc
         ka(cloud)=ka_clouds(cindex(cloud))
         ks(cloud)=ks_clouds(cindex(cloud))
         g(cloud)=g_clouds(cindex(cloud))
         do angle=1,Na
            do j=1,2
               phase(cloud,angle,j)=
     &              phase_clouds(cindex(cloud),angle,j)
            enddo               ! j
         enddo                  ! angle
      enddo                     ! cloud
      
      return
      end
