c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine refraction(dim,n1,n2,u1,n,u2)
      implicit none
      include 'max.inc'
      include 'param.inc'
c
c     Purpose: to modify a propagation direction according to the law of refraction
c     An incoming photon is going from a first cell to a second cell (is it crossing the interface)
c     The incoming direction is "u1" and "n" is the local normal to the interface at
c     the intersection point.
c
c     Inputs:
c       + dim: dimension of vectors
c       + n1: index of refraction in the cell the photon goes from
c       + n2: index of refraction in the cell the photon goes to
c       + u1: incident propagation direction at intersection position
c       + n: normal at intersection position, directed inside the second cell
c
c     Output:
c       + u2: propagation direction in the second cell (after refraction)
c

c     I/O
      integer dim
      double precision n1,n2
      double precision u1(1:Nmat_mx)
      double precision n(1:Nmat_mx)
c     output
      double precision u2(1:Nmat_mx)
c     temp
      integer strlen,i
      double precision alpha1,alpha2
      double precision alpha1_lim
      double precision epsilon_alpha
      double precision prod
      double precision c1,c2
      double precision k1,k2,k11,k12,k21,k22
      integer nsol
      double precision p1,p2
      double precision u21(1:Nmat_mx)
      double precision u22(1:Nmat_mx)
      character*100 label
      parameter(epsilon_alpha=1.0D-6) ! rad
      label='subroutine refraction'

      if (n1.eq.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'n1=',n1
         stop
      endif
      if (n2.eq.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'n2=',n2
         stop
      endif
      
      call scalar_product(dim,u1,n,prod)
c     the local normal and the incoming propagation direction should be in the same directions
      if (prod.lt.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'propagation direction u1=',(u1(i),i=1,dim)
         write(*,*) 'and local normal n=',(n(i),i=1,dim)
         write(*,*) 'are not in agreement'
         write(*,*) 'scalar product:',prod
         stop
      endif
      c1=prod
      alpha1=dacos(c1) ! > 0
      if (n1.gt.n2) then
         alpha1_lim=dasin(n2/n1)
         if (alpha1.ge.alpha1_lim) then
            alpha2=pi/2.0D+0-epsilon_alpha
         else
            call fresnel(n1,n2,alpha1,alpha2)
         endif
      else
         call fresnel(n1,n2,alpha1,alpha2)
      endif
      c2=dcos(alpha2)
      
      if (alpha1.eq.0.0D+0) then
         call copy_vector(dim,u1,u2)
      else
         k11=-dsin(alpha2)/dsin(alpha1)
         k12=-k11
         call calc_k2(k11,c1,c2,k21)
         call calc_k2(k12,c1,c2,k22)
         call calc_u2(dim,u1,n,k11,k21,u21)
         call calc_u2(dim,u1,n,k12,k22,u22)
         call scalar_product(dim,u21,u1,p1)
         call scalar_product(dim,u22,u1,p2)
         if (p1.gt.p2) then
            call copy_vector(dim,u21,u2)
         else
            call copy_vector(dim,u22,u2)
         endif
      endif
c     Debug
c      write(*,*) 'u2=',(u2(i),i=1,dim)
c     Debug

      return
      end


      subroutine fresnel(n1,n2,alpha1,alpha2)
      implicit none
      double precision n1,n2,alpha1,alpha2

      alpha2=dasin(n1/n2*dsin(alpha1)) ! Fresnel-Descartes' law

      return
      end


      subroutine calc_k2(k1,c1,c2,k2)
      implicit none
      double precision k1,c1,c2
      double precision k2

      k2=c2-k1*c1

      return
      end


      subroutine calc_u2(dim,u1,n,k1,k2,u2)
      implicit none
      include 'max.inc'
      integer dim
      double precision u1(1:Nmat_mx)
      double precision n(1:Nmat_mx)
      double precision k1,k2
      double precision u2(1:Nmat_mx)
      double precision un1(1:Nmat_mx)
      double precision nn(1:Nmat_mx)
      double precision t1(1:Nmat_mx)
      double precision t2(1:Nmat_mx)
      
      call normalize_vector(dim,n,nn)
      call normalize_vector(dim,u1,un1)
      call scalar_vector(dim,k1,un1,t1)
      call scalar_vector(dim,k2,nn,t2)
      call add_vectors(dim,t1,t2,u2)

      return
      end
