c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine path(db,dim,event,app,
     &     r1,r2,theta1,theta2,phi1,phi2,
     &     P_in,u_in,surface_in,
     &     pf,Nc,Na,phase,g,
     &     ccoord,ka_gas,ks_gas,ka_cloud,ks_cloud,
     &     ca,cb,cc,
     &     Npositions,P_out,u_out,surface_out,path_length,tau_path,
     &     x_path,valid)
      implicit none
      include 'max.inc'
      include 'mpif.h'
c
c     Purpose: to generate a scattering optical path,
c     from a position "P_in" and a direction "u_in"
c     on a surface, to the first exit point
c
c     Inputs:
c       + dim: dimension of arrays
c       + event: index of the event
c       + app: if equal to 1, ground surface must be computed as the real surface at ground level
c       + r1, theta1, phi1, r2, theta2, phi2: limits of the spherical box
c       + P_in: (r,theta,phi) spherical coordinates of the entry point
c       + u_in: entry direction
c       + surface_in: index of the surface where position "P_in" is located [1,6]
c       + pf: option for phase function choice
c       + Nc: number of clouds that intersect with the current box
c       + Na: number of angles the phase function has been discretized into
c       + phase: discretized phase function array for the 'Nc' clouds
c       + g: asymetry parameter for the 'Nc' clouds
c       + ccoord: limits of the Nc clouds that intersect the current box
c       + ka_gas,ks_gas: absorption and scattering coefficients (m) of the gas
c       + ka_cloud: absorption coefficient of the Nc clouds
c       + ks_cloud: scattering coefficient of the Nc clouds
c       + ca,cb,cc: a, b and c coefficients for the 4 quarters of the angular sector
c
c     Outputs:
c       + Npositions: number of positions that compose the path
c       + P_out: (r,theta,phi) spherical coordinates of the exit point
c       + u_out: output direction
c       + surface_out: index of the surface where position "P_out" is located [1,6]
c       + path_length: total length of the scattering path
c       + tau_path: absorption and scattering optical thickness along the path,
c                  at each scattering position
c         - tau_path(1,i): absorption optical depth at position i, i=1,Npositions
c         - tau_path(2,i): scattering optical depth at position i, i=1,Npositions
c       + x_path: list of the spherical coordinates of each position
c         - x_path(1,i): r at position index i (i=1,Npositions)
c         - x_path(2,i): theta at position index i (i=1,Npositions)
c         - x_path(3,i): phi at position index i (i=1,Npositions)
c       + valid: validity of the path
c         - valid=0: an error occured; path is invalid
c         - valid=1: path is valid
c

c     MPICH
      integer ierr,errorcode,np,pindex
      integer stat(MPI_STATUS_SIZE)
c     MPICH
c     inputs
      integer dim,event,app
      double precision r1,r2,theta1,theta2,phi1,phi2
      double precision P_in(1:Nmat_mx)
      double precision u_in(1:Nmat_mx)
      integer surface_in
      integer pf
      integer Nc,Na
      double precision phase(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g(1:Nclouds_mx)
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision ka_gas,ks_gas
      double precision ka_cloud(1:Nclouds_mx)
      double precision ks_cloud(1:Nclouds_mx)
      double precision ca(1:4)
      double precision cb(1:4)
      double precision cc(1:4)
c     outputs
      integer Npositions
      double precision P_out(1:Nmat_mx)
      double precision u_out(1:Nmat_mx)
      integer surface_out
      double precision path_length
      double precision tau_path(1:2,1:Ndiff_mx)
      double precision x_path(1:Nmat_mx,1:Ndiff_mx)
      integer valid
c     temp
      integer strlen
      integer surface_box,ok,inc
      double precision ka,ks
      double precision length
      double precision alpha
      integer i,in,type,cont
      double precision n_in(1:Nmat_mx)
      double precision n_out(1:Nmat_mx)
      double precision n1(1:Nmat_mx)
      double precision prod
      character*100 label
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision u1(1:Nmat_mx)
      integer intf
      double precision Pint(1:Nmat_mx)
      double precision P_tmp(1:Nmat_mx),pdf_tmp
      double precision P_adjusted(1:Nmat_mx)
      double precision u_tmp(1:Nmat_mx)
      double precision u_new(1:Nmat_mx)
      double precision taua,taus
      double precision epsilon_prod
      integer Nksp,Ncp
      double precision ksp(1:Nclouds_mx+1)
      double precision phasep(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision gp(1:Nclouds_mx)
      double precision r_ground
      double precision dz_max
c     ATTENTION !!!!!
c     produire les tableaux local_phase et local_g !!!!!!!
      double precision local_phase(1:Nangle_mx,1:2)
      double precision local_g
c     ATTENTION !!!!!
      integer ons,surf1,surf_int
      character*100 log_file
      character*100 string
      character*100 stringp_file
      integer nl
c     Debug
      integer j,db
      double precision pos(1:2)
      double precision g_radius
c     Debug
      parameter(epsilon_prod=1.0D-8)
      dz_max=0.0D+0

c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH

      label='subroutine path'
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :in'
      endif
      if ((surface_in.lt.1).or.(surface_in.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_in=',surface_in
         write(*,*) 'position 1'
         stop
      endif
c     Debug
      
      call test_box(r1,r2,theta1,theta2,phi1,phi2,label)
      call test_surface(surface_in,label)

      if (app.eq.1) then
         call reset_onground(P_in,theta1,theta2,phi1,phi2,ca,cb,cc)
      endif

      call is_in_cell(P_in,app,r1,r2,theta1,theta2,
     &     phi1,phi2,ca,cb,cc,ok)

      if (ok.eq.0) then

         call logfile_name(pindex,event,log_file)
         string='./nlines'
         call stringp_file_name(pindex,string,stringp_file)
         call get_nlines(log_file,stringp_file,nl)
         open(10,file=log_file(1:strlen(log_file)))
         do i=1,nl
            read(10,*)
         enddo
         if (nl.gt.0) then
            write(10,*)
         endif

         write(10,*) 'Error from ',label(1:strlen(label)),':'
         write(10,*) 'Position "P" is not within the box'
         write(10,*) 'P_in=',(P_in(i),i=1,dim)
         write(10,*) 'r1=',r1,' r2=',r2
         write(10,*) 'theta1=',theta1,' theta2=',theta2
         write(10,*) 'phi1=',phi1,' phi2=',phi2
         write(10,*) 'surface_in=',surface_in
         if (app.eq.1) then
            write(10,*) 'app=',app
            write(10,*) 'ground_radius=',r_ground
         endif

         close(10)
         valid=0
         goto 666
c         stop
      endif
      
c     coherence check
      if ((surface_in.eq.1).and.(app.eq.1)) then
         call ground_normal(dim,P_in,theta1,theta2,phi1,phi2,
     &        ca,cb,cc,n_in)
      else
         call inside_normal(surface_in,P_in,n_in)
      endif
      call scalar_product(dim,u_in,n_in,prod)
      if (prod.lt.0.0D+0) then

         call logfile_name(pindex,event,log_file)
         string='./nlines'
         call stringp_file_name(pindex,string,stringp_file)
         call get_nlines(log_file,stringp_file,nl)
         open(10,file=log_file(1:strlen(log_file)))
         do i=1,nl
            read(10,*)
         enddo
         if (nl.gt.0) then
            write(10,*)
         endif

         write(10,*) 'Error from ',label(1:strlen(label)),' :'
         write(10,*) 'Emission position:'
         write(10,*) 'P_in=',(P_in(i),i=1,dim)
         write(10,*) 'r1=',r1,' r2=',r2
         write(10,*) 'surface_in=',surface_in,' app=',app
         write(10,*) 'Inside normal at emission position:'
         write(10,*) 'n_in=',(n_in(i),i=1,dim)
         write(10,*) 'Emission direction:'
         write(10,*) 'u_in=',(u_in(i),i=1,dim)
         write(10,*) 'scalar product:',prod
         write(10,*) 'emission direction points outwards !'

         close(10)
         valid=0
         goto 666
c         stop
      endif
      
      call copy_vector(dim,u_in,u1)
      call copy_vector(dim,P_in,P1)
      cont=1
c     Debug
      if (db.eq.1) then
         write(*,*) 'P1=',(P1(j),j=1,dim)
      endif
c     Debug
c     set initial values
      Npositions=0
      path_length=0.0D+0
c     add the first position to the path
      call update_path(dim,x_path,Npositions,P1,0.0D+0,0.0D+0,
     &     path_length,tau_path)

      do while (cont.eq.1)
c     get absorption and scattering coefficients at starting position
         call coefficients(dim,Na,
     &        P1,u1,Nc,ccoord,
     &        ka_gas,ks_gas,ka_cloud,ks_cloud,phase,g,
     &        ka,ks,Nksp,ksp,Ncp,phasep,gp)
c     generate scattering length
         call sample_scattering_length(ks,length)
c     Debug
c         write(*,*) 'ks=',ks,' length=',length
c     Debug
c     compute position of next intersection
         call next_intersection(db,dim,event,app,
     &        r1,r2,theta1,theta2,phi1,phi2,
     &        Nc,ccoord,P1,u1,length,
     &        ca,cb,cc,
     &        Pint,type,surface_box)
c     Debug
      if ((type.ne.0).and.
     &        ((surface_box.lt.1).or.(surface_box.gt.6))) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_box=',surface_box
         write(*,*) 'after next_intersection'
         write(*,*) 'type=',type
         stop
      endif
c     Debug

         if (type.eq.0) then ! error code
c     In this case, the path could not be generated
c     Debug
c            write(*,*) 'warning from routine: ',label(1:strlen(label))
c            write(*,*) 'path wasted'
c            stop
c     Debug
            valid=0
            goto 666
         else if (type.eq.1) then ! box output position
c
c     modif - 21/03/2009
c     I suppress surface check, because it has already been tested in the case type=1
c     by routine "next_intersection"
c
c            if (app.eq.1) then
c               pos(1)=Pint(2)
c               pos(2)=Pint(3)
c               call ground_radius(pos,theta1,theta2,phi1,phi2,
c     &              ca,cb,cc,r_ground)
c            endif
c
c            if ((surface_box.eq.1).and.(app.eq.1)) then
c               call is_on_ground(Pint,theta1,theta2,
c     &              phi1,phi2,ca,cb,cc,ons)
c               surf_int=1
c            else if ((app.eq.1).and.
c     &              ((surface_box.eq.3).or.(surface_box.eq.4))) then
c               if ((Pint(2).ge.theta1).and.(Pint(2).le.theta2)
c     &              .and.(Pint(1).ge.r_ground)) then
c                  ons=1
c                  surf_int=surface_box
c               else
c                  ons=0
c               endif
c            else if ((app.eq.1).and.
c     &              ((surface_box.eq.5).or.(surface_box.eq.6))) then
c               if ((Pint(3).ge.phi1).and.(Pint(3).le.phi2)
c     &              .and.(Pint(1).ge.r_ground)) then
c                  ons=1
c                  surf_int=surface_box
c               else
c                  ons=0
c               endif
c            else
c               call is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
c     &              Pint,ons,surf_int)
c            endif
            ons=1
            surf_int=surface_box
c     Debug
            if ((surf_int.lt.1).or.(surf_int.gt.6)) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'surf_int=',surf_int
               write(*,*) 'type=',type
               write(*,*) 'surface_box=',surface_box
               stop
            endif
c     Debug
c     modif - 21/03/2009
            if (ons.eq.1) then
c     if output position belong to the surface of the cell
               if ((surface_box.eq.1).and.(app.eq.1)) then
                  call ground_normal(dim,Pint,theta1,theta2,
     &                 phi1,phi2,ca,cb,cc,n_out)
               else
                  call inside_normal(surf_int,Pint,n_out)
               endif
               call scalar_product(dim,u1,n_out,prod)
               do while (prod.gt.0.0D+0)
c     are there cases when this happens ?
c     then a new output direction has to be chosen
                  call surface_emission(dim,app,r1,r2,theta1,theta2,
     &                 phi1,phi2,ca,cb,cc,surf_int,P_tmp,u_new,pdf_tmp)
c     "u_new" is an input direction -> take the opposite direction
                  call scalar_vector(dim,-1.0D+0,u_new,u1)
                  call scalar_product(dim,u1,n_out,prod)
c     Debug
                  if (db.eq.1) then
                     write(*,*) 'new direction chosen, u1=',
     &                    (u1(i),i=1,dim)
                  endif
c     Debug
               enddo
            else
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),':'
               write(*,*) 'Path ending position:',(Pint(j),j=1,dim)
               write(*,*) 'does not belong to cell surface'
               write(*,*) 'r1=',r1
               write(*,*) 'r2=',r2
               write(*,*) 'theta1=',theta1
               write(*,*) 'theta2=',theta2
               write(*,*) 'phi1=',phi1
               write(*,*) 'phi2=',phi2
               write(*,*) 'intersection type:',type
               write(*,*) 'app=',app,' surface_box=',surface_box
               if (app.eq.1) then
                  pos(1)=Pint(2)
                  pos(2)=Pint(3)
                  call ground_radius(pos,theta1,theta2,phi1,phi2,
     &                 ca,cb,cc,g_radius)
                  write(*,*) 'ground_radius=',g_radius
                  write(*,*) 'ca=',(ca(j),j=1,4)
                  write(*,*) 'cb=',(cb(j),j=1,4)
                  write(*,*) 'cc=',(cc(j),j=1,4)
               endif
               stop
            endif
            call copy_vector(dim,u1,u_tmp)
            cont=0
         else if (type.eq.2) then ! scattering position
            call is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &        Pint,ons,surf_int)
c     Debug
            if ((ons.eq.1).and.
     &           ((surf_int.lt.1).or.(surf_int.gt.6))) then
               write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
               write(*,*) 'surf_int=',surf_int
               write(*,*) 'type=',type
               stop
            endif
            if (db.eq.1) then
               write(*,*) 'ons=',ons
            endif
c     Debug
            if (ons.eq.1) then
c     in the case the scattering position is on the surface of the box, or at least
c     very close from it, we consider the path ends here
c     first, the exit position has to be adjusted on the output surface
               call set_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
     &              Pint,surf_int,P_adjusted)
               call copy_vector(dim,P_adjusted,Pint)
               call inside_normal(surf_int,Pint,n_out)
               prod=1.0D+0
               do while (prod.gt.0.0D+0)
c     then an output direction has to be chosen
                  call surface_emission(dim,app,r1,r2,theta1,theta2,
     &                 phi1,phi2,ca,cb,cc,surf_int,P_tmp,u_new,pdf_tmp)
c     "u_new" is an input direction -> take the opposite direction
                  call scalar_vector(dim,-1.0D+0,u_new,u_tmp)
                  call scalar_product(dim,u_tmp,n_out,prod)
               enddo
               cont=0
            else
               call coefficients(dim,Na,
     &              Pint,u1,Nc,ccoord,
     &              ka_gas,ks_gas,ka_cloud,ks_cloud,phase,g,
     &              ka,ks,Nksp,ksp,Ncp,phasep,gp)
c     choose a scattering angle
               call scattering_angle(pf,Na,Ncp,phasep,gp,
     &              ks_gas,ks,Nksp,ksp,alpha)
c     compute the new propagation direction "u2"
               call new_direction(dim,u1,alpha,u_tmp)
c     verification
               call scalar_product(dim,u1,u_tmp,prod)
               if (dabs(prod-cos(alpha)).gt.epsilon_prod) then
                  write(*,*) 'Error from ',
     &                 label(1:strlen(label)),':'
                  write(*,*) 'alpha=',alpha,
     &                 ' cos(',alpha,')=',dcos(alpha)
                  write(*,*) 'prod=',prod
                  stop
               endif
c     verification
               cont=1
            endif
         else if (type.eq.3) then ! intersection with a cloud
c     copy input direction as new propagation direction
            call copy_vector(dim,u1,u_tmp)
            cont=1
         else
            write(*,*) 'Error from ',label(1:strlen(label)),':'
            write(*,*) 'type=',type
            stop
         endif ! type
c     Debug
         if (db.eq.1) then
            write(*,*) 'type=',type,' cont=',cont
            write(*,*) 'Pint=',(Pint(j),j=1,dim)
            write(*,*) 'u_tmp=',(u_tmp(j),j=1,dim)
         endif
c     Debug
c     compute absorption and scattering optical depths between P1 and Pint
         call optical_depths(dim,
     &        P1,Pint,Nc,Na,ccoord,
     &        ka_gas,ks_gas,ka_cloud,ks_cloud,phase,g,
     &        taua,taus)
         call update_path(dim,x_path,Npositions,Pint,taua,taus,
     &        path_length,tau_path)
c     take output position and direction as new starting position and propagation direction
         call copy_vector(dim,Pint,P1)
         call copy_vector(dim,u_tmp,u1)
      enddo                     ! while cont=1

c     Debug
      if ((surf_int.lt.1).or.(surf_int.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surf_int=',surf_int
         write(*,*) 'position 2'
         write(*,*) 'type=',type
         write(*,*) 'cont=',cont
         stop
      endif
c     Debug

c     if the path has been stopped (type=1), then P1=Pint is the box output position
      call copy_vector(dim,P1,P_out)
      call copy_vector(dim,u1,u_out)

      valid=1

c     quick coherence check
c     modif - 21/03/2009
c      if (app.eq.1) then
c         pos(1)=P_out(2)
c         pos(2)=P_out(3)
c         call ground_radius(pos,theta1,theta2,phi1,phi2,
c     &        ca,cb,cc,r_ground)
c      endif
c
c      if ((surface_box.eq.1).and.(app.eq.1)) then
c         call is_on_ground(P_out,theta1,theta2,
c     &        phi1,phi2,ca,cb,cc,ons)
c         surface_out=1
c      else if ((app.eq.1).and.
c     &        ((surface_box.eq.3).or.(surface_box.eq.4))) then
c         if ((P_out(2).ge.theta1).and.(P_out(2).le.theta2)
c     &        .and.(P_out(1).ge.r_ground)) then
c            ons=1
c            surface_out=surface_box
c         else
c            ons=0
c         endif
c      else if ((app.eq.1).and.
c     &        ((surface_box.eq.5).or.(surface_box.eq.6))) then
c         if ((P_out(3).ge.phi1).and.(P_out(3).le.phi2)
c     &        .and.(P_out(1).ge.r_ground)) then
c            ons=1
c            surface_out=surface_box
c         else
c            ons=0
c         endif
c      else
c         call is_on_surface(dim,r1,r2,theta1,theta2,phi1,phi2,
c     &        P_out,ons,surface_out)
c      endif
c      if (ons.eq.0) then
c         write(*,*) 'Error from ',label(1:strlen(label)),':'
c         write(*,*) 'last position is not on surface'
c         write(*,*) 'P_out=',(P_out(j),j=1,dim)
c         write(*,*) 'app=',app
c         if (app.eq.1) then
c            pos(1)=P_out(2)
c            pos(2)=P_out(3)
c            call ground_radius(pos,theta1,theta2,phi1,phi2,
c     &           ca,cb,cc,g_radius)
c            write(*,*) 'ground_radius=',g_radius
c            write(*,*) 'ca=',(ca(j),j=1,4)
c            write(*,*) 'cb=',(cb(j),j=1,4)
c            write(*,*) 'cc=',(cc(j),j=1,4)
c         endif
c         stop
c      endif
      surface_out=surf_int
c     Debug
      if ((surface_out.lt.1).or.(surface_out.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_out=',surface_out
         write(*,*) 'position 3'
         write(*,*) 'surf_int=',surf_int
         stop
      endif
c     Debug
c     modif - 21/03/2009
      if ((surface_out.eq.1).and.(app.eq.1)) then
         call ground_normal(dim,P_out,theta1,theta2,phi1,phi2,
     &        ca,cb,cc,n_out)
      else
         call inside_normal(surface_out,P_out,n_out)
      endif
      call scalar_product(dim,u_out,n_out,prod)
c     if output direction is directed to the inside of the cell
c     if ((prod.gt.0.0D+0).and.(type.eq.1)) then
      if (prod.gt.0.0D+0) then
         write(*,*) 'Error from ',label(1:strlen(label)),':'
         write(*,*) 'last position:',(P_out(j),j=1,dim)
         write(*,*) 'is on surface',surface_out
         write(*,*) 'r1=',r1
         write(*,*) 'r2=',r2
         write(*,*) 'theta1=',theta1
         write(*,*) 'theta2=',theta2
         write(*,*) 'phi1=',phi1
         write(*,*) 'phi2=',phi2
         write(*,*) 'intersection type:',type
         write(*,*) 'with exit direction:',(u_out(j),j=1,dim)
         write(*,*) 'normal direction at exit point:'
         write(*,*) (n_out(j),j=1,dim)
         if ((surface_box.eq.1).and.(app.eq.1)) then
            write(*,*) 'computed by ground_normal'
         else
            write(*,*) 'computed by inside_normal'
         endif
         write(*,*) 'scalar product:',prod
         write(*,*) 'output direction points inside !'
         stop
      endif

      if (app.eq.1) then
         call reset_onground(P_out,theta1,theta2,phi1,phi2,ca,cb,cc)
      endif
c     Debug
      if (db.eq.1) then
         write(*,*) label(1:strlen(label)),' :out'
      endif
c     Debug
c     Debug
      if ((surface_out.lt.1).or.(surface_out.gt.6)) then
         write(*,*) 'Error from ',
     &        label(1:strlen(label)),' :'
         write(*,*) 'surface_out=',surface_out
         write(*,*) 'position 4'
         write(*,*) 'surf_int=',surf_int
         stop
      endif
c     Debug

 666  continue
      return
      end



      subroutine optical_depths(dim,
     &     P1,P2,Nc,Na,ccoord,
     &     ka_gas,ks_gas,ka_cloud,ks_cloud,phase,g,
     &     taua,taus)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the absorption and scattering optical depths
c     between two positions "P1" and "P2"
c
c     Inputs:
c       + dim: dimension of vectors
c       + P1: (r,theta,phi) spherical coordinates of the first position
c       + P2: (r,theta,phi) spherical coordinates of the second position
c       + Nc: number of clouds that intersect with the current box
c       + Na: number of angles the phase function has been discretized into
c       + ccoord: limits of the Nc clouds that intersect the current box
c       + ka_gas,ks_gas: absorption and scattering coefficients (m) of the gas
c       + ka_cloud: array of the Nc clouds absorption coefficients (m)
c       + ks_cloud: array of the Nc clouds scattering coefficients (m)
c       + phase: phase function for the 'Nc' clouds
c       + g: asymetry parameter for the 'Nc' clouds
c
c     Outputs:
c       + taua,taus: absorption and scattering optical depths between
c                   positions "P1" and "P2"
c

c     input
      integer dim
      double precision P1(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      integer Nc,Na
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision ka_gas,ks_gas
      double precision ka_cloud(1:Nclouds_mx)
      double precision ks_cloud(1:Nclouds_mx)
      double precision phase(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g(1:Nclouds_mx)
c     output
      double precision taua,taus
c     temp
      double precision P1c(1:Nmat_mx)
      double precision P2c(1:Nmat_mx)
      double precision seg(1:Nmat_mx)
      double precision u1(1:Nmat_mx)
      double precision ka,ks
      double precision d
      integer Nksp
      double precision ksp(1:Nclouds_mx+1)
      integer Ncp
      double precision phasep(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision gp(1:Nclouds_mx)
      character*100 label
      label='subroutine optical_depths'

      call spher2cart(label,1,P1(1),P1(2),P1(3),
     &     P1c(1),P1c(2),P1c(3))
      call spher2cart(label,1,P2(1),P2(2),P2(3),
     &     P2c(1),P2c(2),P2c(3))
      call substract_vectors(dim,P2c,P1c,seg)
      call normalize_vector(dim,seg,u1)

      call coefficients(dim,Na,
     &     P1,u1,Nc,ccoord,
     &     ka_gas,ks_gas,ka_cloud,ks_cloud,phase,g,
     &     ka,ks,Nksp,ksp,Ncp,phasep,gp)
      call distance_spher(label,P1(1),P1(2),P1(3),
     &     P2(1),P2(2),P2(3),d)
      taua=ka*d  ! m*m=unitless
      taus=ks*d  ! m*m=unitless

      
      return
      end


      subroutine coefficients(dim,Na,
     &     P,u,Nc,ccoord,
     &     ka_gas,ks_gas,ka_cloud,ks_cloud,phase,g,
     &     ka,ks,Nksp,ksp,Ncp,phasep,gp)
      implicit none
      include 'max.inc'
c     
c     Purpose: to compute the total absorption and scattering coefficients
c     at a given position
c
c     Inputs:
c       + dim: dimension of vectors
c       + Na: number of angles the phase function is discretized into
c       + P: (r,theta,phi) spherical coordinates of the position
c       + u: propagation direction
c       + Nc: number of clouds that intersect with the current box
c       + ccoord: coordinates of the 'Nc' clouds
c       + ka_gas,ks_gas: absorption and scattering coefficients (m) of the gas
c       + ka_cloud: array of the Nc clouds absorption coefficients (m)
c       + ks_cloud: array of the Nc clouds scattering coefficients (m)
c       + phase: phase function of the 'Nc' clouds
c       + g: asymetry parameter of the 'Nc' clouds
c
c     Outputs:
c       + ka,ks: total absorption and scattering coefficients
c       + Nksp: number of different scattering coefficients at position "P" 
c         (gas + Ncp clouds that intersect at the current position)
c       + ksp: array of Nksp scattering coefficients
c       + Ncp: number of clouds at current position (Nksp-1)
c       + phasep: phase function of the 'Ncp' clouds
c       + gp: asymetry parameter of the 'Ncp' clouds
c

c     input
      integer dim,Na
      double precision P(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      integer Nc
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision ka_gas,ks_gas
      double precision ka_cloud(1:Nclouds_mx)
      double precision ks_cloud(1:Nclouds_mx)
      double precision phase(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision g(1:Nclouds_mx)
c     output
      double precision ka,ks
      integer Nksp,Ncp
      double precision ksp(1:Nclouds_mx+1)
      double precision phasep(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision gp(1:Nclouds_mx)
c     temp
      integer cloud,cio
      integer Ncpos
      integer cindex(1:Nclouds_mx)
      integer ons,surface
      double precision n(1:Nmat_mx)
      double precision prod
      integer j,angle
c     Debug
      integer k
c     Debug

      call nclouds_at_position(Nc,ccoord,P,Ncpos,cindex)
      ka=ka_gas
      ks=ks_gas
      Nksp=1
      Ncp=0
      ksp(Nksp)=ks_gas
      do cloud=1,Ncp
         call is_on_surface(dim,
     &        ccoord(cindex(cloud),1),ccoord(cindex(cloud),2),
     &        ccoord(cindex(cloud),3),ccoord(cindex(cloud),4),
     &        ccoord(cindex(cloud),5),ccoord(cindex(cloud),6),
     &        P,ons,surface)
         if (ons.eq.1) then
            call inside_normal(surface,P,n)
            call scalar_product(dim,u,n,prod)
c     Debug
c            if (ons.eq.1) then
c               write(*,*) 'cloud surface:'
c               write(*,*) 'r1=',ccoord(cloud,1),
c     &              ' r2=',ccoord(cloud,2)
c               write(*,*) 'theta1=',ccoord(cloud,1),
c     &              ' theta2=',ccoord(cloud,2)
c               write(*,*) 'phi1=',ccoord(cloud,1),
c     &              ' phi2=',ccoord(cloud,2)
c               write(*,*) 'P:',(P(k),k=1,dim)
c               write(*,*) 'u:',(u(k),k=1,dim)
c               write(*,*) 'n:',(n(k),k=1,dim)
c               write(*,*) 'prod=',prod
c               stop
c            endif
c     Debug
            if (prod.gt.0.0D+0) then
               cio=1
            else
               cio=0
            endif
         endif
         if (ons.eq.0) then
            ka=ka+ka_cloud(cindex(cloud)) ! m
            ks=ks+ks_cloud(cindex(cloud)) ! m
            Ncp=Ncp+1
            Nksp=Nksp+1
            ksp(Nksp)=ks_cloud(cindex(cloud)) ! m
            do angle=1,Na
               do j=1,2
                  phasep(Ncp,angle,j)=phase(cindex(cloud),angle,j)
               enddo ! j
            enddo ! angle
            gp(Ncp)=g(cindex(cloud))
         endif
      enddo

      return
      end



      subroutine identical_clouds(Ncp1,Ncp2,cindex1,cindex2,id)
      implicit none
      include 'max.inc'
c     
c     Purpose: to check whether or not clouds at positions 1 and 2
c     are the same.
c
c     Inputs:
c       + Ncp1: number of clouds at first position
c       + Ncp2: number of clouds at second position
c       + cindex1: indexes of clouds at first position
c       + cindex2: indexes of clouds at second position
c
c     Output:
c       + id:
c         - id=0: clouds are NOT identical at both positions
c         - id=1: clouds are identical at both positions
c

c     input
      integer Ncp1,Ncp2
      integer cindex1(1:Nclouds_mx)
      integer cindex2(1:Nclouds_mx)
c     output
      integer id
c     temp
      integer cloud

      id=1 ! default
      if (Ncp1.ne.Ncp2) then
         id=0
         goto 666
      endif

      do cloud=1,Ncp1
         if (cindex1(cloud).ne.cindex2(cloud)) then
            id=0
            goto 666
         endif
      enddo

 666  continue
      return
      end



      subroutine nclouds_at_position(Nc,ccoord,position,Ncp,cindex)
      implicit none
      include 'max.inc'
c
c     Purpose: to identify clouds at a given position
c
c     Inputs:
c       + Nc: number of clouds that intersect with the box "position" is into
c       + ccoord: limits of these Nc clouds
c       + position: (r,theta,phi) coordinates of the position
c
c     Outputs:
c       + Ncp: number of clouds at position
c       + cindex: indexes of these Ncp clouds
c

c     input
      integer Nc
      double precision ccoord(1:Nclouds_mx,1:6)
      double precision position(1:Nmat_mx)
c     output
      integer Ncp
      integer cindex(1:Nclouds_mx)
c     temp
      integer cloud,in

      Ncp=0
      do cloud=1,Nc
         call is_in_box(position,
     &        ccoord(cloud,1),ccoord(cloud,2),
     &        ccoord(cloud,3),ccoord(cloud,4),
     &        ccoord(cloud,5),ccoord(cloud,6),in)
         if (in.eq.1) then
            Ncp=Ncp+1
            cindex(Ncp)=cloud
         endif
      enddo
      
      return
      end

      


      subroutine update_path(dim,x_path,Npositions,position,
     &     taua,taus,path_length,tau_path)
      implicit none
      include 'max.inc'
c
c     Purpose: to add a position to the current scattering path, and
c     update the number of positions it is made of.
c
c     Inputs:
c       + dim : dimension of vectors
c       + x_path: array of (r,theta,phi) spherical coordinates of the scattering positions
c       + Npositions: number of positions the scattering path is made of
c       + position: (r,theta,phi) spherical coordinates of the position that has to be added to the path
c       + taua,taus: absorption and scattering optical depth since last position
c       + path_length: length of the path
c       + tau_path: array of absorption and scattering optical depths
c         - tau_path(1,i): absorption optical depth at position i, i=1,Npositions
c         - tau_path(2,i): scattering optical depth at position i, i=1,Npositions
c
c     Outputs:
c       + x_path: updated path (input path + new position)
c       + Npositions: updated number of positions (input Npositions+1)
c       + path_length: updated path length (input length + distance between last 2 positions)
c       + tau_path: array of absorption and scattering optical depths
c

c     I/O
      integer dim
      double precision x_path(1:Nmat_mx,1:Ndiff_mx)
      integer Npositions
      double precision position(1:Nmat_mx)
      double precision taua,taus
      double precision path_length
      double precision tau_path(1:2,1:Ndiff_mx)
c     temp
      integer i,j
      double precision position1(1:Nmat_mx)
      double precision position2(1:Nmat_mx)
      double precision length
      character*100 label
      label='subroutine update_path'

c     update the number of positions "x_path" is made of
      Npositions=Npositions+1
      if (Npositions.ge.Ndiff_mx) then
         write(*,*) 'Error from update_path:'
         write(*,*) 'Ndiff_mx has been reached'
         stop
      endif
c     add "position" to the "x_path"
      do i=1,dim
         x_path(i,Npositions)=position(i)
      enddo
c     compute distance between the last two positions
      if (Npositions.gt.1) then
         tau_path(1,Npositions)=tau_path(1,Npositions-1)+taua
         tau_path(2,Npositions)=tau_path(2,Npositions-1)+taus
         do i=1,dim
            position1(i)=x_path(i,Npositions-1)
         enddo
         do i=1,dim
            position2(i)=x_path(i,Npositions)
         enddo
         call distance_spher(label,
     &        position1(1),position1(2),position1(3),
     &        position2(1),position2(2),position2(3),length)
      else
         tau_path(1,Npositions)=taua
         tau_path(2,Npositions)=taus
         length=0.0D+0
      endif
c     update the total length of the path
      path_length=path_length+length

      return
      end




      subroutine new_direction(dim,u,alpha,v)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to compute vector "v", based on initial direction "u" and scattering angle "alpha".
c
c     Inputs:
c       + dim: dimension of vectors
c       + u: initial direction
c       + alpha: scattering angle [0,pi]
c
c     Outputs:
c       + v: final direction
c     

c     inputs
      integer dim
      double precision u(1:Nmat_mx)
      double precision alpha
c     outputs
      double precision v(1:Nmat_mx)
c     temp
      integer strlen
      double precision alpha1,alpha2
      double precision raf,theta,phi
      double precision j1(1:Nmat_mx)
      double precision t(1:Nmat_mx)
      double precision v1,v2
      character*100 label
      label='subroutine new_direction'

c     Tests
      if ((alpha.lt.0.0D+0).or.(alpha.gt.pi)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'alpha=',alpha
         write(*,*) 'is not between 0 and pi'
         stop
      endif
c     Tests
c     get "theta" and "phi", the angles that define direction "u"
      call cart2spher(u(1),u(2),u(3),raf,theta,phi)
c     FIRST ROTATION of vector "u", by an angle "alpha1", around axe "j1" -> vector "t"
      j1(1)=-dsin(phi)
      j1(2)=dcos(phi)
      j1(3)=0.0D+0
      alpha1=alpha
      call rotation(dim,u,alpha1,j1,t)
c     SECOND ROTATION of vector "t", by an angle "alpha2" (chosen uniformally between
c     0 and 2pi), around vector "u"
      v1=0.0D+0
      v2=2.0D+0*pi
      call uniform(v1,v2,alpha2)
      call rotation(dim,t,alpha2,u,v)

      return
      end


      subroutine scattering_angle(pf_in,Na,Ncp,phasep,gp,
     &     ks_gas,ks_total,Nksp,ksp,alpha)
      implicit none
      include 'param.inc'
      include 'max.inc'
c
c     Purpose: to return a scattering angle alpha [0,pi]
c
c     Input:
c       + pf_in: option for choosing the phase function that must be used
c         - pf_in=1: use the user-defined phase function
c         - pf_in=2: use the Henyey-Greenstein phase function
c       + Na: number of angles the phase function has been discretized into
c       + Ncp: number of clouds that intersect at the current position
c       + phasep: array that contains descrete values of the user-defined phase-function
c         - phase(i,1): value of the ith cos(angle) (first value must be -1.0D+0, last value must be 1.0D+0)
c         - phase(i,2): value of the ith user-defined phase function for phase(i,1)
c       + gp: Henyey-Greenstein phase-function asymetry factor
c       + ks_gas: scattering coefficient (m^-1) at point
c       + ks_total: total scattering coefficient (m^-1) at point
c       + Nksp: number of scattering coefficients at the current position (Ncp+1)
c       + ksp: array of scattering coefficients at the current position
c     
c     Output:
c       + alpha: scattering angle [0,pi]
c

c     input
      integer pf_in
      integer Na,Ncp
      double precision phasep(1:Nclouds_mx,1:Nangle_mx,1:2)
      double precision gp(1:Nclouds_mx)
      double precision ks_gas,ks_total
      integer Nksp
      double precision ksp(1:Nclouds_mx+1)
c     output
      double precision alpha
c     temp
      integer strlen,pf
      double precision R
      double precision phase(1:Nangle_mx,1:2)
      double precision g
      integer i,iks,iksf,interval,intervalf,cloud,angle,j
      double precision cumul(1:Nangle_mx)
      double precision u,S,mu
      double precision sum,sum_ks
      double precision elem(1:Ncumul_mx)
      double precision cumul_ks(0:Ncumul_mx)
      character*100 label
      label='subroutine scattering_angle'
      pf=pf_in

      if (ks_gas.gt.ks_total) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Gas scattering coefficient:',ks_gas
         write(*,*) 'Total scattering coefficient:',ks_total
         stop
      endif
      sum_ks=0.0D+0
      do i=1,Nksp
         sum_ks=sum_ks+ksp(i)
      enddo
      if (sum_ks.gt.ks_total) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Sum of input scattering coefficients:',sum_ks
         write(*,*) 'Total scattering coefficient:',ks_total
         write(*,*) 'Nksp=',Nksp
         do i=1,Nksp
            write(*,*) 'ksp(',i,')=',ksp(i)
         enddo
         stop
      endif
c     cumulative of scattering coefficients
      do i=1,Nksp
         elem(i)=ksp(i)
      enddo
c      call create_cumulative(Nksp,ksp,cumul_ks)
      call create_cumulative(Nksp,elem,cumul_ks)

      if ((pf.lt.1).or.(pf.gt.2)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Bad value for input argument pf:'
         write(*,*) 'pf=',pf
         stop
      endif

      call invert_cumulative(Nksp,cumul_ks,iks)

      if (iks.eq.1) then ! scattering by the gas
         pf=0
      else ! scattering by cloud index (iks-1)
         cloud=iks-1
         do angle=1,Na
            do j=1,2
               phase(angle,j)=phasep(cloud,angle,j)
            enddo ! j
         enddo ! angle
         g=gp(cloud)
      endif

      call random_gen(R)

      if (pf.eq.0) then         ! use the Rayleigh phase-function
         u=4.0D+0*R-2.0D+0
         S=(u+dsqrt(1.0D+0+u**2))**(1.0D+0/3.0D+0)
         mu=S-1.0D+0/S
      else if (pf.eq.1) then    ! use the user-defined phase function contained in "phase"
c     Tests
         if (Na.le.0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Bad value for input argument Na:'
            write(*,*) 'Na=',Na
            stop
         endif
         if (phase(1,1).ne.-1.0D+0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Bad value for input argument phase:'
            write(*,*) 'phase(1,1)=',phase(1,1)
            write(*,*) 'should be 1.0D+0'
            stop
         endif
         if (phase(Na,1).ne.1.0D+0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Bad value for input argument phase:'
            write(*,*) 'phase(',Na,',1)=',phase(Na,1)
            write(*,*) 'should be 0.0D+0'
            stop
         endif
         if (Na.ge.2) then
            do i=2,Na
               if (phase(i,1).lt.phase(i-1,1)) then
                  write(*,*) 'Error from '
     &                 ,label(1:strlen(label)),' :'
                  write(*,*) 'phase(',i-1,',1)=',phase(i-1,1)
                  write(*,*) ' > phase(',i,',1)=',phase(i,1)
                  stop
               endif
            enddo
         else
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Na=',Na,' is too low'
            stop
         endif
         do i=1,Na
            if (phase(i,2).lt.0.0D+0) then
                  write(*,*) 'Error from ',
     &              label(1:strlen(label)),' :'
                  write(*,*) 'phase(',i,',2)=',phase(i,2)
                  write(*,*) 'is negative'
                  stop
            endif
         enddo
c     Tests

c     computation of the cumulative of the user-defined phase function
         cumul(1)=0.0D+0
         do i=2,Na
            cumul(i)=cumul(i-1)
     &           +(phase(i,2)+phase(i-1,2))
     &           *(phase(i,1)-phase(i-1,1))/2.0D+0
         enddo
c     normalization
         sum=cumul(Na)
         if (sum.ne.1.0D+0) then
            do i=1,Na
               cumul(i)=cumul(i)/sum
            enddo
         endif
c     inversion of the user-defined phase function
         intervalf=0
         do i=2,Na
            if ((R.gt.cumul(i-1)).and.(R.lt.cumul(i))) then
               interval=i
               intervalf=1
               goto 123
            endif
         enddo
 123     continue
c     Tests
         if (intervalf.eq.0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'No angular interval could be found for R=',R
            write(*,*) 'and phase function cumulative:'
            do i=1,Na
               write(*,*) 'mu(',i,')=',phase(i,1),
     &              ' cumul(',i,')=',cumul(i)
            enddo
            stop
         endif
c     Tests
         mu=phase(interval-1,1)
     &        +(phase(interval,1)-phase(interval-1,1))
     &        /(cumul(interval)-cumul(interval-1))
     &        *(R-cumul(interval-1))
      else if (pf.eq.2) then    ! use the Henyey-Greenstein phase-function
         if ((g.lt.-1.0D+0).or.(g.gt.1.0D+0)) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Bad value for input argument g=',g
            stop
         endif
         mu=2.D+0*R*(1.D+0+g)**2*(g*(R-1.D+0)+1.D+0)/
     &        (1.D+0-g*(1.D+0-2.D+0*R))**2-1.D+0
      endif

      alpha=dacos(mu)

      return
      end



      subroutine sample_scattering_length(ks,length)
      implicit none
c
c     Purpose: to return a scattering length [0,+infty]
c
      double precision ks,length
      double precision taus

      call sample_scattering_opt_depth(taus)
      length=taus/ks

      return
      end



      subroutine sample_scattering_opt_depth(taus)
      implicit none
c
c     Purpose: to return a scattering optical depth [0,+infty]
c
      double precision taus
      double precision R

      call random_gen(R)
      taus=-dlog(R)

      return
      end


      subroutine jump_to(dim,P1,u,length,P2)
      implicit none
      include  'max.inc'
c     
c     Purpose: to compute the coordinates of position "P2" that
c     results from the propagation in a straight line from position "P1"
c     along direction "u"
c
c     Inputs:
c       + dim: dimension of the vectors
c       + P1: (r,theta,phi) spherical coordinates of the starting point
c       + u: coordinates of the propagation direction
c       + length: length of the flight
c
c     Outputs:
c     + P2: (r,theta,phi) spherical coordinates of the ending point
c
      integer dim
      double precision length
      double precision P1(1:Nmat_mx)
      double precision u(1:Nmat_mx)
      double precision P2(1:Nmat_mx)
      double precision nu(1:Nmat_mx)
      double precision pos1(1:Nmat_mx)
      double precision flight(1:Nmat_mx)
      double precision pos2(1:Nmat_mx)
      character*100 label
      label='subroutine jump_to'

c     normalize direction
      call normalize_vector(dim,u,nu)
c     get cartesian coordinates of the starting point
      call spher2cart(label,1,P1(1),P1(2),P1(3),pos1(1),pos1(2),pos1(3))
c     get the flight vector
      call scalar_vector(dim,length,u,flight)
c     add flight vector to first position
      call add_vectors(dim,pos1,flight,pos2)
c     get spherical coordinates of second poistion
      call cart2spher(pos2(1),pos2(2),pos2(3),P2(1),P2(2),P2(3))

      return
      end

