c     Copyright (C) 2008-2009 Vincent Eymet
c
c     Planet_EMC is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 3, or (at your option)
c     any later version.

c     Planet_EMC is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with Planet_EMC; if not, see <http://www.gnu.org/licenses/>
c
      subroutine spectral_data(Nr,Ntheta,Nphi,
     &     ka_gas,ks_gas,n_ref,
     &     ground_emissivity,space_emissivity,
     &     Nzones,emissivity,
     &     band,quad,proc)
      implicit none
      include 'max.inc'
      include 'mpif.h'
c
c     Purpose: to retrieve spectral data for a single k, and send it
c     to the proper process
c
c     Inputs:
c       + Nr,Ntheta,Nphi: number of cells
c       + ka_gas: gas absorption coefficients for the whole configuration
c       + ks_gas: gas scattering coefficients for the whole configuration
c       + n_ref: array of refraction indexes
c       + ground_emissivity: emissivities at ground level
c       + space_emissivity: emissivities at space level
c       + Nzones: number of special zones
c       + emissivity: emissivities of special zones
c       + band: index of the narrowband spectral interval
c       + quad: quadrature index
c       + proc: process index
c

c     inputs
      integer Nr(1:Ntheta_mx,1:Nphi_mx)
      integer Ntheta,Nphi
      double precision ka_gas(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision ks_gas(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx,1:Nq_mx)
      double precision n_ref(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx,1:Nb_gas_mx)
      double precision ground_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      double precision space_emissivity(1:Ntheta_mx,1:Nphi_mx,
     &     1:Nb_gas_mx)
      integer Nzones
      double precision emissivity(1:Nzones_mx,1:Nb_gas_mx)
      integer band,quad,proc
c     MPICH
      integer ierr
c     MPICH
c     temp
      integer np,pindex,ios
      integer ir,itheta,iphi,izone
      double precision ka_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ks_gas_1q(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision n_ref_1b(1:Nz_mx,1:Ntheta_mx,1:Nphi_mx)
      double precision ground_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision space_emissivity_1b(1:Ntheta_mx,1:Nphi_mx)
      double precision emissivity_1b(1:Nzones_mx)
      integer strlen
      character*100 datafile
      character*100 label
      label='subroutine spectral_data'
      
c     MPICH
c     Get 'np', the number of processes
      call MPI_COMM_SIZE(MPI_COMM_WORLD,np,ierr)
c     Get 'pindex', the index of the current process
      call MPI_COMM_RANK(MPI_COMM_WORLD,pindex,ierr)
c     MPICH

c     security checks
      if (pindex.ne.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'pindex=',pindex
         stop
      endif
      if ((proc.lt.1).or.(proc.gt.np-1)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Wrong input parameter proc=',proc
         stop
      endif

c     actually doing the job
      do itheta=1,Ntheta
         do iphi=1,Nphi
            do ir=1,Nr(itheta,iphi)
               ka_gas_1q(ir,itheta,iphi)=
     &              ka_gas(ir,itheta,iphi,band,quad)
               ks_gas_1q(ir,itheta,iphi)=
     &              ks_gas(ir,itheta,iphi,band,quad)
               n_ref_1b(ir,itheta,iphi)=
     &              n_ref(ir,itheta,iphi,band)
            enddo               ! ir
            ground_emissivity_1b(itheta,iphi)=
     &           ground_emissivity(itheta,iphi,band)
            space_emissivity_1b(itheta,iphi)=
     &           space_emissivity(itheta,iphi,band)
         enddo                  ! iphi
      enddo                     ! itheta

      do izone=1,Nzones
         emissivity_1b(izone)=
     &        emissivity(izone,band)
      enddo                     ! izone
      
      call send_spectral_data(ka_gas_1q,ks_gas_1q,n_ref_1b,
     &     ground_emissivity_1b,space_emissivity_1b,
     &     emissivity_1B,proc)

      return
      end




      subroutine narrowband_choice(Nb_gas,Nq,band_min,band_max,
     &     spectral_pdf,spectral_cumul,
     &     band,quad,spectral_weight)
      implicit none
      include 'max.inc'
c
c     Purpose: to choose a narrowband spectral interval and a quadrature indexn,
c     and return the ponderation weight associated to this choice
c
c     Inputs:
c       + Nb_gas: number of narrowband spectral intervals for gas data
c       + Nq: quadrature order
c       + band_min: index of the first narrowband interval that must be taken into account
c       + band_max:  index of the last narrowband interval that must be taken into account
c       + spectral_pdf: discrete pdf function associated to spectral narrowbands
c       + spectral_cumul: discrete cumulative function associated to spectral_pdf
c
c     Outputs:
c       + band: index of the narrowband spectral interval that was chosen
c       + quad: index of quadrature that was chosen
c       + spectral_weight: ponderation weight associated to this choice
c

c     inputs
      integer Nb_gas,Nq,band_min,band_max
      double precision spectral_pdf(1:Nb_gas_mx,1:Nq_mx)
      double precision spectral_cumul(0:Ncumul_mx)
c     outputs
      integer band,quad
      double precision spectral_weight
c     temp
      integer i
      integer strlen
      character*100 label
      label='subroutine narrowband_choice'

      call invert_cumulative(Nb_gas*Nq,spectral_cumul,i)
      band=(i-1)/Nq+1
      quad=i-(band-1)*Nq
c     coherence test
      if ((band.lt.band_min).or.(band.gt.band_max)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'narrowband interval chosen:',band
         write(*,*) 'while band_min=',band_min,
     &        ' and band_max=',band_max
         stop
      endif
      if ((quad.lt.1).or.(quad.gt.Nq)) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'quadrature index chosen:',quad
         write(*,*) 'while Nq=',Nq
         stop
      endif
      spectral_weight=1.0D+0/spectral_pdf(band,quad)

      return
      end




      subroutine spectral_range(nT,T,eps_st4,Nb_gas,Nq,w_quad,
     &     lambda_gas_lo,lambda_gas_hi,band_min,band_max,
     &     spectral_pdf,spectral_cumul)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the limits of the wavelength interval radiative transfer
c     has to be computed for, at a given temperature level
c
c     Inputs:
c       + nT: number of temperatures in array "T"
c       + T: temperature array [K]
c       + eps_st4: percentage of total energy (sigma*T^4) that will be
c                  neglected on both sides (before lambda_min and after lambda_max)
c       + Nb_gas: number of narrowband spectral intervals for gas data
c       + Nq: quadrature order
c       + w_quad: quadrature weights
c       + lambda_gas_lo: lower limts of the Nb_gas intervals
c       + lambda_gas_hi: higher limts of the Nb_gas intervals
c
c     Outputs:
c       + band_min: index of the first narrowband interval that must be taken into account
c       + band_max:  index of the last narrowband interval that must be taken into account
c       + spectral_pdf: discrete probability density function of each narrowband interval
c       + spectral_cumul: discrete cumulative function of "spectral_pdf"
c

c     inputs
      integer nT
      double precision T(1:Nz_mx),eps_st4
      integer Nb_gas,Nq
      double precision w_quad(1:Nq_mx)
      double precision lambda_gas_lo(1:Nb_gas_mx)
      double precision lambda_gas_hi(1:Nb_gas_mx)
c     outputs
      integer band_min,band_max
      double precision spectral_pdf(1:Nb_gas_mx,1:Nq_mx)
      double precision spectral_cumul(0:Ncumul_mx)
c     temp
      integer i,band_minf,band_maxf,band,quad
      double precision Tmin,Tmax
      double precision lambda_min,lambda_max
      double precision F_wavlng,planck_int_wavlng
      double precision lambda0,F0
      double precision lambda8,F8
      parameter(lambda0=1.0D-5) ! m
      parameter(lambda8=1.0D+5) ! m
      double precision elem(1:Ncumul_mx)
      integer strlen
      character*100 label
c     Debug
c      double precision sum
c     Debug
      label='subroutine spectral_range'
      
c     tests
      Tmin=T(1)
      Tmax=0.0D+0
      do i=1,nT
         if (T(i).lt.0.0D+0) then
            write(*,*) 'Error from ',label(1:strlen(label)),' :'
            write(*,*) 'Incorrect input parameter:'
            write(*,*) 'T(',i,')=',T(i)
            stop
         endif
         if (T(i).lt.Tmin) then
            Tmin=T(i)
         endif
         if (T(i).gt.Tmax) then
            Tmax=T(i)
         endif
      enddo ! i

      F0=F_wavlng(lambda0,Tmax)
      if (eps_st4.lt.F0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'eps_st4=',eps_st4
         write(*,*) 'F0=',F0
         stop
      endif
      F8=F_wavlng(lambda8,Tmin)
      if (1.0D+0-eps_st4.gt.F8) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) '1-eps_st4=',1.0D+0-eps_st4
         write(*,*) 'F8=',F8
         stop
      endif

c     looking for lambda_min and lambda_max, the limits of the computation
c     spectral interval, in terms of wavelengths
      call dichotomy_planck(lambda0,lambda8,F0,F8,eps_st4,
     &     1.0D-6,Tmax,lambda_min)
      call dichotomy_planck(lambda0,lambda8,F0,F8,1.0D+0-eps_st4,
     &     1.0D-6,Tmin,lambda_max)

c     looking for the fisrt and last narrowband intervals in the [lambda_min,lambda_max]
      band_minf=0
      if (lambda_gas_lo(1).ge.lambda_min) then
         band_minf=1
         band_min=1
      endif
      if (band_minf.eq.0) then
         do band=1,Nb_gas
            if ((lambda_gas_lo(band).le.lambda_min).and.
     &           (lambda_gas_hi(band).gt.lambda_min)) then
               band_minf=1
               band_min=band
               goto 123
            endif
         enddo                  ! band
      endif
 123  continue
      if (band_minf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'band_min was not found'
         do band=1,Nb_gas
            write(*,*) band,'[',lambda_gas_lo(band),'-',
     &           lambda_gas_hi(band),']'
         enddo
         write(*,*) 'lambda_min=',lambda_min
         stop
      endif

      band_maxf=0
      if (lambda_gas_hi(Nb_gas).le.lambda_max) then
         band_maxf=1
         band_max=Nb_gas
      endif
      if (band_maxf.eq.0) then
         do band=Nb_gas,1,-1
            if ((lambda_gas_lo(band).lt.lambda_max).and.
     &           (lambda_gas_hi(band).ge.lambda_max)) then
               band_maxf=1
               band_max=band
               goto 124
            endif
         enddo                  ! band
      endif
 124  continue
      if (band_maxf.eq.0) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'band_max was not found'
         do band=1,Nb_gas
            write(*,*) band,'[',lambda_gas_lo(band),'-',
     &           lambda_gas_hi(band),']'
         enddo
         write(*,*) 'lambda_max=',lambda_max
         stop
      endif

      if (band_max.lt.band_min) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'band_min=',band_min
         write(*,*) 'band_max=',band_max
         stop
      endif
      
c     Debug
c      write(*,*) 'band_min=',band_min
c      write(*,*) 'band_max=',band_max
c     Debug

      do band=1,Nb_gas
         if ((band.lt.band_min).or.(band.gt.band_max)) then
            do quad=1,Nq
               spectral_pdf(band,Nq)=0.0D+0
            enddo ! quad
         else
            do quad=1,Nq
               spectral_pdf(band,quad)=
     &              planck_int_wavlng(lambda_gas_lo(band),
     &              lambda_gas_hi(band),Tmax) ! W/m/sr
     &              /planck_int_wavlng(lambda_gas_lo(band_min),
     &              lambda_gas_hi(band_max),Tmax) ! / W/m/sr
     &              *w_quad(quad) ! * unitless = [unitless]
            enddo ! quad
         endif
      enddo ! band

c     Debug
c      sum=0.0D+0
c      do band=1,Nb_gas
c         write(*,*) 'spectral_pdf(',band,')=',spectral_pdf(band)
c         sum=sum+spectral_pdf(band)
c      enddo
c      write(*,*) 'sum=',sum
c     Debug

      if (Nb_gas*Nq.gt.Ncumul_mx) then
         write(*,*) 'Error from ',label(1:strlen(label)),' :'
         write(*,*) 'Nb_gas=',Nb_gas
         write(*,*) 'while Ncumul_mx=',Ncumul_mx
         stop
      endif
      i=0
      do band=1,Nb_gas
         do quad=1,Nq
            i=i+1
            elem(i)=spectral_pdf(band,quad)
         enddo ! quad
      enddo ! band
      call create_cumulative(Nb_gas*Nq,elem,spectral_cumul)

c     creation of the log file
      open(10,file='./logs/spectral_optimization.log')
      write(10,*) 'Spectral optimizations log file'
      write(10,*) 
      write(10,*) 'Percentage of energy neglected on both sides:'
      write(10,*) eps_st4
      write(10,*) 'Maximum temperature [K]:'
      write(10,*) Tmax
      write(10,*) 'Minimum wavelength [m]:'
      write(10,*) lambda_min
      write(10,*) 'Minimum temperature [K]:'
      write(10,*) Tmin
      write(10,*) 'Maximum wavelength [m]:'
      write(10,*) lambda_max
      write(10,*)
      write(10,*) 'band / quad / spectral pdf / spectral cumulative'
      i=0
      do band=1,Nb_gas
         do quad=1,Nq
            i=i+1
            write(10,*) band,quad,spectral_pdf(band,quad),
     &           spectral_cumul(i)
         enddo ! quad
      enddo ! band
      close(10)

      return
      end
